/* 
 * File:   Battery_utility.h
 * Author: don
 *
 * Created on June 16, 2014, 1:55 PM
 */

#ifndef BATTERY_UTILITY_H
#define	BATTERY_UTILITY_H
#include "gfx/gfx.h"

#define GAS_GAUGE_STATUS                      0x00
#define GAS_GAUGE_CONTROL                     0x01
#define GAS_GAUGE_ACCUMULATE_CHARGE_MSB       0x02
#define GAS_GAUGE_ACCUMULATE_CHARGE_LSB       0x03
#define GAS_GAUGE_CHARGE_THRESHOLD_HIGH_MSB   0x04
#define GAS_GAUGE_CHARGE_THRESHOLD_HIGH_LSB   0x05
#define GAS_GAUGE_CHARGE_THRESHOLD_LOW_MSB    0x06
#define GAS_GAUGE_CHARGE_THRESHOLD_LOW_LSB    0x07

#define BATTERY_GOOD                          0b00000000
#define BATTERY_ACCUMULATOR_OVER_UNDERFLOW    0b00100000
#define BATTERY_CHARGE_ALERT_HIGH             0b00001000
#define BATTERY_CHARGE_ALERT_LOW              0b00000100
#define BATTERY_LOW_BATTERY                   0b00000010
#define BATTERY_UNDERVOLTAGE_LOCKOUT          0b00000001

#define VBAT_ALERT3_0                         0b11000000
#define VBAT_ALERT2_9                         0b10000000
#define VBAT_ALERT2_8                         0b01000000
#define VBAT_ALERT_OFF                        0b00000000
#define PRESCALE_128                          0b00111000
#define PRESCALE_64                           0b00110000
#define PRESCALE_32                           0b00101000
#define PRESCALE_16                           0b00100000
#define PRESCALE_8                            0b00011000
#define PRESCALE_4                            0b00010000
#define PRESCALE_2                            0b00001000
#define PRESCALE_1                            0b00000000
#define CONFIGURE_PIN_NOT_ALLOWED             0b00000110
#define CONFIGURE_PIN_ALERT                   0b00000100
#define CONFIGURE_PIN_CHARGE_COMPLETE         0b00000010
#define CONFIGURE_PIN_DISABLED                0b00000000
#define POWER_DOWN_SAVE_POWER                 0b00000001
#define POWER_DOWN_OPERATE                    0b00000000
       
#define BATTERY_GAS_GAUGE_ADDRESS             0b01100100

#define CHARGE_ALERT_HIGH                     0b00001000
#define CHARGE_ALERT_LOW                      0b00000100
#define VBAT_ALERT                            0b00000010
#define UNDERVOLTAGE_LOCKOUT_ALERT            0b00000001


void resetChargeAccumulator(void);
void configure_gasGauge_threshold(void);
unsigned int read_gasGauge(void);
unsigned int read_C_D_register(void);
void ShowScreenBatteryDiagnostic(void);
bool ProcessMessage_BATTERY_test(WORD, GFX_GOL_OBJ_HEADER*, GFX_GOL_MESSAGE*);
void Battery_Diagnostic(void);
BYTE Battery_status(void);
BOOL clearGasGaugeAlert(void);
void monitorBatteryLevel(void);
bool setControlRegister(void);
void PowerDownSequence(void);
void ShowScreenUserPowerDown(void);
void userPowerDown(void);
void external_power_applied(void);
char chargerStatus_blocking(void);
void ShowScreenBatteryAlert(void);
void pollExternalPower(void);
void powerDown(void);
void gauge_power(BYTE power);

//#define PRESCALER_M PRESCALE_2  // quicker battery test
#define PRESCALER_M PRESCALE_64
#define ON  0b00000000
#define OFF 0b00000001

typedef enum {
    EXTERNAL_PWR_APPLIED = 0,
    DEBOUNCE_PWR,
    READ_STATUS,
    SLEEP,
    READ_CHARGE_STATUS,
    POWER_OFF
} CHARGING_STATES;


typedef enum {
    DEFECTIVE_BATTERY = 0,
    FAIL_TEMPERATURE,
    PARTIAL_CHARGE,
    FULL_CHARGE
} CHARGE_STATUS;



extern BYTE old_status;
extern BOOL radioInitialized;

#endif	/* BATTERY_UTILITY_H */