
/********************************************************************
 ********************************************************************
 * FileName:     IR_Temperature_hal.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This example demonstrates reading from Ir thermo sensor MLX90614ESF-BAA
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/23/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/
/******************************************************************************
 * Includes
 *******************************************************************************/
#include "IR_Temperature_hal.h"
#include <i2c.h>
#include "system.h"
#include "string.h"


/******************************************************************************
 * Module Preprocessor Constants
 *******************************************************************************/


/******************************************************************************
 * Module Preprocessor Macros
 *******************************************************************************/
#define WRITE 0
#define READ  1

/******************************************************************************
 * Module Typedefs
 *******************************************************************************/

/******************************************************************************
 * Module Variable Definitions
 *******************************************************************************/
static uint8_t i2c_address;


/******************************************************************************
 * Function Prototypes
 *******************************************************************************/


/******************************************************************************
 * Function Definitions
 *******************************************************************************/

int8_t IR_i2c_init( uint8_t address_id ) {

    i2c_address = ( address_id << 1 );
    CloseI2C2();			   		 //Disable I2C2 module if enabled previously
    //OpenI2C2(I2C_ON | I2C_7BIT_ADD, GAUGE_I2C1BRG);
    OpenI2C2(0x8000, GAUGE_I2C1BRG);

    return 0;
}

void IR_i2c_hal_write( uint8_t address, uint16_t DataSz, uint8_t *buff ) {
    uint8_t buffer[MAX_READ_SIZE];
    uint8_t Index = 0;
    uint8_t sendDataSz;

    buffer[0] = ( i2c_address | WRITE );
    buffer[1] = address;
    memcpy( &buffer[2], buff, DataSz );
    sendDataSz = DataSz + 2;

    StartI2C2(); //Send the Start Bit
    IdleI2C2();  //Wait to complete

    while (sendDataSz) {
        MasterWriteI2C2(buffer[Index++]);
        IdleI2C2(); //Wait to complete
        sendDataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C2STATbits.ACKSTAT)
            break;
    }
    StopI2C2(); //Send the Stop condition
    IdleI2C2(); //Wait to complete
}

void IR_i2c_hal_read(  uint8_t register_address, uint16_t num, uint8_t *buff ) {
    char i2cData[5];
    char DataSz;
    int Index = 0;
    uint8_t i;
    
    i2cData[0] = i2c_address | WRITE; //Device Address &WR
    i2cData[1] = register_address;
    DataSz = 2;

    StartI2C2();    // Send the Start Bit
    IdleI2C2();     // Wait to complete
    while (DataSz) {
        MasterWriteI2C2(i2cData[Index++]);
        IdleI2C2(); //Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C2STATbits.ACKSTAT)
            break;
    }

    //now send a start sequence again
    RestartI2C2();  // Send the Restart condition
    IdleI2C2();     // Wait to complete
    MasterWriteI2C2(i2c_address | READ); //transmit read command
    IdleI2C2();     // Wait to complete
    for (i = 0; i < num; i++) {
        *(buff++) = MasterReadI2C2(); // read byte
        IdleI2C2();  // Wait to complete
        AckI2C2();
        IdleI2C2();  // Wait to complete
    }
    StopI2C2();      // Send the Stop condition
    IdleI2C2();      // Wait to complete
}



/*************** END OF FUNCTIONS ************/
