/********************************************************************
 ********************************************************************
 * FileName:     IR_Temperature_hal.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This example demonstrates reading from Ir thermo sensor MLX90614ESF-BAA
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/23/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/
/******************************************************************************
 * Includes
 *******************************************************************************/
#include "IR_Temperature_hal.h"
#include <i2c.h>
#include "system.h"
#include "string.h"


/******************************************************************************
 * Module Preprocessor Constants
 *******************************************************************************/

/******************************************************************************
 * Module Preprocessor Macros
 *******************************************************************************/

/******************************************************************************
 * Module Typedefs
 *******************************************************************************/

/******************************************************************************
 * Module Variable Definitions
 *******************************************************************************/

/******************************************************************************
 * Function Prototypes
 *******************************************************************************/

/******************************************************************************
 * Function Definitions
 *******************************************************************************/

/******************************************************************************
 * System Initialize
 *******************************************************************************/
void System_Init(void) {
    IR_i2c_init( MLX90614_I2C_ADR );
}


uint8_t IR_getTemperature(uint8_t temperature_register, int16_t *quotient, uint8_t *remainder) {
    uint8_t result = 1;
    uint8_t temperatureBuffer[2];
    int16_t temp;
    int32_t temperature;

    IR_i2c_hal_read(temperature_register, 2, temperatureBuffer);
    if ((ERROR_BIT & temperatureBuffer[1]) == NO_ERROR) {
        result = 0;
        temp    = temperatureBuffer[1];
        temp = temp << 8;
        temperature = temp + temperatureBuffer[0];
        temperature    = temperature * 2 - ABSOLUTE_ZERO;
        *quotient = temperature / 100;
        *remainder = temperature % 100;
    }
    return result;
}
