/********************************************************************
 ********************************************************************
 * FileName:     IR_temperature_hal.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  Hardware Application Layer
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/10/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#ifndef IR_HAL_H
#define	IR_HAL_H

#include <xc.h>
#include <stdint.h>

#define MAX_READ_SIZE    64
#define MLX90614_I2C_ADR 0x5A
#define _AMB_TEMP        0x06   //The linearized die temperature is available in RAM cell 0x06
#define _OBJ_TEMP        0x07
#define ERROR_BIT        0x80
#define NO_ERROR         0x00
#define ABSOLUTE_ZERO    27315

#ifdef	__cplusplus
extern "C" {
#endif


/******************************************************************************
* Preprocessor Constants
*******************************************************************************/

/**
 * @brief Initializes i2c module
 * @param address_id - slave device address
 */
int8_t IR_i2c_init( uint8_t address_id );

/**
 * @brief Writes data through the i2c line
 * @param address - memory adress of the slave device
 * @param num     - number of bytes to send
 * @param buff    - buffer containing the bytes
 */
void IR_i2c_hal_write( uint8_t register_address, uint16_t num, uint8_t *buff );

/**
 * @brief Reads bytes from i2c bus, stores to a desired buffer
 * @param address - desired memory address on the slave device
 * @param num     - number of bytes to be read
 * @param buff    - buffer to where the read bytes will be stored
 */
void IR_i2c_hal_read(  uint8_t register_address, uint16_t num, uint8_t *buff );
    



#ifdef	__cplusplus
}
#endif

#endif	/* IR_HAL_H */

