/********************************************************************
 ********************************************************************
 * FileName:     IrThermo_main.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This example demonstrates reading from Ir thermo sensor MLX90614ESF-BAA
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/23/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <stdint.h>
#include "IR_hw.h"
#include "IR_temperature_hal.h"
#include "system_config.h"
#include "timer.h"
#include <string.h>
#include "stdio.h"
#include"../Utilities.h"
#include "gfx/gfx.h"


void CLICK_demo(void);


// Globals
extern APP_SCREEN_UPDATE_STATES update_display;
APP_SCREEN_IR_DEMO_STATES CLICK_DEMO_STATE;
extern GFX_GOL_OBJ_SCHEME pScheme_button;
static DWORD demoTick = 0;

/****************************************************************************
  Function:
  void Display_IR_temperature(void)

  Description:
  This function displays formated data to the display

  Precondition:
 Graphic objects must be created by calling ShowScreenIR_demo()

  Parameters:
    None

  Returns:
    None

  Remarks:

 ***************************************************************************/
void Display_IR_temperature(void) {
    uint8_t remainder;
    int16_t quotient;

    if (IR_getTemperature(_AMB_TEMP, &quotient, &remainder) == 0) {
        sprintf((char *) &(pageInformation.instructions2), (char*) "Ambient: %4d.%2d C", quotient, remainder);
    } else {
        sprintf((char *) &(pageInformation.instructions2), (char*) "  ERROR   ");
    }
    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    if (IR_getTemperature(_OBJ_TEMP, &quotient, &remainder) == 0) {
        sprintf((char *) &(pageInformation.instructions1), (char*) "Object: %4d.%2d C", quotient, remainder);
    } else {
        sprintf((char *) &(pageInformation.instructions1), (char*) "  ERROR   ");
    }

    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
    update_display = GLOBAL_UPDATE; //
}

/****************************************************************************
  Function:
   void CLICK_demo( void )

  Description:
    This function initializes the hardware.  Shows temperature from the IR sensor
 * every 10 seconds.

  Precondition:
 Graphic objects must be created by calling ShowScreenIR_demo()

  Parameters:
    None

  Returns:
    None

  Remarks:
    The ProcessMessage_DEMO_test() function will exit the test and return to 
 * the main display
 ***************************************************************************/
void CLICK_demo(void) {


    if (tick > demoTick) {
        demoTick = tick  + 1000;
        switch (CLICK_DEMO_STATE) {
            case INITIALIZE:
                power_mode(HIGH_POWER);
                System_Init();
                MKROE_power_on();
                CLICK_DEMO_STATE = SPLASH_SCREEN;
                break;
            case SPLASH_SCREEN:

                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions1, " IR Demo ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions, "MLX90614");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                CLICK_DEMO_STATE = DISPLAY_TEMPERATURE;
                break;
            case DISPLAY_TEMPERATURE:
                power_mode(HIGH_POWER);
                Display_IR_temperature();
                break;
            case EXIT_DEMO:
                screenState = SCREEN_DISPLAY_MAIN;
                power_mode(HIGH_POWER);
                MKROE_power_off();
                strcpy(pageInformation.instructions1, "Power Off");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
        }
    }
}

/****************************************************************************
  Function:
    void ShowScreenIR_demo( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenClick_demo(void) {
    strcpy(pageInformation.title, "IR DEMO ");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects

    strcpy(pageInformation.instructions2, "IR Demo ");
    strcpy(pageInformation.instructions1, "MLX90614");
    strcpy(pageInformation.instructions, "");

    strcpy(pageInformation.instructions3, "");
    strcpy(pageInformation.buttonTextBottomLeft, " OK");

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 61, 127, 61 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 81, 74, 95, 2,                // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,       // will be displayed after creation
            NULL,                             // bitmap
            NULL,                             // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //
}

/****************************************************************************
  Function:
    bool ProcessMessage_DEMO_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                        GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for Click demo test selection screen. 

  Precondition:
    Call ShowScreenIR_demo() prior to using this function to display the
    correct screen and generate the required graphic objects.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    global change variable to used  clean up later.
 ***************************************************************************/
bool ProcessMessage_DEMO_test(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD controlID;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                CLICK_DEMO_STATE = EXIT_DEMO;
                return false; // Hidden button
            } else {
                return false;
            }
            break;
        default:
            return false;
    }

    return true; // Call the default handler to show the button state.
}
