
/********************************************************************
 ********************************************************************
 * FileName:     Thunderclick_main.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  * Description:
     This is a sample program which demonstrates Simple demonstration 
     of using AS3935 Lighting sensor chip:
 *
 *   Example will detect three sources of events: Lighting, disturbance and Noise.
 *   Each event will be displayed on the EPD and in case of lighting, approximate energy
 *   and distance from thunderstorm will be calculated.  Also the number of strikes
 *   are accumulated
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/23/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <stdint.h>
#include "rtcc.h"
#include "system_config.h"
#include "timer.h"
#include <string.h>
#include "stdio.h"
#include"../Utilities.h"
#include "gfx/gfx.h"
#include "drv_spi_THUNDER.h"


extern APP_SCREEN_UPDATE_STATES update_display;
APP_SCREEN_IR_DEMO_STATES CLICK_DEMO_STATE;
extern GFX_GOL_OBJ_SCHEME pScheme_button;
static DWORD demoTick = 0;

/****************************************************************************
  Function:
    void ShowScreenClick_demo( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenClick_demo(void) {
    strcpy(pageInformation.title, "  THUNDER  ");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects

    strcpy(pageInformation.instructions2, "THUNDER Demo ");
    strcpy(pageInformation.instructions1, "ae AS3935");
    strcpy(pageInformation.instructions, "");

    strcpy(pageInformation.instructions3, "");
    strcpy(pageInformation.buttonTextBottomLeft, " OK");

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 61, 127, 61 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 81, 74, 95, 2,                // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,       // will be displayed after creation
            NULL,                             // bitmap
            NULL,                             // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //
}

/****************************************************************************
  Function:
    void Display_Lightning_information( void )

  Description:
    This function will update the display if an event is detected.

  Precondition:
 module must be initialized

  Parameters:
    None

  Returns:
    None

  Remarks:
    
 ***************************************************************************/
void Display_Lightning_information() {
    uint8_t interrupt_source = 0;
    uint32_t energy;
    uint32_t divisor;
    uint8_t  distance_km = 63;  //47,40,34,31,27,24,20,17,14,12,10,8,6,1 possible values
    static uint16_t Total_Lightings_Detected = 0;
    uint8_t i;
    uint8_t decimal[10];


    delay_ms(10);         // Whenever an interrupt is issued, the external unit 
    //                    // should wait 2ms before reading the Interrupt register.
    interrupt_source = DRV_Thunder_Read(0x03) & 0x0F;
    //interrupt_source = LIGHTNING_DETECTED;  //for testing output screen
    switch (interrupt_source) {
        case LIGHTNING_DETECTED:
//          the energy is calculated. The result of the energy calculation is 
//          then stored in the registers REG0x06[4:0], REG0x05[7:0] and 
//          REG0x04[7:0]. This value is just a pure number and has no 
//          physical meaning.
            
//            energy = DRV_Thunder_Read_Energy();
//            divisor = 1000000000;
//            for (i = 0; i < 9; i++) {
//             decimal[i] = energy / divisor;
//             energy %= divisor;
//             divisor /= 10;
//              }
//            decimal[9] = energy;
//            
//            sprintf((char *) &(pageInformation.instructions1), (char*) "Energy: %1d%1d%1d%1d%1d%1d%1d%1d%1d%1d",
//            decimal[0], decimal[1], decimal[2], decimal[3], decimal[4], decimal[5], decimal[6], decimal[7], decimal[8], decimal[9]);
//            time stamp
            RTCCProcessEvents();
            strcpy(pageInformation.instructions1, _time_str);
            
            
            distance_km = DRV_Thunder_Read_distance();
            sprintf((char *) &(pageInformation.instructions2), (char*) "Distance: %2d Km", distance_km);
            Total_Lightings_Detected ++;
            sprintf((char *) &(pageInformation.instructions), (char*) "Total:  %2d strikes", Total_Lightings_Detected );
            break;
        case DISTURBER_DETECTED:
            sprintf((char *) &(pageInformation.instructions2), (char*) "Disturber");
            sprintf((char *) &(pageInformation.instructions1), (char*) "detected");
            break;
        case NOISE_LEVEL_TOO_HIGH:
            sprintf((char *) &(pageInformation.instructions2), (char*) "Noise Level");
            sprintf((char *) &(pageInformation.instructions1), (char*) "too high");
            break;
    }
    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    update_display = GLOBAL_UPDATE; //

}

/****************************************************************************
  Function:
   void CLICK_demo( void )

  Description:
    This function initializes the hardware.  Shows strike distance,energy value 
 * and the number of total strikes every 10 seconds.

  Precondition:
 Graphic objects must be created by calling ShowScreenClick_demo()

  Parameters:
    None

  Returns:
    None

  Remarks:
    The ProcessMessage_DEMO_test() function will exit the test and return to 
 * the main display
 ***************************************************************************/
void CLICK_demo(void) {

//    static uint8_t RunTestCNT;

    if (tick > demoTick) {
        demoTick = tick + 1000;
        switch (CLICK_DEMO_STATE) {
            case INITIALIZE:
                power_mode(HIGH_POWER);
                DRV_Thunder_Init();    //   Initialize Thunder module
                CLICK_DEMO_STATE = SPLASH_SCREEN;
                break;
            case SPLASH_SCREEN:
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions1, " Thunder Demo ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions, "ae AS3935");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;

                CLICK_DEMO_STATE = DISPLAY_THUNDER;
                break;
            case DISPLAY_THUNDER:
                if (LIGHTNINGDETECTED()) {
                    power_mode(HIGH_POWER);
                    Display_Lightning_information();
                }
//
//                if (RunTestCNT++ > 1) {
//                    RunTestCNT = 0;
//                    power_mode(HIGH_POWER);
//                    Display_Lightning_information();
//                }
                break;
            case EXIT_DEMO:
                screenState = SCREEN_DISPLAY_MAIN;
                DRV_THUNDER_POWER_DOWN();
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions1, "Demo Exit ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
        }
    }
}

/****************************************************************************
  Function:
    bool ProcessMessage_DEMO_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                        GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for Click demo test selection screen. 

  Precondition:
    Call ShowScreenIR_demo() prior to using this function to display the
    correct screen and generate the required graphic objects.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    global change variable to used  clean up later.
 ***************************************************************************/
bool ProcessMessage_DEMO_test(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE * pMsg) {
    WORD controlID;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                CLICK_DEMO_STATE = EXIT_DEMO;
                return false; // Hidden button

            } else {
                return false;
            }
            break;
        default:
            return false;
    }

    return true; // Call the default handler to show the button state.
}


