/********************************************************************
 ********************************************************************
 * FileName:    drv_spi_THUNDER.h
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  * Description:
 This file contains the low level drivers for the Lightning board
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/23/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/
#ifndef _THUNDER_H
// DOM-IGNORE-BEGIN
    #define _THUNDER_H
#include <stdint.h>
#include "system.h"
#include "string.h"

#define _INDOOR  0x12
#define _OUTDOOR 0x0E

#define  NOISE_LEVEL_TOO_HIGH  0x01
#define  DISTURBER_DETECTED    0X04
#define  LIGHTNING_DETECTED    0x08

#define THUNDERCSLow()   MIKROBUS_CS_LAT = 0;
#define THUNDERCSHigh()  MIKROBUS_CS_LAT = 1;

#define LIGHTNINGDETECTED() MIKROBUS_INT == 1

/******************************************************************************
  Function:
    void DRV_LIGHTING_Initialize( const SYS_MODULE_INDEX index,
                                           const SYS_MODULE_INIT * const init )

  Summary:
    Initializes hardware and data for the given instance of the Lightning click
 *  module

  Description:
    This routine initializes hardware for the instance of the Lightning click module,
    using the hardware initialization given data.  It also initializes all
    necessary internal data.

  Parameters:
    pInitData - Pointer to the data structure containing all data
                necessary to initialize the hardware. This pointer may
                be null if no data is required and static initialization
                values are to be used.

  Returns:
    None
 ******************************************************************************/
void DRV_LIGHTNING_Initialize(DRV_SPI_INIT_DATA *pInitData);
void DRV_Thunder_Write(uint8_t address, uint8_t data1);
uint8_t DRV_Thunder_Read(uint8_t address);
uint8_t DRV_Thunder_Read_distance(void);
uint32_t DRV_Thunder_Read_Energy(void);
void DRV_Thunder_Init(void);
void DRV_THUNDER_POWER_UP(void);
void DRV_THUNDER_POWER_DOWN(void);

#endif

