/********************************************************************
 ********************************************************************
 * FileName:     heartRate_main.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file is the high level app
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/9/2016    DRW       Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <stdint.h>
#include "system_config.h"
#include "timer.h"
#include <string.h>
#include "stdio.h"
#include"../Utilities.h"
#include "gfx/gfx.h"
#include "heartrate1_hw.h"
#include "heartrate1_hal.h"

void heartRate(void);
void System_Init(void);
BOOL log_heartRateData(void);
void computeMovingAVE(uint8_t size);
void determineSlope(void);
uint8_t determineHeartRate(void);
void Display_HR_information(uint8_t hr);

extern APP_SCREEN_UPDATE_STATES update_display;
APP_SCREEN_IR_DEMO_STATES CLICK_DEMO_STATE;
extern GFX_GOL_OBJ_SCHEME pScheme_button;
static DWORD demoTick = 0;


#define Slope_offset       20
#define waitforRisingEdge  0
#define waitforFallingEdge 1 

/****************************************************************************
  Function:
    void System_Init( void )

  Description:
    This function sets up the i2c address and sensor sample rate and pulse width


  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    
 ***************************************************************************/
void System_Init(void) {
    MIKROBUS_INT_TRIS = 1;
    heartrate1_init( MAX30100_I2C_ADR );
}

/****************************************************************************
  Function:
    void ShowScreenClick_demo( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenClick_demo(void) {
    strcpy(pageInformation.title, "HR DEMO ");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects

    strcpy(pageInformation.instructions2, "HR Demo ");
    strcpy(pageInformation.instructions1, "MAX30100");
    strcpy(pageInformation.instructions, "");

    strcpy(pageInformation.instructions3, "");
    strcpy(pageInformation.buttonTextBottomLeft, " OK");

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 61, 127, 61 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 81, 74, 95, 2,                // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,       // will be displayed after creation
            NULL,                             // bitmap
            NULL,                             // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //
}

/****************************************************************************
  Function:
   void Display_HR_information( uint8_t hr )

  Description:
    This function prints the heart rate data

  Precondition:
 graphic objects must he created

  Parameters:
 heart rate beats per minute

  Returns:
    None

  Remarks:
 
 ***************************************************************************/
void Display_HR_information(uint8_t hr) {
    
    if(hr == 0)
         sprintf((char *) &(pageInformation.instructions2), (char*) "Put finger on sensor");
    else
        sprintf((char *) &(pageInformation.instructions2), (char*) "Heart Rate: %3d bpm", hr);
    
        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

//        sprintf((char *) &(pageInformation.instructions1), (char*) " SP02: %3d ", 93);
//        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
//        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
//        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
//
//        sprintf((char *) &(pageInformation.instructions), (char*) " Temp: %3d C", 27);
//        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
//        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
//        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

        update_display = GLOBAL_UPDATE; //
}

/****************************************************************************
  Function:
   void CLICK_demo( void )

  Description:
    This function initializes the hardware.  Shows temperature from the IR sensor
 * every 3 seconds.

  Precondition:
 Graphic objects must be created by calling click_demo()

  Parameters:
    None

  Returns:
    None

  Remarks:
    The ProcessMessage_DEMO_test() function will exit the test and return to 
 * the main display
 ***************************************************************************/
void CLICK_demo(void) {
    uint8_t partID;
    uint8_t revisionID;
    static uint8_t heartRate;

    if (tick > demoTick) {
        demoTick = tick  + 500;
        switch (CLICK_DEMO_STATE) {
            case INITIALIZE:
                power_mode(HIGH_POWER);
                MKROE_power_on();
                delay_ms(50);  // allow time for power to stabilize
                System_Init();
                CLICK_DEMO_STATE = SPLASH_SCREEN;
                break;
            case SPLASH_SCREEN:
                power_mode(HIGH_POWER);
                revisionID = heartrate1_get_revision_id();
                partID = heartrate1_get_part_id();
                sprintf((char *) &(pageInformation.instructions1), (char*) "ID:%3d  Rev:%3d", partID, revisionID);
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions, "MAX30100");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                CLICK_DEMO_STATE = AQUIRE_HEARTRATE;
                break;
            case AQUIRE_HEARTRATE:
                power_mode(HIGH_POWER);
                log_heartRateData();// this is a blocking function. key press is not detected
                CLICK_DEMO_STATE = COMPUTE_HEARTRATE;
                break;
            case COMPUTE_HEARTRATE:
                power_mode(HIGH_POWER);
                computeMovingAVE(32);
                determineSlope();                         // 
                heartRate = determineHeartRate();         // increment when the slope changes 
                if(((heartRate > 50) && (heartRate < 90) )|| (heartRate == 0))  // keep from showing crazy numbers
                    CLICK_DEMO_STATE = DISPLAY_RESULTS;   // this is only a demo 
                else
                     CLICK_DEMO_STATE = AQUIRE_HEARTRATE;
                break;
            case DISPLAY_RESULTS:
                power_mode(HIGH_POWER);
                Display_HR_information(heartRate);
                CLICK_DEMO_STATE = AQUIRE_HEARTRATE;
                break;
            case EXIT_DEMO:
                MKROE_power_off();
                screenState = SCREEN_DISPLAY_MAIN;
                power_mode(HIGH_POWER);
                
                strcpy(pageInformation.instructions1, "Power OFF ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

                strcpy(pageInformation.instructions, " ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

                strcpy(pageInformation.instructions2, " ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);


                update_display = GLOBAL_UPDATE;
                break;
        }
    }
}


/****************************************************************************
  Function:
    bool ProcessMessage_DEMO_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                        GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for Click demo test selection screen. 

  Precondition:
    Call ShowScreenIR_demo() prior to using this function to display the
    correct screen and generate the required graphic objects.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    global change variable to used  clean up later.
 ***************************************************************************/
bool ProcessMessage_DEMO_test(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD controlID;
    
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                CLICK_DEMO_STATE = EXIT_DEMO;
                return false; // Hidden button

            } else {
                return false;
            }
            break;
        default:
            return false;
    }
    return true; // Call the default handler to show the button state.
}


/****************************************************************************
  Function:
    bool log_heartRateData( void )

  Summary:
    This routine logs data to a large buffer to be processed later. The data is 
 * to be contiguous to ensure proper heart rate evaluation

  Description:
 This routine gathers at most 16 data samples and store the data into a larger buffer.
 * If a data overflow is detected the data sample is retaken.  

  Precondition:
     I2C channel 2 must be set up

  Parameters:
     none

  Return Values:
 always true.  need to add failure recovery

  Remarks:
 * make sure the data sampling does not trip the wdt.  
 ***************************************************************************/
BOOL log_heartRateData(void) {
    uint16_t dataPointCount = 0;
    uint8_t dataPoints;
    uint8_t interupStatus;
    uint16_t  ir_buff[16]  = {0}, red_buff[16] = {0};
    uint8_t i;
    dataPointCount = 0;
    // do not over run the buffer
    while (dataPointCount < (SAMPLES - 16)) {
        if (DataIsReady()) {
            interupStatus = heartrate1_get_status();  // this will clear the data ready flag
            dataPoints = heartrate1_read_ir_red( ir_buff, red_buff );
            if (dataPoints == 0xFF) {// data over run has occured
                dataPointCount = 0;  // start over
                resetFIFO_PointerRegisters();
            } else {
                for (i = 0; i < dataPoints ; i++) {
                    hr_buffer[dataPointCount] = ir_buff[i];
                    //red_buffer[dataPointCount] = red_buff[i];
                    dataPointCount++;
                }
            }
        }
    }
    return true;
}


/****************************************************************************
  Function:
    void computeMovingAVE( void )

  Summary:
    This routine performs a moving average of the hr_rate data buffer

  Description:
 This routine gathers should use a moving average size of 256,128,64,32 to make 
 * the division simple. The compiler should recognize to only shift left

  Precondition:
    

  Parameters:
 moving average size.  Pick powers of 2 to make the math fast. 

  Return Values:
 none

  Remarks:
   
 ***************************************************************************/
void computeMovingAVE(uint8_t size) {
    uint16_t i;
    uint16_t j;
    int32_t average;
    
    for (i = 0; i < (SAMPLES - size); i++) {
        average = 0;
        for (j = 0; j < size; j++) {
            average += hr_buffer[i + j];
        }
        hr_buffer[i] = average/size;  
    }
}



/****************************************************************************
  Function:
    uint8_t determineHeartRate( void )

  Summary:
    This routine looks through the buffer and counts the local slope changes

  Description:
 This routine needs to see a negative to positive transition before incrementing
 * the heart rate counter.  30 second data sample  multiply by 2

  Precondition:
 proper placement of the tip of the finger on sensor

  Parameters:
   

  Return Values:
 heart rate count

  Remarks:
   
 ***************************************************************************/
uint8_t determineHeartRate(void){
    uint16_t i;
    uint8_t state = waitforRisingEdge;
    uint8_t pulse_bpm = 0;
    
    for(i=0;i<900;i++)
    {
        switch(state){
            case waitforRisingEdge:
                if(hr_buffer[i] > (Slope_offset + 1))
                    state = waitforFallingEdge;
                break;
            case waitforFallingEdge:
                 if(hr_buffer[i] < (Slope_offset -1)){
                     state = waitforRisingEdge;
                     pulse_bpm++;
                 }
                break;
            default:
                state = waitforRisingEdge;
        }  
    }
    return pulse_bpm *2 ;  
}


/****************************************************************************
  Function:
    void determineSlope( void )

  Summary:
    This routine looks through the buffer 

  Description:
 This routine

  Precondition:


  Parameters:
   

  Return Values:
 heart rate count

  Remarks:
   
 ***************************************************************************/
void determineSlope(void){
    uint16_t result_index = 0;
    uint16_t i;
    uint16_t sumA,sumB;
    uint8_t j;
    
    for(i=0;i<SAMPLES-4;i+=3)
    {
        sumA=0;
        sumB=0;
        for(j=0;j<3;j++)
        {
            if(hr_buffer[i+j] > 10000)  // is finger present
              sumA += hr_buffer[i+j];
            if(hr_buffer[i+j+3] > 10000)
              sumB += hr_buffer[i+j+3];
        }
        hr_buffer[result_index++] = (Slope_offset + sumA) - sumB;   // keep it positive
    }
}


/****************************************************************************
  Function:
    void testCLICK_HR_Status( void )

  Summary:
    This routine verifies the 12c communications with external devices.

  Description:
     The status is requested from the gas gauge every 2 seconds.  The expected 
 * return values are 0b10000000) or 0b10000001. 

  Precondition:
     I2C channel 3 must be set up

  Parameters:
     none

  Return Values:
     none

  Remarks:
 this is a forever test loop, wdt must be disabled
 ***************************************************************************/
void testCLICK_HR_Status(void) {
    uint8_t status;
    int rate = 200;
    DWORD displayTick_10ms = 0;
    uint8_t cfg_reset[1] = {0};
    MKROE_power_off();
    
    RCONbits.SWDTEN = 0; // disable watch dog timer
    heartrate1_i2c_init( MAX30100_I2C_ADR );
    delay_ms(1000);
    MKROE_power_on();
    while (1) {
        if ((tick - displayTick_10ms) > rate) {
            displayTick_10ms = tick;
            heartrate1_i2c_hal_read( MODE_CONFIG, 1, cfg_reset );
            status = cfg_reset[0];
            status_1_pin ^= 1;
        }
    }
}

