/**
* \file
*
* \brief The EPD configurations
*
* Copyright (c) 2012-2015 Pervasive Displays Inc. All rights reserved.
*
*  Authors: Pervasive Displays Inc.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include "system.h"
#include <xc.h>
#include <libpic30.h>

#ifndef CONF_EPD_H_INCLUDED
#define CONF_EPD_H_INCLUDED

#if !defined(FALSE)
#define FALSE 0 /**< define FALSE=0 */
#endif

#if !defined(TRUE)
#define TRUE (1) /**< define TRUE=1 */
#endif

#if !defined(NULL)
#define NULL (void *)0  /**< define NULL */
#endif

#if !defined(_NOP)
#define _NOP() asm("nop")
#endif

#if !defined(bool)
#define bool uint8_t
#endif


typedef  unsigned int EInt;
/**
 * \brief Developer needs to create an external function if wants to read memory */
typedef void (*EPD_read_memory_handler) (   EInt memory_address,
                                            uint8_t *target_buffer,
                                            uint8_t byte_length);

#if (USE_EPD_Type==EPD_144)
#define D_PartialUpdate_StageTime            240  //ms
#elif (USE_EPD_Type==EPD_200)
#define D_PartialUpdate_StageTime            240  //ms
#elif (USE_EPD_Type==EPD_270)
#define D_PartialUpdate_StageTime            250  //ms
#endif

#define DELAY_US_CONSTANT (SYS_CLK_FrequencyInstructionGet()/1000000L)
#define DELAY_MS_CONSTANT (SYS_CLK_FrequencyInstructionGet()/1000L)

//  #define delay_us(x) __delay32(x*DELAY_US_CONSTANT)  // delays x us
//  #define delay_ms(x) __delay32(x*DELAY_MS_CONSTANT)  // delays x ms

#include "COG_FPL.h"
#include "EPD_hardware_driver.h"
#include "EPD_interface.h"
#endif /* CONF_EPD_H_INCLUDED */

