/******************************************************************************

    E paper dev kit Utilities

Description:
    This file contains utilities for the PIC24F E paper development Kit.

    Set Date/Time
    The main screen for the PIC24F E paper development Kit demonstration 
    program displays the current date and time.  If the initial date and time 
    are incorrect,use this utility to set it.  The currently selected parameter is indicated
    by a inverse display.  Select the different parameter by using the left
    and right touchpads.  Adjust the selected parameter by using the left and
    right touchpads.  The RTCC is UPDATEd with each change.  Touch the center
    touchpad to exit.


 *******************************************************************************/
//DOM-IGNORE-BEGIN
/******************************************************************************

 * File Name:       Utilities.c
 * Dependencies:    See the #include files below
 * Processor:       PIC24FJ256GB1
 * Compiler:        XC16 V1.25
 * Company:         Electronic Design Solutions LLC



Author          Date    Comments
--------------------------------------------------------------------------------
DRW          14-Jun-2014 First release

 *******************************************************************************/
//DOM-IGNORE-END

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "resources/font_resources_epd_demo.h"
#include "rtcc.h"
#include "Utilities.h"
#include "Network.h"
#include "stripChart.h"

#include "system.h"
#include "../framework/miwi/miwi_api.h"
#include "revision_history.h"



//******************************************************************************
//******************************************************************************
// Constants and Enumerations
//******************************************************************************
//******************************************************************************


#define ID_UTILITIES_RSSI_TEST      ID_BOTTOM_LEFT
#define ID_UTILITIES_BAT_MON        ID_TOP_LEFT
#define ID_UTILITIES_HOME           ID_TOP_RIGHT
#define ID_UTILITIES_USB_TEST       ID_BOTTOM_RIGHT

#define ID_TIME_MONTH               ID_TOP_LEFT
#define ID_TIME_DAY                 ID_BOTTOM_LEFT
#define ID_TIME_HOUR                ID_TOP_RIGHT
#define ID_TIME_MINUTES             ID_BOTTOM_RIGHT

#define ID_START                    50
#define ID_MONTH                    (ID_START +  0)
#define ID_MONTH_PLUS               (ID_START +  1)
#define ID_MONTH_MINUS              (ID_START +  2)
#define ID_DAY                      (ID_START +  3)
#define ID_DAY_PLUS                 (ID_START +  4)
#define ID_DAY_MINUS                (ID_START +  5)
#define ID_YEAR                     (ID_START +  6)
#define ID_YEAR_PLUS                (ID_START +  7)
#define ID_YEAR_MINUS               (ID_START +  8)
#define ID_HOUR                     (ID_START +  9)
#define ID_HOUR_PLUS                (ID_START + 10)
#define ID_HOUR_MINUS               (ID_START + 11)
#define ID_MINUTE                   (ID_START + 12)
#define ID_MINUTE_PLUS              (ID_START + 13)
#define ID_MINUTE_MINUS             (ID_START + 14)
#define ID_HOME                     (ID_START + 15)




//******************************************************************************
//******************************************************************************
// Global Variables
//******************************************************************************
//******************************************************************************


// Set Date/Time Variables

BYTE daysPerMonth[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
XCHAR monthNames[12][3] = {
    { 'J', 'a', 'n', },
    { 'F', 'e', 'b', },
    { 'M', 'a', 'r', },
    { 'A', 'p', 'r', },
    { 'M', 'a', 'y', },
    { 'J', 'u', 'n', },
    { 'J', 'u', 'l', },
    { 'A', 'u', 'g', },
    { 'S', 'e', 'p', },
    { 'O', 'c', 't', },
    { 'N', 'o', 'v', },
    { 'D', 'e', 'c'}
};


DIAGNOSTIC_STATES testState = DIAGNOSTIC_INIT;
NETWORK_STATUS_STATES network_status_state = NETWORK_STATUS_INIT;
ECHO_RSSI_STATES echo_rssi_state = ECHO_RSSI_TX;
DWORD rssiDelay_10ms;
extern DWORD tick_10ms;

STATUS_BITS REMOTE_DEVICE_STATUS[32];
extern volatile BYTE PeerIndex;

BOOL networkScanStart;
DWORD monitorNetwork_tick10ms;

SHORT Utilities_currentControlSet;
DWORD pingDelayTime;                    // periodic time to ping for rssi echo
extern APP_SCREEN_UPDATE_STATES update_display;


extern GFX_GOL_OBJ_SCHEME  pScheme_EditBox;
extern GFX_GOL_OBJ_SCHEME  pScheme_button;


//extern CONNECTION_ENTRY ConnectionTable[CONNECTION_SIZE];

//extern IMAGE_FLASH intro;
//*****************************************************************************
//******************************************************************************
// Function Prototypes
//******************************************************************************
//******************************************************************************

void UpdateDateTime(void);
BOOL sendRequest_RemoteDeviceStatus(BYTE);

/****************************************************************************
  Function:
    WORD ProcessMessageUtilities( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                    GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for the utility selection screen.

  Precondition:
    None

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    None
 * GFX_GOL_MESSAGE
 ***************************************************************************/
bool ProcessMessageUtilities(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD buttonID;

    buttonID = GFX_GOL_ObjectIDGet(pObj);

    switch (buttonID) {
        case ID_UTILITIES_BAT_MON:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_BATTERY;
            }
            break;
        case ID_UTILITIES_USB_TEST: //ID_BOTTOM_RIGHT
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_USB_MEMORY;
            }
            break;
        case ID_UTILITIES_RSSI_TEST:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_MAIN;
            }
            break;
        case ID_UTILITIES_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_GRAPH;
            }
            break;
    }
    return FALSE; // We do not need to call the default handler.
}

/****************************************************************************
  Function:
    void ShowScreenUtilities( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    the screen to allow the user to select which utility to execute.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenUtilities(void) {
    strcpy(pageInformation.title, "Utilities");

    strcpy(pageInformation.buttonTextTopLeft, "Bat Mon");
    strcpy(pageInformation.buttonTextBottomLeft, "Home"); //
    strcpy(pageInformation.buttonTextTopRight, "  RSSI ");
    strcpy(pageInformation.buttonTextBottomRight, "USB Test"); //
    strcpy(pageInformation.buttonTextThirdrow, "");

    pageInformation.buttonBitmapTopLeft = NULL;
    pageInformation.buttonBitmapBottomLeft = NULL;
    pageInformation.buttonBitmapTopRight = NULL;
    pageInformation.buttonBitmapBottomRight = NULL;

    strcpy(pageInformation.instructions, VERSION_STRING);

    ShowScreenGeneric();
}

///////////////////////////////////// TIME /////////////////////////////////////

/****************************************************************************
  Function:
    void UpdateDateTime( void )

  Description:
    This function updates the displayed time and date to match the value
    currently in the RTCC.

  Precondition:
    ShowScreenTime() must be called before calling this routine to display
    the correct controls.

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void UpdateDateTime(void) {
    GFX_GOL_OBJ_HEADER* obj;

    RTCCProcessEvents(); // update the time and date

    switch (Utilities_currentControlSet) {
        case CONTROL_SET_MONTH:
            obj = GFX_GOL_ObjectFind(ID_MONTH);
            GFX_GOL_EditBoxTextSet((GFX_GOL_EDITBOX*) obj, &_date_str[0]);
            GFX_GOL_ObjectStateSet(obj, GFX_GOL_EDITBOX_DRAW_STATE);
            break;
        case CONTROL_SET_DAY:
            obj = GFX_GOL_ObjectFind(ID_DAY);
            GFX_GOL_EditBoxTextSet((GFX_GOL_EDITBOX*) obj, &_date_str[4]);
            GFX_GOL_ObjectStateSet(obj, GFX_GOL_EDITBOX_DRAW_STATE);
            break;
        case CONTROL_SET_YEAR:
            obj = GFX_GOL_ObjectFind(ID_YEAR);
            GFX_GOL_EditBoxTextSet((GFX_GOL_EDITBOX*) obj, &_date_str[8]);
            GFX_GOL_ObjectStateSet(obj, GFX_GOL_EDITBOX_DRAW_STATE);
            break;
        case CONTROL_SET_HOUR:
            obj = GFX_GOL_ObjectFind(ID_HOUR);
            GFX_GOL_EditBoxTextSet((GFX_GOL_EDITBOX*) obj, (GFX_XCHAR*) & _time_str[4]);
            GFX_GOL_ObjectStateSet(obj, GFX_GOL_EDITBOX_DRAW_STATE);
            break;
        case CONTROL_SET_MINUTE:
            obj = GFX_GOL_ObjectFind(ID_MINUTE);
            GFX_GOL_EditBoxTextSet((GFX_GOL_EDITBOX*) obj, &_time_str[7]);
            GFX_GOL_ObjectStateSet(obj, GFX_GOL_EDITBOX_DRAW_STATE);
            break;
        default:
            break;
    }


}

void ShowScreenTimeAdjust(void) {

    InitializeScreen(); // clear linked object list, add icon graphic and title

    strcpy(pageInformation.buttonTextTopLeft, "  -"); // left key
    strcpy(pageInformation.buttonTextBottomLeft, " OK"); // center key
    strcpy(pageInformation.buttonTextTopRight, "  + "); // right key
    addSoftMenuKeys();



    // add correct edit box
    switch (Utilities_currentControlSet) {
        case CONTROL_SET_MONTH:
            GFX_GOL_EditBoxCreate(
                    ID_MONTH,
                    34,
                    16,
                    102,
                    34,
                    GFX_GOL_EDITBOX_DRAW_STATE,
                    &_date_str[0],
                    3,
                    GFX_ALIGN_CENTER,
                    &pScheme_EditBox);
            break;
        case CONTROL_SET_DAY:
            GFX_GOL_EditBoxCreate(
                    ID_DAY,
                    34,
                    16,
                    102,
                    34,
                    GFX_GOL_EDITBOX_DRAW_STATE,
                    &_date_str[4],
                    3,
                    GFX_ALIGN_CENTER,
                    &pScheme_EditBox);
            break;
        case CONTROL_SET_YEAR:
            GFX_GOL_EditBoxCreate(
                    ID_YEAR,
                    34,
                    16,
                    102,
                    34,
                    GFX_GOL_EDITBOX_DRAW_STATE, // will be displayed after creation
                    &_date_str[8], // use the year string value
                    3,
                    GFX_ALIGN_CENTER,
                    &pScheme_EditBox);
            break;
        case CONTROL_SET_HOUR:
            GFX_GOL_EditBoxCreate(
                    ID_HOUR,
                    34,
                    16,
                    102,
                    34,
                    GFX_GOL_EDITBOX_DRAW_STATE, // will be dislayed after creation
                    &_time_str[4], // use the hour string value year string value
                    3,
                    GFX_ALIGN_CENTER,
                    &pScheme_EditBox);
            break;
        case CONTROL_SET_MINUTE:
            GFX_GOL_EditBoxCreate(
                    ID_MINUTE,
                    34,
                    16,
                    102,
                    34,
                    GFX_GOL_EDITBOX_DRAW_STATE, // will be dislayed after creation
                    &_time_str[7], // use the hour string valuehe hour string value year string value
                    3,
                    GFX_ALIGN_CENTER,
                    &pScheme_EditBox);
            break;
        default:
            break;
    }
}

bool ProcessMessageTime(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    signed char change;
    WORD controlID;
    signed char day;
    signed char dayMax;
    signed char hour;
    signed char minute;
    unsigned char month;
    signed char year;

    change = 0;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_INC:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                change = +1;
            } else {
                return FALSE;  // dont show key release action
            }
            break;
        case ID_BUTTON_GENERIC_DEC:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                change = -1;
            } else {
                return FALSE;
            }
            break;
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_MAIN;
                return FALSE; // Hidden button
            } else {
                return FALSE;
            }
            break;
        default:
            return FALSE;
    }

    // Update the selected field.

    RTCCProcessEvents();
    hour = RTCCGetBinHour();
    minute = RTCCGetBinMin();
    day = RTCCGetBinDay();
    month = RTCCGetBinMonth();
    year = RTCCGetBinYear();

    dayMax = daysPerMonth[month - 1];

    // February has one day more for a leap year, unless it is on the thousands
    if ((month == 2) && ((year % 4) == 0) && (year != 0)) {
        dayMax++;
    }

    // Change the appropriate setting.  Allow the controls to roll over.
    switch (Utilities_currentControlSet) {
        case CONTROL_SET_DAY:
            day += change;
            if (day < 1) {
                day = dayMax;
            }
            if (day > dayMax) {
                day = 1;
            }
            break;

        case CONTROL_SET_MONTH:
            month += change;
            if (month < 1) {
                month = 12;
            }
            if (month > 12) {
                month = 1;
            }
            break;

        case CONTROL_SET_YEAR:
            year += change;
            if (year < 0) {
                year = 99;
            }
            if (year > 99) {
                year = 0;
            }
            break;
        case CONTROL_SET_HOUR:
            hour += change;
            if (hour < 0) {
                hour = 23;
            }
            if (hour > 23) {
                hour = 0;
            }
            break;

        case CONTROL_SET_MINUTE:
            minute += change;
            if (minute < 0) {
                minute = 59;
            }
            if (minute > 59) {
                minute = 0;
            }
            break;

        default:
            return FALSE;
    }

    RTCCOff();
    RTCCSetBinHour(hour);
    RTCCSetBinMin(minute);
    RTCCSetBinSec(0);       // second will be set to zero anytime adjustment
    RTCCSetBinMonth(month);
    RTCCSetBinYear(year);
    RTCCSetBinDay(day);
    RTCCCalculateWeekDay();
    RTCCSet();              // Copy the new values to the RTCC registers
    RTCCOn();
    UpdateDateTime();       // Update the display.

    update_display = GLOBAL_UPDATE;
    return FALSE;           // Call the default handler to show the button state.
}

bool ProcessMessageTIMESubMenu(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD controlID;

    if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
        controlID = GFX_GOL_ObjectIDGet(pObj);

        switch (controlID) {
            case ID_TIME_MONTH: //ID_TOP_LEFT
                screenState = SCREEN_DISPLAY_ADJUST_MONTH;
                break;

            case ID_TIME_MINUTES: //ID_BOTTOM_RIGHT
                screenState = SCREEN_DISPLAY_ADJUST_MINUTES;
                break;

            case ID_TIME_DAY: //ID_BOTTOM_LEFT
                screenState = SCREEN_DISPLAY_ADJUST_DAY;
                break;

            case ID_TIME_HOUR: //ID_TOP_RIGHT
                screenState = SCREEN_DISPLAY_ADJUST_HOUR;
                break;
        }
    }
    return FALSE; // We already called the default handler.
}

/****************************************************************************
  Function:
    void ShowScreenAdjustTime( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    the Set time menu screen.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenAdjustTime(void) {
    RTCCProcessEvents();

    strcpy(pageInformation.title, " ADJUST TIME ");
    strcpy(pageInformation.buttonTextTopLeft, "MONTH");
    strcpy(pageInformation.buttonTextBottomLeft, "DAY");
    strcpy(pageInformation.buttonTextTopRight, "HOUR");
    strcpy(pageInformation.buttonTextBottomRight, "MINUTES");
    strcpy(pageInformation.buttonTextThirdrow, "");

    pageInformation.buttonBitmapTopLeft = NULL;
    pageInformation.buttonBitmapBottomLeft = NULL;
    pageInformation.buttonBitmapTopRight = NULL;
    pageInformation.buttonBitmapBottomRight = NULL;

    strcpy(pageInformation.instructions, _date_str);
    strcat(pageInformation.instructions, "  ");
    strcat(pageInformation.instructions, _time_str);

    ShowScreenGeneric(); // create 4 buttons, title and a instructions line
}


///////////////////////////////////// Soft keys ////////////////////////////////

void addSoftMenuKeys(void) {

    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 78, 74, 92, 2,          // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE , // will be displayed after creation
            pageInformation.buttonBitmapTopLeft, // bitmap
            NULL,                       // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 

    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_INC, // ID
            88, 78, 113, 92, 2,         // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE , // will be displayed after creation
            NULL,                       // bitmap  no pressed image used  
            NULL,                       // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextTopRight, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 

    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_DEC, // ID
            10, 78, 35, 92, 2,           // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,  // will be displayed after creation
            NULL,                        // bitmap  no pressed image used  
            NULL,                        // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextTopLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button);            // 
}

void TranslateMessageSoftKeys(GFX_GOL_MESSAGE * pMsg) {
    // If the message is valid, we need to translate
    // pMsg->param1 to the proper control ID depending
    // on what control set is selected.
    // setting param2 will set translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED

    if ((pMsg->uiEvent == EVENT_KEYSCAN) &&
            (pMsg->type == TYPE_KEYBOARD) &&
            (pMsg->param1 == ID_TOUCH_PAD)) {
        if ((pMsg->param2 == SCAN_RIGHT_PRESSED)) {
            pMsg->param1 = ID_BUTTON_GENERIC_INC;
            pMsg->param2 = SCAN_CR_PRESSED;
            power_mode(HIGH_POWER);
        } else if (pMsg->param2 == SCAN_LEFT_PRESSED) {
            pMsg->param1 = ID_BUTTON_GENERIC_DEC;
            pMsg->param2 = SCAN_CR_PRESSED;
            power_mode(HIGH_POWER);
        } else if (pMsg->param2 == SCAN_CR_PRESSED) {
            pMsg->param1 = ID_BUTTON_GENERIC_HOME;
            pMsg->param2 = SCAN_CR_PRESSED;
            power_mode(HIGH_POWER);
        }
    }
}


///////////////////////////////////// USB //////////////////////////////////////

void showScreenTestUSBCapture(void) {
    InitializeScreen(); // clear linked object list, add icon graphic and title

    strcpy(pageInformation.buttonTextBottomLeft, " OK"); // center key
    strcpy(pageInformation.instructions2, "  "); // clear previous message



    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 78, 74, 92, 2,                   // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,          // will be displayed after creation
            pageInformation.buttonBitmapTopLeft, // bitmap
            NULL,                                // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

}


////////////////////////////////// Network health  /////////////////////////////

void radioDiagnostics_master(void) {
    BYTE transmittCount;
    static char failcount = 0;
    static char eachRadio;
    static BYTE test_count;

    BYTE RemoteShortAddress[2] = {0x01, 0x00}; //

    BYTE PacketRSSI;
    BYTE pktCmd;
    BYTE batteryStatus;
    BYTE unitID;
    static BYTE numberConnections;
    static BYTE index;

    switch (testState) {
        case DIAGNOSTIC_INIT:
            testState = DIAGNOSTIC_SHOW_CONNECTIONS;
            eachRadio = 0;
            test_count = 0;
            index = 0;
            tick_10ms = tick;
            numberConnections = Number_of_Network_Connections();
            break;
        case DIAGNOSTIC_SHOW_CONNECTIONS:
            if (tick > (tick_10ms + 1000)) {
                //              tick_10ms = tick;
                //              if(index < numberConnections){
                //                  if(ConnectionTable[index].status.bits.isValid){
                //                      // Display Peer information
                //                      sprintf((char *) pageInformation.instructions2, "index:%02d  Addr:%02x%02x    ", index, ConnectionTable[index].Address[1], ConnectionTable[index].Address[0]);
                //                      pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                //                      GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
                //                      GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                //                  }
                //                  index++;
                //              } else {
                //                  testState = DIAGNOSTIC_DELAY;
                //              }
            }

            break;
        case DIAGNOSTIC_DELAY:
            if (tick > (tick_10ms + 1000))
                testState = DIAGNOSTIC_TX_REQUEST;
            break;
        case DIAGNOSTIC_TX_REQUEST:
            failcount = 0;
            test_count++;
            for (transmittCount = 0; transmittCount < 3; transmittCount++) {
                MiApp_FlushTx();
                MiApp_WriteData(DIAGNOSTIC_PKT);
                MiApp_WriteData(test_count);
                MiApp_WriteData(transmittCount);
                if (MiApp_UnicastAddress(RemoteShortAddress, SHORT_ADDRESS, NO_ENCRYPTION) == FALSE) failcount = 1; // use alternate network address
                //DelayMs(100);
                delay_ms(10);
            }
            if (failcount != 0) {
                sprintf((char *) pageInformation.instructions1, "remote:%2d Failed", eachRadio);
                testState = DIAGNOSTIC_END;
            } else {
                sprintf((char *) pageInformation.instructions1, "remote:%2d passed", eachRadio);
                testState = DIAGNOSTIC_RX;
            }
            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions1);
            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
            tick_10ms = tick;
            break;
        case DIAGNOSTIC_RX:
            if (tick > (tick_10ms + 1000)) {
                sprintf((char *) pageInformation.instructions1, "remote:%2d Time Out", eachRadio);
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                testState = DIAGNOSTIC_END;
                tick_10ms = tick;
            } else {
                if (MiApp_MessageAvailable()) {
                    pktCmd = rxMessage.Payload[0];
                    unitID = rxMessage.Payload[1];
                    batteryStatus = rxMessage.Payload[2];
                    PacketRSSI = rxMessage.PacketRSSI;
                    MiApp_DiscardMessage();
                    if (pktCmd == DIAGNOSTIC_PKT) {
                        sprintf((char *) pageInformation.instructions1, "ID:%2d    BAT:%2d    RSSI:%2d", unitID, batteryStatus, PacketRSSI);
                        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions1);
                        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                        testState = DIAGNOSTIC_END;
                        tick_10ms = tick;
                    }
                }
            }
            break;
        case DIAGNOSTIC_END:
            if (tick > (tick_10ms + 250)) {
                testState = DIAGNOSTIC_TX_REQUEST;
            }
            break;
        default:
            testState = DIAGNOSTIC_INIT;
            break;
    }
}

/*******************************************************************************
 * Function:         BYTE displayConnections(void)
 *
 * PreCondition:     must be cooridinator and start a network
 *
 * Input:		    none
 *
 *
 * Output:		    number of connections
 *
 *
 * Side Effects:	    none
 *
 * Overview:		    displays all the known network nodes from
 *                          the network table
 * Note:
 ******************************************************************************/
BYTE Number_of_Network_Connections(void) {
    BYTE NumberOfConnections = 0;
    static BYTE i = 0;

    for (i = 0; i < CONNECTION_SIZE; i++) {
        if ((ConnectionTable[i].status.bits.isValid) &&
                (ConnectionTable[i].status.bits.isFamily)) {
            NumberOfConnections++;
        }
    }
    return NumberOfConnections;
}

BOOL sendRequest_RemoteDeviceStatus(BYTE remote) {
    BYTE Remote1[2] = {0x00, 0x00};
    Remote1[0] = remote + 1;
    MiApp_FlushTx();
    MiApp_WriteData(NETWORK_RSD_STATUS_PKT);
    MiApp_WriteData(0x4D); // MiWi Rocks!
    MiApp_WriteData(0x69);
    MiApp_WriteData(0x57);
    MiApp_WriteData(0x69);
    MiApp_WriteData(0x20);
    MiApp_WriteData(0x52);
    MiApp_WriteData(0x6F);
    MiApp_WriteData(0x63);
    MiApp_WriteData(0x6B);
    MiApp_WriteData(0x73);
    MiApp_WriteData(0x21);
    return (MiApp_UnicastAddress(Remote1, SHORT_ADDRESS, NO_ENCRYPTION));
    //    return 1;
}

void respondToNetworkStatusRequest(BYTE unit_id) {
    BYTE MasterShortAddress[2] = {0x00, 0x00};
    BYTE batteryStatus = 11;                // TODO hardcoded for now
    MiApp_FlushTx();
    MiApp_WriteData(NETWORK_RSD_STATUS_PKT); //                     0
    MiApp_WriteData(unit_id);               //                     1
    MiApp_WriteData(0);                      //                     2
    MiApp_WriteData(rxMessage.PacketRSSI);  // send back the rssi  3
    MiApp_WriteData(batteryStatus);         // battery status      4
    MiApp_UnicastAddress(MasterShortAddress, SHORT_ADDRESS, NO_ENCRYPTION);
}

/****************************************************************************
  Function:
    void monitorNetwork(void)

  Description:
    This function periodically steps through the connection table to update
 the UnitID to connection index array.  This ocurs every 10 seconds


  Precondition:
 Radio must be initialized

  Parameters:
    None

  Returns:
    None

  Remarks:

 ***************************************************************************/
void monitorNetwork(void) {
    if (radioInitialized) {
        if (tick > monitorNetwork_tick10ms + 10000) {
            monitorNetwork_tick10ms = tick;
            networkScanStart  = TRUE;   // lock out the message recieve/discard
        }                               // code in the main loop
        network_status_update_array();
    }
}

void network_status_update_array(void) {
    static BYTE node_count;
    static BYTE each_index;
    BYTE unitID;
    static BYTE tryCount;
    static DWORD status_tick_10ms;
    BYTE pktCmd;

    switch (network_status_state) {
        case NETWORK_STATUS_INIT:
            node_count = Number_of_Network_Connections();
            each_index = 0;
            network_status_state = NETWORK_STATUS_ALL_INDEX;
            break;
        case NETWORK_STATUS_ALL_INDEX:
            if (each_index < node_count) {
                tryCount = 0;
                network_status_state = NETWORK_STATUS_REQUEST;
            } else {
                network_status_state = NETWORK_STATUS_EXIT;
                networkScanStart  = FALSE;
            }
            break;
        case NETWORK_STATUS_REQUEST:
            status_tick_10ms = tick;
            tryCount++;
            if (sendRequest_RemoteDeviceStatus(each_index)) {
                network_status_state = NETWORK_STATUS_WAIT;
            }// no ack from remote
            else {
                network_status_state = NETWORK_STATUS_RETRY_DELAY;
            }
            break;
        case NETWORK_STATUS_WAIT:
            if (tick > status_tick_10ms + 100) // request time out
            {
                status_tick_10ms = tick;
                network_status_state = NETWORK_STATUS_RETRY_DELAY;
            }
            if (MiApp_MessageAvailable()) {
                pktCmd = rxMessage.Payload[0];
                if (pktCmd == NETWORK_RSD_STATUS_PKT) //
                {
                    unitID = rxMessage.Payload[1];
                    REMOTE_DEVICE_STATUS[unitID].ConnectionIndex = each_index;
                    REMOTE_DEVICE_STATUS[unitID].RACESTATUS = rxMessage.Payload[2];
                    REMOTE_DEVICE_STATUS[unitID].RSSI_VALUE = rxMessage.Payload[3];
                    REMOTE_DEVICE_STATUS[unitID].BATTERY = rxMessage.Payload[4];

                    network_status_state = NETWORK_STATUS_NEXT_INDEX;
                }
                MiApp_DiscardMessage();
            }// if message
            break;
        case NETWORK_STATUS_RETRY_DELAY:
            if (tryCount > 3) {
                // "no response"
                network_status_state = NETWORK_STATUS_NEXT_INDEX;
            } else
                if (tick > status_tick_10ms + 100) // retry delay
            {
                network_status_state = NETWORK_STATUS_REQUEST;
            }
            break;
        case NETWORK_STATUS_NEXT_INDEX:
            status_tick_10ms = tick;
            each_index++;
            network_status_state = NETWORK_STATUS_ALL_INDEX;
            break;
        case NETWORK_STATUS_EXIT:
            if (networkScanStart == TRUE)
                network_status_state = NETWORK_STATUS_INIT;
            break;
        default:
            network_status_state = NETWORK_STATUS_INIT;
            break;
    }

}

BOOL echo_rssi(void) {
    BYTE pktCmd;
    BOOL finished_rssi_value = false;
    static BYTE responseTimeout_counter;

    switch (echo_rssi_state) {
        case ECHO_RSSI_TX:
            sendRequest_RemoteDeviceStatus(0); // send to the first remote device in the connection list
            echo_rssi_state = ECHO_RSSI_RX;
            finished_rssi_value = false;
            rssiDelay_10ms = tick;
            responseTimeout_counter = 0;
            break;
        case ECHO_RSSI_RX:
            if (tick > rssiDelay_10ms + 2) {
                if (MiApp_MessageAvailable()) {
                    pktCmd = rxMessage.Payload[0];
                    if (pktCmd == NETWORK_RSD_STATUS_PKT) //
                    {
                        myRSSI = (int)rxMessage.Payload[3];
                    }
                    MiApp_DiscardMessage();
                    finished_rssi_value = true;
                    echo_rssi_state = ECHO_RSSI_TX;
                } else {
                    rssiDelay_10ms = tick;
                    if(responseTimeout_counter++ > 5)
                        echo_rssi_state = ECHO_FAULT;
                } 
            }
            break;
        case ECHO_FAULT:
            myRSSI = -1;
            finished_rssi_value = true;
            echo_rssi_state = ECHO_RSSI_TX;
            break;
        default:
            echo_rssi_state = ECHO_RSSI_TX;
            break;
    }
    return finished_rssi_value;
}




//// precondition  golinit must be called
//
//void testPixel() {
//    short x, y;
//    DWORD displayTick_10ms = 0;
//
//    RCONbits.SWDTEN = 0; // disable watch dog timer
//
//
//    SetColor(WHITE);
//    ClearDevice(); // set screen to current color
//    EPD_Global_Update();
//    displayTick_10ms = tick;
//
//
//
//    while (1) {
//        if((tick - displayTick_10ms) == 100){
//            SetColor(BLACK);
//            status_1_pin = 1;
//            for (y = 10; y < 30; y++)
//                for (x = 10; x < 120; x++)
//                    PutPixel(x, y);
//            EPD_Global_Update();
//        } else if((tick - displayTick_10ms) == 1000){
//            SetColor(WHITE);
//            status_1_pin = 0;
//            for (y = 10; y < 30; y++)
//                for (x = 10; x < 120; x++)
//                    PutPixel(x, y);
//            EPD_Global_Update();
//
//        } else if((tick - displayTick_10ms) > 2000){
//            displayTick_10ms = tick;
//
//        }
//    }
//}
//
//void testPixel_partialUpdate() {
//    short x, y;
//    DWORD displayTick_10ms = 0;
//
//    RCONbits.SWDTEN = 0; // disable watch dog timer
//
//
//    SetColor(WHITE);
//    ClearDevice(); // set screen to current color
//
//
//    EPD_Global_Update();
//    displayTick_10ms = tick;
//
//
//    //EPD_power_init(USE_EPD_Type);
//    while (1) {
//        //if((tick - displayTick_10ms) == 100){
//
//
//
//        SetColor(BLACK);
//        status_1_pin = 1;
//        EPD_power_init(USE_EPD_Type);
//
//        for (y = 10; y < 30; y++) {
//            for (x = 10; x < 120; x++)
//                PutPixel(x, y);
//            EPD_Partial_Update();
//        }
//
//        SetColor(WHITE);
//        status_1_pin = 0;
//        for (x = 10; x < 120; x++) {
//            for (y = 10; y < 30; y++)
//                PutPixel(x, y);
//            EPD_Partial_Update();
//        }
//        SetColor(BLACK);
//        status_1_pin = 1;
//        for (y = 30; y > 10; y--) {
//            for (x = 10; x < 120; x++)
//                PutPixel(x, y);
//            EPD_Partial_Update();
//        }
//        SetColor(WHITE);
//        status_1_pin = 0;
//        for (x = 120; x > 10; x--) {
//            for (y = 10; y < 30; y++)
//                PutPixel(x, y);
//            EPD_Partial_Update();
//        }
//
//    }
//}
//






///////////////////////// Battery Diagnostics //////////////////////////////////




//////////////////////////////////  delete /////////////////////////////////////



/****************************************************************************
  Function:
   BYTE network_status_update(void)

  Description:
    This function steps through all the connected radios.  The end
    device unitID,battery and radio strength data will be populated into the
    status array.
  Precondition:
    radio must be initialized
 initialize network_status_state to NETWORK_STATUS_INIT

  Parameters:

    global network_status_state

  Returns:

    number of devices in the network

  Remarks:
 Updates global network status array.  this array should be zeroed before
 filling with data
 ***************************************************************************/
//void network_status_update(void) {
//    static BYTE node_count;
//    static BYTE each_index;
//    BYTE unitID;
//    static BYTE tryCount;
//    static DWORD status_tick_10ms;
//    BYTE pktCmd;
//
//    switch (network_status_state) {
//        case NETWORK_STATUS_INIT:
//            ShowScreenInitializeRadio();  // this will creat three lines for text
//            node_count = Number_of_Network_Connections();
//            each_index = 0;
//            network_status_state = NETWORK_STATUS_ALL_INDEX;
//            break;
//        case NETWORK_STATUS_ALL_INDEX:
//            if (each_index < node_count) {
//                tryCount = 0;
//                network_status_state = NETWORK_STATUS_REQUEST;
//            } else {
//                network_status_state = NETWORK_STATUS_EXIT;
//            }
//            break;
//        case NETWORK_STATUS_REQUEST:
//            status_tick_10ms = tick;
//            tryCount++;
//            if (sendRequest_RemoteDeviceStatus(each_index)) {
//                network_status_state = NETWORK_STATUS_WAIT;
//            }// no ack from remote
//            else {
//                network_status_state = NETWORK_STATUS_RETRY_DELAY;
//            }
//            break;
//        case NETWORK_STATUS_WAIT:
//            if (tick > status_tick_10ms + 100) // request time out
//            {
//                status_tick_10ms = tick;
//                network_status_state = NETWORK_STATUS_RETRY_DELAY;
//            }
//            if (MiApp_MessageAvailable()) {
//                pktCmd = rxMessage.Payload[0];
//                if (pktCmd == NETWORK_RSD_STATUS_PKT) //
//                {
//                    unitID = rxMessage.Payload[1];
//                    REMOTE_DEVICE_STATUS[unitID].ConnectionIndex = each_index; //rxMessage.Payload[2];
//                    REMOTE_DEVICE_STATUS[unitID].RACESTATUS = rxMessage.Payload[2];
//                    REMOTE_DEVICE_STATUS[unitID].RSSI = rxMessage.Payload[3];
//                    REMOTE_DEVICE_STATUS[unitID].BATTERY = rxMessage.Payload[4];
//                    sprintf((char *) &(pageInformation.instructions1), (char*) "ID:%02d   Index:%02d   RSSI:%02d",
//                            unitID, REMOTE_DEVICE_STATUS[unitID].ConnectionIndex,
//                            REMOTE_DEVICE_STATUS[unitID].RSSI);
//                    sprintf((char *) &(pageInformation.instructions2), (char*) "Battery:%2d ", REMOTE_DEVICE_STATUS[unitID].BATTERY);
//                    network_status_state = NETWORK_STATUS_PRINT_STATUS;
//                }
//                MiApp_DiscardMessage();
//            }// if message
//            break;
//        case NETWORK_STATUS_RETRY_DELAY:
//            if (tryCount > 3) {
//                strcpy(pageInformation.instructions1, "no response");
//                sprintf((char *) &(pageInformation.instructions2), (char*) "Index:%2d ", each_index);
//                network_status_state = NETWORK_STATUS_PRINT_STATUS;
//            } else
//                if (tick > status_tick_10ms + 100) // retry delay
//            {
//                network_status_state = NETWORK_STATUS_REQUEST;
//            }
//            break;
//        case NETWORK_STATUS_PRINT_STATUS:
//            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
//            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions1);
//            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
//            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
//            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
//            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
//            status_tick_10ms = tick;
//            each_index++;
//            network_status_state = NETWORK_STATUS_PRINT_DELAY;
//            break;
//        case NETWORK_STATUS_PRINT_DELAY:
//            if (tick > status_tick_10ms + 1000) // show screen for 10 seconds
//                network_status_state = NETWORK_STATUS_ALL_INDEX;
//            break;
//        case NETWORK_STATUS_EXIT:
//            screenState = SCREEN_DISPLAY_MAIN;
//            break;
//        case NETWORK_STATUS_NEXT_INDEX:
//            screenState = SCREEN_DISPLAY_MAIN;
//            break;
//    }
//
//}


