/********************************************************************
 ********************************************************************
 * FileName:     main.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file is the main top level program
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/9/2016    DRW       Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/*
 
 TODO
 1 
    
 2 
 
 
 
 
 
 */



// *****************************************************************************
// Section: Includes
// *****************************************************************************
#include "system_config.h"

#include <stdint.h>
#include <string.h>
#include <stdlib.h>
// real time clock calender
#include "rtcc.h"
// radio
#include"../framework/miwi/miwi_api.h"
// display
#include "gfx/gfx.h"
#include "resources/font_resources_epd_demo.h"
#include "Pervasive_Displays_small_EPD/EPD_interface.h"
#include "../framework/driver/gfx/drv_gfx_display.h"
#include "drv_serial_ram_mchp23k256.h"
#include "../framework/driver/eeprom_spi/drv_nvm_eeprom_spi_25aa02e48.h"
#include "gfx/gfx/drv_gfx_pdi_e_paper.h"
// app specific header files
#include "Utilities.h"
#include "Battery_utility.h"
#include "USER2_FUNCTIONS.h"
#include "USER1_FUNCTIONS.h"
#include "BLE_LoopBack.h"
#include "Network.h"
#include "stripChart.h"
#include "stdio.h"
// usb
#include "usb.h"
//      usb host
#include "usb_host_msd.h"
#include "app_host_msd_data_logger.h"
//      usb device
#include "usb_device.h"
#include "usb_device_cdc.h"
#include "app_device_cdc_basic.h"
#include "../../../../Microchip/mla/v2015_05_15/framework/miwi/miwi_api.h"
#include "Bluetooth.h"
#include "driver/uart/drv_uart1.h"

#include"MikroElektronika/IRClick/IR_hw.h"
#include"MikroElektronika/heartClick/heartrate1_hw.h"
#include"MikroElektronika/rfidClick/drv_spi_RFID.h"
#include"MikroElektronika/ThunderClick/drv_spi_THUNDER.h"



#define USE_AND_OR

#define ID_MAIN_SET_DATE_TIME       ID_TOP_LEFT
#define ID_MAIN_USER2               ID_BOTTOM_LEFT
#define ID_MAIN_BLE                 ID_BOTTOM_LEFT
#define ID_MAIN_UTILITIES           ID_TOP_RIGHT
#define ID_MAIN_USER1               ID_BOTTOM_RIGHT
#define MESSAGE_DEAD_TIME           (1000/MILLISECONDS_PER_TICK) 

//if ADDITIONAL_NODE_ID_SIZE > 0
//    uint8_t AdditionalNodeID[ADDITIONAL_NODE_ID_SIZE] = {1};
//#endif

// *****************************************************************************
// Local Prototypes
// *****************************************************************************

// menu and graphic functions
void ShowScreenMain(void);        // add objects to screen
bool ProcessMessageMain(WORD, GFX_GOL_OBJ_HEADER*, GFX_GOL_MESSAGE*);
void TouchSenseButtonsMsg(GFX_GOL_MESSAGE*);
void TranslateTouchpad(GFX_GOL_MESSAGE*);
void TranslateMessageGeneric(GFX_GOL_MESSAGE *);
void transfer_pixel_data_from_sram_to_display(void);
void initialize_scheme(void);     // initialize fonts
bool APP_ObjectMessageCallback(
        GFX_GOL_TRANSLATED_ACTION objectMessage,
        GFX_GOL_OBJ_HEADER *pObject,
        GFX_GOL_MESSAGE *pMessage);
bool APP_ObjectDrawCallback(void);



// radio functions
void networkBeaconResponse(void); // allow devices to join network
void handleRemoteCommands(void);  // remote device to respond to master

// system functions
void switchXTAL_32(char);         // switch from frc 8mhz to 32Mhz external pll
void heartbeat(void);             // periodic pulse, held diagnose issues
// individual test cases with out invoking the graphics lib and driving the display
void testSram(void);
void testEEuid(void);
void testBatteryStatus(void);
void testUSB_HOST(void);
void testUSB_DEVICE(void);
void testClock_switching(void);   // forever loop for testing power save methods
BOOL write_read_EE_test(BYTE);
void Conserve_power_in_the_weeds(void);  // sleep mode hangs.
void redefine_permanent_address_definition(void);


// *****************************************************************************
// Application Variables
// *****************************************************************************

TOUCHPADS buttons;
TOUCHPADS buttonsPrevious;

GFX_GOL_OBJ_SCHEME pScheme_button;
GFX_GOL_OBJ_SCHEME VB14_Scheme, VB10_Scheme;
GFX_GOL_OBJ_SCHEME VB12_Scheme;
GFX_GOL_OBJ_SCHEME selectScreenScheme; // select screen style scheme
GFX_GOL_OBJ_SCHEME pScheme_EditBox;
GFX_GOL_OBJ_HEADER *pObject;
APP_SCREEN_UPDATE_STATES update_display;      // partial or global
APP_SCREEN_STATES screenState;        // current state of main demo state machine

volatile DWORD tick_1ms = 0;          // one millisecond tick timer4 int
DWORD tick_10ms;                      // updated every 10 ms in timer4 int
DWORD previousTick = 0;
DWORD displayChangeTime;

unsigned int GAUGE_I2C1BRG;           // I2C timing depends on system clock 
unsigned int PWM_OC1RS;               // 4,000,000/24 = 166khz
unsigned int PWM_OC1R;                // 24/12 = 50 percent duty cycle
extern uint8_t myLongAddress[MY_ADDRESS_LENGTH];

PAGE_INFORMATION pageInformation;

char switch_duration = 0;             // turn off device if button press is held
BYTE UID48[6] = {0, 0, 0, 0, 0, 0};
static USB_HOST_MSD_DATA_LOGGER_DEMO_STATE demoState;
extern bool allowStateChange;         // prevent from re-entering ATTACH
bool USB_or_BLE_DEMO_or_RSSI_DEMO;                 // variable to inhibit clock switch from 16 to 4 mhz


/*******************************************************************************
 Demo Notes:
 *******************************************************************************

    1.  RSSI testing should be performed on a display that will work with
        partial updates.  The end device should not run from the main display.  
 *      The periodic time updater will interfere with the RSSI value.  When the 
 *      display is updating it is blocking the radio code from responding
 *      to the echo request  

 *******************************************************************************/

/*******************************************************************************
    Main
 *******************************************************************************/
int main(void) {

    GFX_GOL_MESSAGE msg; // GOL message structure to interact with GOL
    initialize_scheme();

    switchXTAL_32(32);   // switch over from 8mhz fpc to 32Mhz PRIPLL

    SYSTEM_InitializeBoard(); // Initialize board, drivers and graphics library
    if (reset_reason != RESET_INSTRUCTION)
        RTCCInit();               // Initialize the RTCC

    
    
    ///////   while(1) test code ///////////////////////////////////////////////
    //  testPixel();          // write pixels to screen to form a rectangle
    //  testSram();           // verify reads and writes ram
    //  testEEuid();          // verify reads and writes non volitale memory
    //  testBatteryStatus();  // verify I2C communication
    //  testUSB_HOST();       // create and write data to a file
    //  testUSB_DEVICE();     // simple echo test
    //  testClock_switching();// run 32mhz,doze on/off; 8 mhz doze on/off ; 32khz idle,sleep
    //  testCLICK_HR_Status();
    //  testCLICK_RFID_read_tag();
    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
   

    MCHP25AA02E48GetEUI48NodeAddress(UID48); // get unique id from ee memory
    redefine_permanent_address_definition();
    
    tick_10ms = tick;         // initialize for init radio

    // set the message callback function pointer
    GFX_GOL_MessageCallbackSet(APP_ObjectMessageCallback);
    // set the draw callback function pointer
    GFX_GOL_DrawCallbackSet(APP_ObjectDrawCallback);
    
    if(reset_reason != RESET_INSTRUCTION)
      screenState = SCREEN_DISPLAY_USER_POWER_DOWN;
    else
      screenState = SCREEN_DISPLAY_RADIO_SETUP;
    //screenState = SCREEN_DISPLAY_MAIN;   // skip radio init

    while (1) {
        // Draw GOL objects
        if (GFX_GOL_ObjectListDraw() == GFX_STATUS_SUCCESS) {
            transfer_pixel_data_from_sram_to_display();
            
            if(!USB_or_BLE_DEMO_or_RSSI_DEMO)power_mode(LOW_POWER);
                                    // low power is the default power level
                                    // set to high power for graphic operations
                                    // when objectListDraw is complete set the
                                    // power level to low power
          
           // Conserve_power_in_the_weeds();// something causes routine to hang
            TouchSenseButtonsMsg(&msg);      // update msg when key changes  
            if ((msg.uiEvent != EVENT_INVALID) &&
                    ((tick - displayChangeTime) > MESSAGE_DEAD_TIME)) // process 100 ticks after message
            {
                TranslateTouchpad(&msg);     // Translate the raw message
                GFX_GOL_ObjectMessage(&msg); // Process message
            }
        }

#ifdef PAN_COORDINATOR
        networkBeaconResponse();     // allow for remote devices to join network
#else
            handleRemoteCommands();      // need to add LOCK semaphore
        joinNetwork();               // attempt to join every 30 seconds
#endif
        heartbeat();                 // flash led every second
        monitorBatteryLevel();       // power battery is < 2.9v or < 6000 coloumbs
//        pollExternalPower();         // disable operation when external power detected
        userPowerDown();             // press and hold center button 3 seconds
        ClrWdt();                    // only one watch dog timer clear  
    }
}


void redefine_permanent_address_definition(void){
    BYTE i;
    for(i= 0; i<MY_ADDRESS_LENGTH ;i++){
           myLongAddress[i] = UID48[i];
    }
}

/****************************************************************************
  Function:
    void initialize_scheme( void )

  Description:
    This function initializes the fonts and colors used by the graphic library
  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:

 ***************************************************************************/
void initialize_scheme() {
    //////////////////////////////////////////
    GFX_GOL_SchemeTextColorSet(&VB12_Scheme, BLACK, WHITE, WHITE);
    GFX_GOL_SchemeColorSet(&VB12_Scheme, BLACK, WHITE, BLACK);
    GFX_GOL_SchemeBackgroundColorSet(&VB12_Scheme, WHITE);
    GFX_GOL_SchemeEmbossSet(&VB12_Scheme, BLACK, WHITE, 1);
    GFX_GOL_SchemeFontSet(&VB12_Scheme, (GFX_RESOURCE_HDR*) & Verdana_Bold_12);
    GFX_GOL_SchemeBackgroundTypeSet(&VB12_Scheme, WHITE);
    //////////////////////////////////////////   

    /////////////////////////////////////////
    GFX_GOL_SchemeTextColorSet(&VB14_Scheme, BLACK, WHITE, WHITE);
    GFX_GOL_SchemeColorSet(&VB14_Scheme, WHITE, BLACK, WHITE);
    GFX_GOL_SchemeBackgroundColorSet(&VB14_Scheme, WHITE);
    GFX_GOL_SchemeEmbossSet(&VB14_Scheme, BLACK, BLACK, 1);
    GFX_GOL_SchemeFontSet(&VB14_Scheme, (GFX_RESOURCE_HDR*) & Verdana_Bold_14);
    GFX_GOL_SchemeBackgroundTypeSet(&VB14_Scheme, WHITE);
    ////////////////////////////////////////////    

    ////////////////////////////////////////////
    GFX_GOL_SchemeTextColorSet(&VB10_Scheme, BLACK, WHITE, WHITE);
    GFX_GOL_SchemeColorSet(&VB10_Scheme, WHITE, BLACK, WHITE);
    GFX_GOL_SchemeBackgroundColorSet(&VB10_Scheme, WHITE);
    GFX_GOL_SchemeEmbossSet(&VB10_Scheme, BLACK, BLACK, 1);
    GFX_GOL_SchemeFontSet(&VB10_Scheme, (GFX_RESOURCE_HDR*) & Verdana_Bold_10);
    GFX_GOL_SchemeBackgroundTypeSet(&VB10_Scheme, WHITE);
    ////////////////////////////////////////////

    ////////////////////////////////////////////////
    // set style  push button scheme
    GFX_GOL_SchemeTextColorSet(&pScheme_button, BLACK, WHITE, WHITE);
    GFX_GOL_SchemeColorSet(&pScheme_button, WHITE, BLACK, WHITE);
    GFX_GOL_SchemeEmbossSet(&pScheme_button, BLACK, BLACK, 1);
    GFX_GOL_SchemeFontSet(&pScheme_button, (GFX_RESOURCE_HDR*) & Verdana_Bold_10);
    GFX_GOL_SchemeBackgroundColorSet(&pScheme_button, WHITE);
    GFX_GOL_SchemeBackgroundTypeSet(&pScheme_button, GFX_BACKGROUND_NONE);
    /////////////////////////////////////////////////

    ////////////////////////////////////////////////
    // set style  edit box scheme
    GFX_GOL_SchemeTextColorSet(&pScheme_EditBox, BLACK, WHITE, WHITE);
    GFX_GOL_SchemeColorSet(&pScheme_EditBox, WHITE, BLACK, WHITE);
    GFX_GOL_SchemeEmbossSet(&pScheme_EditBox, BLACK, BLACK, 1);
    GFX_GOL_SchemeFontSet(&pScheme_EditBox, (GFX_RESOURCE_HDR*) & Verdana_Bold_10);
    GFX_GOL_SchemeBackgroundTypeSet(&pScheme_EditBox, WHITE);
    /////////////////////////////////////////////////

}

/****************************************************************************
  Function:
    bool APP_ObjectMessageCallback(
                                GFX_GOL_TRANSLATED_ACTION objectMessage,
                                GFX_GOL_OBJ_HEADER *pObject,
                                GFX_GOL_MESSAGE *pMessage)

    Summary:
        Required application layer function. GFX_GOL_ObjectMessage()
        function calls this function each time a valid message for the
        object is received. See GFX_GOL_MessageCallbackSet()
        documentation in the Graphics Library Help file for details.

    Parameters:
        objectMessage - translated message for the object,
        pObject - pointer to the object,
        pMessage - pointer to the non-translated, raw GOL message

    Returns:
        true  - The message will be processed using default settings
        false - The message will not be processed by object layer
                and application may or may not process the message.

 ******************************************************************************/
bool APP_ObjectMessageCallback(
        GFX_GOL_TRANSLATED_ACTION objectMessage,
        GFX_GOL_OBJ_HEADER *pObject,
        GFX_GOL_MESSAGE *pMessage) {

    uint16_t objID;
    objID = GFX_GOL_ObjectIDGet(pObject);

    switch (screenState) {
        case SCREEN_BOARD_TEST:
            // No messages are processed in this state.
            break;

        case SCREEN_DISPLAY_RADIO_SETUP:
            // No messages are processed in this state.
            break;

        case SCREEN_DISPLAY_MAIN:
            // This state exists for the draw callback only.
            break;

        case SCREEN_MAIN:
            return ProcessMessageMain(objectMessage, pObject, pMessage);
            break;

        case SCREEN_DISPLAY_UTILITIES:
            // This state exists for the draw callback only.
            break;

        case SCREEN_UTILITIES:
            return ProcessMessageUtilities(objectMessage, pObject, pMessage);
            break;

        case SCREEN_DISPLAY_TIME:
            // This state exists for the draw callback only.
            break;
        case SCREEN_TIME:
            return ProcessMessageTIMESubMenu(objectMessage, pObject, pMessage);
            break;
        case SCREEN_DISPLAY_ADJUST_MONTH:
            // This state exists for the draw callback only.
            break;
        case SCREEN_ADJUST_MONTH:
            return ProcessMessageTime(objectMessage, pObject, pMessage);
            break;
        case SCREEN_DISPLAY_ADJUST_MINUTES:
            // This state exists for the draw callback only.
            break;
        case SCREEN_ADJUST_MINUTES:
            return ProcessMessageTime(objectMessage, pObject, pMessage);
            break;
        case SCREEN_DISPLAY_ADJUST_DAY:
            // This state exists for the draw callback only.
            break;
        case SCREEN_ADJUST_DAY:
            return ProcessMessageTime(objectMessage, pObject, pMessage);
            break;
        case SCREEN_DISPLAY_ADJUST_HOUR:
            // This state exists for the draw callback only.
            break;
        case SCREEN_ADJUST_HOUR:
            return ProcessMessageTime(objectMessage, pObject, pMessage);
            break;

        case SCREEN_DISPLAY_MKROE:
            // This state exists for the draw callback only.
            break;
        case SCREEN_MKROE:
#ifdef MKROE
            return ProcessMessage_DEMO_test(objectMessage, pObject, pMessage);
#else
            return ProcessMessageUSER1Menu(objectMessage, pObject, pMessage);
#endif
            break;

        case SCREEN_DISPLAY_BLE_LOOPBACK:
            // This state exists for the draw callback only.
            break;
        case SCREEN_BLE_LOOPBACK:
            //return ProcessMessage_BLE_LoopBack(objectMessage, pObject, pMessage);
            break;

        case SCREEN_DISPLAY_USB_MEMORY:
            // This state exists for the draw callback only.
            break;
        case SCREEN_USB_MEMORY:
            return ProcessMessageUSB_test(objectMessage, pObject, pMessage);
            break;
        case SCREEN_DISPLAY_GRAPH:
            // This state exists for the draw callback only.
            break;

        case SCREEN_GRAPH:
            return ProcessMessageGraph(objectMessage, pObject, pMessage);
            break;

        case SCREEN_DISPLAY_BATTERY:
            // This state exists for the draw callback only.
            break;
        case SCREEN_BATTERY:
            return ProcessMessage_BATTERY_test(objectMessage, pObject, pMessage);
            break;
        case SCREEN_POWER_DOWN_BATTERY_LOW:
            // This state exists for the draw callback only.
            break;
        case SCREEN_DISPLAY_USER_POWER_DOWN:
            // This state exists for the draw callback only.
            break;
        case SCREEN_POWER_DOWN:
            break;
        case SCREEN_EXTERNAL_POWER_APPLIED:
            break;
        default:
            break;
    }

    return (true);
}

/*****************************************************************************
 Function:
    bool APP_ObjectDrawCallback(void)

    Summary:
        Required application layer function. GFX_GOL_ObjectListDraw()
        function calls it each time when object layer drawing is completed.
        User drawing should be done here. See GFX_GOL_DrawCallbackSet()
        documentation in the Graphics Library Help file for details.

    Parameters:
        None.

    Returns:
        true  - When rendering control will be passed back to object layer
                rendering of the Graphics Library.
        false - When rendering control stays with the application.
                Do this when application needs more time to render.

 ******************************************************************************/
bool APP_ObjectDrawCallback(void) {

    static char previousSecond;
    static char every6thSecond;


    switch (screenState) {
        case SCREEN_BOARD_TEST: // 

            // and set the screanState = SCREEN_DISPLAY_MAIN
            break;
        case SCREEN_DISPLAY_RADIO_SETUP:
            initializeRadio(); // screenState is changed to SCREEN_DISPLAY_MAIN
            break;             // after the set up process
        case SCREEN_DISPLAY_MAIN:
            power_mode(HIGH_POWER);
            USB_or_BLE_DEMO_or_RSSI_DEMO = false;
            ShowScreenMain(); // resets the screen, creates buttons, adds text
            displayChangeTime = tick;
            monitorNetwork_tick10ms = tick;
            screenState = SCREEN_MAIN;
            update_display = GLOBAL_UPDATE;
            break;
        case SCREEN_MAIN:
            
#ifdef PAN_COORDINATOR

           monitorNetwork(); // update the connection table every 100 seconds
#endif
            RTCCProcessEvents();
            if (previousSecond != _time_str[11]) {
                
                previousSecond = _time_str[11];

                if (every6thSecond++ > 60) {
                     power_mode(HIGH_POWER);
                    every6thSecond = 0;
                    //  strcpy(pageInformation.instructions, _date_str);
                    //  strcat(pageInformation.instructions, "  ");
                    //  strcat(pageInformation.instructions, _time_str);
                    strcpy(pageInformation.instructions, _time_str);
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

                    sprintf((char *) &(pageInformation.instructions3), (char*) "Temp: %4d C", (display_Temperature + 6));
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_3);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions3);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    update_display = GLOBAL_UPDATE; //
                }
            }
            break;

        case SCREEN_DISPLAY_UTILITIES:
            ShowScreenUtilities();
            displayChangeTime = tick;
            screenState = SCREEN_UTILITIES;
            update_display = GLOBAL_UPDATE;
            break;
        case SCREEN_UTILITIES:
            // process message will change the screen state
            break;
        case SCREEN_DISPLAY_TIME:
            ShowScreenAdjustTime();
            displayChangeTime = tick;
            screenState = SCREEN_TIME;
            break;
        case SCREEN_TIME:
            break;
        case SCREEN_DISPLAY_ADJUST_MONTH:
            Utilities_currentControlSet = CONTROL_SET_MONTH;
            strcpy(pageInformation.title, "Set Month");
            ShowScreenTimeAdjust();
            displayChangeTime = tick;
            screenState = SCREEN_ADJUST_MONTH;
            break;
        case SCREEN_ADJUST_MONTH:
            break;
        case SCREEN_DISPLAY_ADJUST_MINUTES:
            Utilities_currentControlSet = CONTROL_SET_MINUTE;
            strcpy(pageInformation.title, "Set Minutes");
            ShowScreenTimeAdjust();
            displayChangeTime = tick;
            screenState = SCREEN_ADJUST_MINUTES;
            break;
        case SCREEN_ADJUST_MINUTES:
            break;
        case SCREEN_DISPLAY_ADJUST_DAY:
            Utilities_currentControlSet = CONTROL_SET_DAY;
            strcpy(pageInformation.title, "Set Day");
            ShowScreenTimeAdjust();
            displayChangeTime = tick;
            screenState = SCREEN_ADJUST_DAY;
            break;
        case SCREEN_ADJUST_DAY:
            break;
        case SCREEN_DISPLAY_ADJUST_HOUR:
            Utilities_currentControlSet = CONTROL_SET_HOUR;
            strcpy(pageInformation.title, "Set Hour");
            ShowScreenTimeAdjust();
            displayChangeTime = tick;
            screenState = SCREEN_ADJUST_HOUR;
            break;
        case SCREEN_ADJUST_HOUR:
            break;
        case SCREEN_DISPLAY_MKROE:
#ifdef MKROE
            ShowScreenClick_demo();    // create text objects
            displayChangeTime = tick;
            CLICK_DEMO_STATE = INITIALIZE;   //TODO need macro #ifdef
#else
            ShowScreenUser1Functions();
#endif
            screenState = SCREEN_MKROE;
            break;
        case SCREEN_MKROE:
#ifdef MKROE
     CLICK_demo();       
#else
            
#endif
            
        
            break;

        case SCREEN_DISPLAY_BLE_LOOPBACK:
#ifndef MKROE
            USB_or_BLE_DEMO_or_RSSI_DEMO = true;
            power_mode(HIGH_POWER);
            ShowScreenBLE_LoopBack();
            displayChangeTime = tick;
            update_display = GLOBAL_UPDATE;
            BLE_state_initilize();
            screenState = SCREEN_BLE_LOOPBACK;
#else
            screenState = SCREEN_DISPLAY_MAIN;
#endif
            break;
        case SCREEN_BLE_LOOPBACK:
#ifndef MKROE

            Bluetooth_LoopBack();
#endif
            break;

            ////////////////////////////// usb    //////////////////////////////////////////

        case SCREEN_DISPLAY_USB_MEMORY:
            power_mode(HIGH_POWER);
            USB_or_BLE_DEMO_or_RSSI_DEMO = true;
            ShowScreenUSBMedia_INIT();
            update_display = GLOBAL_UPDATE;
            USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_ON;
            FILEIO_Initialize();
            USBHostInit(0);      //Initialize the stack
            displayChangeTime = tick;
            screenState = SCREEN_USB_MEMORY;
            demoState = WAITING_FOR_ATTACH;
            allowStateChange = true;
            break;
        case SCREEN_USB_MEMORY:
            APP_HostMSDDataLoggerTasks_demo();
            break;

            /////////////////////////////////// graph ////////////////////////////
        case SCREEN_DISPLAY_GRAPH:
            USB_or_BLE_DEMO_or_RSSI_DEMO = true;
            ShowScreenGraph();
            displayChangeTime = tick;
            screenState = SCREEN_GRAPH;
            pingDelayTime = tick;
            break;
        case SCREEN_GRAPH:
            if(echo_rssi())      // periodically ping end device
                UpdateGraph();   // update the graph when data is available
            break;
        case SCREEN_DISPLAY_BATTERY:
            ShowScreenBatteryDiagnostic();
            displayChangeTime = tick;
            tick_10ms = tick;  // initialize timer
            screenState = SCREEN_BATTERY;
            update_display = GLOBAL_UPDATE;
            break;
        case SCREEN_BATTERY:
            if (tick > (tick_10ms + 1000)) {
                tick_10ms = tick;
                power_mode(HIGH_POWER);
                Battery_Diagnostic();
            }
            break;
        case SCREEN_POWER_DOWN_BATTERY_LOW:
            PowerDownSequence();
            break;
        case SCREEN_DISPLAY_USER_POWER_DOWN:
            update_display = GLOBAL_UPDATE;
            power_mode(HIGH_POWER);
#ifdef PAN_COORDINATOR
            if(radioInitialized)
               sendPowerDownMessage();
#endif
            ShowScreenUserPowerDown();
            screenState = SCREEN_POWER_DOWN;
            displayChangeTime = tick;
            break;
        case SCREEN_POWER_DOWN:
            powerDown();
            break;
        case SCREEN_EXTERNAL_POWER_APPLIED:
            external_power_applied();
            break;
    }
    return (true);
}

/****************************************************************************
  Function:
    void TranslateTouchpad( GFX_GOL_MESSAGE* msg )

  Description:
    This function translates the keypad message into the appropriate message
    for the screen, based on what screen is currently displayed.

  Precondition:
    None

  Parameters:
    GFX_GOL_MESSAGE* msg    - The current message information

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void TranslateTouchpad(GFX_GOL_MESSAGE* pMsg) {
    switch (screenState) {
        case SCREEN_BOARD_TEST:
            // Ignore all touchpad messages.
            pMsg->uiEvent = EVENT_INVALID;
            break;
        case SCREEN_DISPLAY_RADIO_SETUP:
            // Ignore all touchpad messages.
            pMsg->uiEvent = EVENT_INVALID;
            break;
        case SCREEN_DISPLAY_MAIN:
            // No translation needed.
            // Ignore all touchpad messages.
            pMsg->uiEvent = EVENT_INVALID;
            break;
        case SCREEN_MAIN:
            TranslateMessageGeneric(pMsg);
            break;

        case SCREEN_DISPLAY_UTILITIES:
            // No translation needed.
            break;
        case SCREEN_UTILITIES:
            TranslateMessageGeneric(pMsg);
            break;
        case SCREEN_DISPLAY_TIME:
            // No translation needed.
            break;
        case SCREEN_TIME:
            TranslateMessageGeneric(pMsg);
            break;
        case SCREEN_DISPLAY_ADJUST_MONTH:
            // No translation needed.
            break;
        case SCREEN_ADJUST_MONTH:
            TranslateMessageSoftKeys(pMsg);
            break;
        case SCREEN_DISPLAY_ADJUST_MINUTES:
            // No translation needed.
            break;
        case SCREEN_ADJUST_MINUTES:
            TranslateMessageSoftKeys(pMsg);
            break;
        case SCREEN_DISPLAY_ADJUST_DAY:
            // No translation needed.
            break;
        case SCREEN_ADJUST_DAY:
            TranslateMessageSoftKeys(pMsg);
            break;
        case SCREEN_DISPLAY_ADJUST_HOUR:
            // No translation needed.
            break;
        case SCREEN_ADJUST_HOUR:
            TranslateMessageSoftKeys(pMsg);
            break;

        case SCREEN_DISPLAY_MKROE:
            // No translation needed.
            break;
        case SCREEN_MKROE:
#ifdef MKROE
            TranslateMessageSoftKeys(pMsg);
#else
            TranslateMessageGeneric(pMsg);
#endif
            break;
        case SCREEN_DISPLAY_BLE_LOOPBACK:
            // No translation needed.
            break;
        case SCREEN_BLE_LOOPBACK:
            TranslateMessageSoftKeys(pMsg);
            break;

        case SCREEN_DISPLAY_USB_MEMORY:
            // No translation needed.
            pMsg->uiEvent = EVENT_INVALID;
            break;
        case SCREEN_USB_MEMORY:
            // Ignore all touchpad messages.
            TranslateMessageSoftKeys(pMsg);
            break;

        case SCREEN_DISPLAY_GRAPH:
            // No translation needed.
            break;
        case SCREEN_GRAPH:
            TranslateMessageGraph(pMsg);
            break;
        case SCREEN_DISPLAY_BATTERY:
            // No translation needed.
            pMsg->uiEvent = EVENT_INVALID;
            break;
        case SCREEN_BATTERY:
            TranslateMessageSoftKeys(pMsg);
            break;
        case SCREEN_POWER_DOWN_BATTERY_LOW:
            // No translation needed.
            pMsg->uiEvent = EVENT_INVALID;
            break;
        case SCREEN_DISPLAY_USER_POWER_DOWN:

            break;
        case SCREEN_POWER_DOWN:
            break;
        case SCREEN_EXTERNAL_POWER_APPLIED:
            // No translation needed.
            break;
    }
}

/****************************************************************************
  Function:
    void TouchSenseButtonsMsg(GFX_GOL_MESSAGE* msg)

  Description:
    This routine creates appropriate messages for button presses & releases.

  Precondition:
    None

  Parameters:
    GFX_GOL_MESSAGE *msg    - The new system message

  Returns:
    None

  Remarks:
    Look for a change in the state of a button.  If there
    has been a state change, we send a message.  Note that this means that
    repetitive messages will not be generated if a touch pad is held for a
    long period of time.
 ***************************************************************************/
void TouchSenseButtonsMsg(GFX_GOL_MESSAGE* msg) {
    msg->uiEvent = EVENT_INVALID;
    msg->type = TYPE_KEYBOARD;

    if (buttons.RIGHT_BTN != buttonsPrevious.RIGHT_BTN) {
        msg->uiEvent = EVENT_KEYSCAN;
        msg->param1 = ID_TOUCH_PAD;
        if (buttons.RIGHT_BTN) {
            msg->param2 = SCAN_RIGHT_PRESSED; //
        } else {
            msg->param2 = SCAN_RIGHT_RELEASED; //
        }
        buttonsPrevious.RIGHT_BTN = buttons.RIGHT_BTN;
        return;
    }

    if (buttons.LEFT_BTN != buttonsPrevious.LEFT_BTN) {
        msg->uiEvent = EVENT_KEYSCAN;
        msg->param1 = ID_TOUCH_PAD;
        if (buttons.LEFT_BTN) {
            msg->param2 = SCAN_LEFT_PRESSED; //
        } else {
            msg->param2 = SCAN_LEFT_RELEASED; //
        }
        buttonsPrevious.LEFT_BTN = buttons.LEFT_BTN;
        return;
    }

    if (buttons.CENTER_BTN != buttonsPrevious.CENTER_BTN) {
        msg->uiEvent = EVENT_KEYSCAN;
        msg->param1 = ID_TOUCH_PAD;
        if (buttons.CENTER_BTN) {
            msg->param2 = SCAN_CR_PRESSED;
        } else {
            msg->param2 = SCAN_CR_RELEASED;
        }
        buttonsPrevious.CENTER_BTN = buttons.CENTER_BTN;
        return;
    }
}

/****************************************************************************
  Function:
    void ShowScreenMain( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    the main menu screen.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenMain(void) {
    RTCCProcessEvents();

    strcpy(pageInformation.title, " MAIN ");
    strcpy(pageInformation.buttonTextTopLeft, "Time");
    strcpy(pageInformation.buttonTextBottomLeft, " BLE ");
    strcpy(pageInformation.buttonTextTopRight, "Utilities");
    strcpy(pageInformation.buttonTextBottomRight, " MKROE");
    strcpy(pageInformation.buttonTextThirdrow, "");

    pageInformation.buttonBitmapTopLeft = NULL;
    pageInformation.buttonBitmapBottomLeft = NULL;
    pageInformation.buttonBitmapTopRight = NULL;
    pageInformation.buttonBitmapBottomRight = NULL;

    strcpy(pageInformation.instructions, _date_str);
    strcat(pageInformation.instructions, "  ");
    strcat(pageInformation.instructions, _time_str);

    ShowScreenGeneric();


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 70, 127, 70 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_LEFT,
            &VB10_Scheme);
}

/****************************************************************************
  Function:
    void TranslateMessageGeneric( GFX_GOL_MESSAGE* msg )

  Description:
    This function translates the keypad message into the appropriate message
    for a generic, 3-button menu screen.  It can be used as the message
    processing function for any generic screen.

  Precondition:
    None

  Parameters:
    GFX_GOL_MESSAGE * msg    - The current message information

  Returns:
    None

  Remarks:
 * display is globally updates the screen on valid key press GLOBAL_UPDATE   
    None
 ***************************************************************************/
void TranslateMessageGeneric(GFX_GOL_MESSAGE * pMsg) {

    if ((pMsg->uiEvent == EVENT_KEYSCAN) &&
            (pMsg->type == TYPE_KEYBOARD) &&
            (pMsg->param1 == ID_TOUCH_PAD)) {
        switch (pMsg->param2) {
            case SCAN_UP_PRESSED:
                break;
            case SCAN_DOWN_PRESSED:
                break;
                // move un press current button and press the next button that is
                // to be selected
            case SCAN_RIGHT_PRESSED:
                power_mode(HIGH_POWER);
                if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                }
                update_display = PARTIAL_UPDATE;  // moving selection this can be a fast update
                
                break;

            case SCAN_LEFT_PRESSED:
                power_mode(HIGH_POWER);
                if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    GFX_GOL_ObjectStateClear((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_DRAW_STATE);
                    GFX_GOL_ObjectStateSet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED | GFX_GOL_BUTTON_DRAW_STATE);
                }

                update_display = PARTIAL_UPDATE;  // moving selection this can be a fast update
                break;

            case SCAN_UP_RELEASED:
                break;

            case SCAN_DOWN_RELEASED:
                break;

            case SCAN_RIGHT_RELEASED:
                break;

            case SCAN_LEFT_RELEASED:
                break;

            case SCAN_CR_PRESSED:
                power_mode(HIGH_POWER);
                if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_TOP_LEFT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_BOTTOM_LEFT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_TOP_RIGHT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_BOTTOM_RIGHT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_THIRDROW), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_THIRDROW;
                }
                update_display = GLOBAL_UPDATE;
                break;

            case SCAN_CR_RELEASED:
                if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_TOP_LEFT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_LEFT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_BOTTOM_LEFT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_TOP_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_TOP_RIGHT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_BOTTOM_RIGHT), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_BOTTOM_RIGHT;
                } else if (GFX_GOL_ObjectStateGet((GFX_GOL_BUTTON *) GFX_GOL_ObjectFind(ID_THIRDROW), GFX_GOL_BUTTON_ACTION_PRESSED)) {
                    pMsg->param1 = ID_THIRDROW;
                }
                break;

        }
    }
}

/****************************************************************************
  Function:
    void transfer_pixel_data_from_sram_to_display( void )

  Description:
    This function initiates the transfer of data from the sram to the display.
    the global variable is set to indicate "local" or "full update" 
    - COG_FPL G1_Aurora_Mb and G2_Aurora_Mb allow "local" update
    - COG_FPL G2_Aurora_Ma only allow full update.  EPD_interface.c will only 
      allow for "Full update" regardless of the type of update specified.
  Precondition:
     should be operating with DOZEN = DISABLE_DOZE

  Parameters:
    update GLOBAL_UPDATE, PARTIAL_UPDATE or default NO_UPDATE

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void transfer_pixel_data_from_sram_to_display(void) {

    switch (update_display) {
        case NO_UPDATE:
            break;
        case GLOBAL_UPDATE:
            DRV_GFX_UpdateBiStableScreen(true,false);
            update_display = NO_UPDATE;
            break;
        case PARTIAL_UPDATE:
            DRV_GFX_UpdateBiStableScreen(false,false);
            update_display = NO_UPDATE;
            break;
        default:
            update_display = NO_UPDATE;
            break;
    }
}

/****************************************************************************
  Function:
    void switchXTAL_32(char speed) 

  Description:
    This function switches from the internal 8 mhz frc clock to the external 
    8 mhz clock using the PLL to operate at 32 mHz.
  Precondition:
 *  external clock must be populated

  Parameters:
     char speed:  32 to run at 32mhz pll
   
  Return Values:
    none
    

  Remarks:
    tick timer,output compare pwm, I2C peripherals timing registers updated
    todo need usb flag to prohibit using usb if using internal FPL clock 
 ***************************************************************************/
void switchXTAL_32(char speed) {

    int delay = 0;
    TIMER_PERIOD_10MS = 5000; // tick timer period
    GAUGE_I2C1BRG = 78;       // I2C clock for gas gauge communication
    PWM_OC1RS = 24;           // pwm for display
    PWM_OC1R = 12;

    if (speed == 32) {
        OSCCONbits.NOSC = 0b011;
        __builtin_write_OSCCONH(OSCCON | 0x03);
        __builtin_write_OSCCONL(OSCCON | 0x01);  //03
        OSCCONbits.OSWEN = 1;
        while (delay < 30000) {
            if (OSCCONbits.OSWEN == 0)
                if (OSCCONbits.LOCK == 1) {
                    TIMER_PERIOD_10MS = 20000;
                    GAUGE_I2C1BRG = 157;
                    PWM_OC1RS = 96;
                    PWM_OC1R = 48;
                    return;   // return if success new frequency is 32 mHz  
                }             // external oscillator
            delay++;
        }

        OSCCONbits.OSWEN = 0;
        OSCCONbits.NOSC = 0b001;
        __builtin_write_OSCCONH(OSCCON | 0x01);
        __builtin_write_OSCCONL(OSCCON | 0x01);
        OSCCONbits.OSWEN = 1;
        delay = 0;
        while (delay < 30000) {
            if (OSCCONbits.OSWEN == 0)
                if (OSCCONbits.LOCK == 1) {
                    TIMER_PERIOD_10MS = 20000;
                    GAUGE_I2C1BRG = 157;
                    PWM_OC1RS = 96;
                    PWM_OC1R = 48;
                    return; // return using internal FRC running pll at 32MHZ
                }
            delay++;
        }
    }
}



/////////////////////////////// MAIN MENU //////////////////////////////////////

/****************************************************************************
  Function:
    WORD ProcessMessageMain( WORD translatedMsg, 
                            GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for the main introduction screen.
    It allows the user to select a Time/Date, utilities, Race Setup or Timer.

  Precondition:
    Call ShowScreenMain() prior to using this function to display the correct
    screen and create required objects.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    The global screenState is updated to proceed to the user's selection.
 ***************************************************************************/
bool ProcessMessageMain(WORD translatedMsg,
        GFX_GOL_OBJ_HEADER* pObj,
        GFX_GOL_MESSAGE* pMsg) {
    uint16_t controlID;

    if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
        controlID = GFX_GOL_ObjectIDGet(pObj);

        switch (controlID) {
            case ID_MAIN_SET_DATE_TIME: //ID_TOP_LEFT
                screenState = SCREEN_DISPLAY_TIME;
                break;

            case ID_MAIN_USER1:        //ID_BOTTOM_RIGHT
                screenState = SCREEN_DISPLAY_MKROE;
                break;

            case ID_MAIN_BLE:          //ID_BOTTOM_LEFT
                screenState = SCREEN_DISPLAY_BLE_LOOPBACK;
                break;

            case ID_MAIN_UTILITIES:    //ID_TOP_RIGHT
                screenState = SCREEN_DISPLAY_UTILITIES;
                break;
        }
    }
    return false; // We already called the default handler.
}

/****************************************************************************
  Function:
    void heartbeat( void )

  Description:
    This function is called in the main forever loop.  this helps indicate
    the health by showing a periodic led pulse.  The led also indicates TRAP 
    errors
 
  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    if the  micro is too busy or if there is blocking code, the led will 
    not blink with a frequency of 1 second.
 ***************************************************************************/
void heartbeat(void) {
    static DWORD timetick_Ms = 0;
    DWORD elapseTime;
    static DWORD oneSecondPeriod = 0;
    if (timetick_Ms != tick) {
        elapseTime = tick - timetick_Ms;
        timetick_Ms = tick;
        oneSecondPeriod += elapseTime;
        if (oneSecondPeriod > 100) {
            oneSecondPeriod = 0;
            status_1_pin ^= 1; // toggle status led every second
        }
    }
}

/****************************************************************************
  Function:
    void handleRemoteCommands( void )

  Description:
    This function is polled and responds to commands from the Coordinator. This
    function will update the sceenstate.
  Precondition:
    must have the same PAN id and frequency to respond to broadcast message

  Parameters:
    None

  Returns:
    None

  Remarks:
 *  make sure that this function is not invoked when the remote device is already
 *  engaged in dialog with the Coordinator.  This function will intercept messages
 *  that could be intended by another function.
 ***************************************************************************/
void handleRemoteCommands(void) {
    BYTE pktCmd;
    if (Network)//  LOCK condition to keep from grabbing messages
        if (MiApp_MessageAvailable()) {
            pktCmd = rxMessage.Payload[0];
            MiApp_DiscardMessage();
            if (pktCmd == POWER_DOWN) {
                screenState = SCREEN_DISPLAY_USER_POWER_DOWN;
            } else if (pktCmd == NETWORK_RSD_STATUS_PKT) {
                respondToNetworkStatusRequest(UID48[5]);
            }
        }
}

/****************************************************************************
  Function:
    void networkBeaconResponse( void )

  Description:
    This function is polled and responds to Beacons and network join requests.
    This function will allow for remote devices to join the network.
  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
 *  make sure that this function is not invoked when the Coordinator is already
 * engaged in dialog with end devices.  This function will intercept messages
 * that could be intended by another function.  

 ***************************************************************************/
void networkBeaconResponse(void) {
    if (radioInitialized)
        // do not process messages if other parts of the code are expecting
        // messages.
        if ((screenState != SCREEN_BOARD_TEST) &&
                (screenState != SCREEN_GRAPH) &&
                (networkScanStart == FALSE))

            if (MiApp_MessageAvailable())
                MiApp_DiscardMessage(); // new connections
}

/****************************************************************************
  Function:
    bool USB_ApplicationEventHandler( uint8_t address, USB_EVENT event,
                void *data, uint32_t size )

  Summary:
    This is the application event handler.  It is called when the stack has
    an event that needs to be handled by the application layer rather than
    by the client driver.

  Description:
    This is the application event handler.  It is called when the stack has
    an event that needs to be handled by the application layer rather than
    by the client driver.  If the application is able to handle the event, it
    returns true.  Otherwise, it returns false.

  Precondition:
    None

  Parameters:
    uint8_t address    - Address of device where event occurred
    USB_EVENT event - Identifies the event that occurred
    void *data      - Pointer to event-specific data
    uint32_t size      - Size of the event-specific data

  Return Values:
    true    - The event was handled
    false   - The event was not handled

  Remarks:
    The application may also implement an event handling routine if it
    requires knowledge of events.  To do so, it must implement a routine that
    matches this function signature and define the USB_HOST_APP_EVENT_HANDLER
    macro as the name of that function.
 ***************************************************************************/
bool USB_ApplicationEventHandler(uint8_t address,
        USB_EVENT event,
        void *data,
        uint32_t size) {
    switch ((int) event) {
        case EVENT_VBUS_REQUEST_POWER:
            // The data pointer points to a byte that represents the amount of power
            // requested in mA, divided by two.  If the device wants too much power,
            // we reject it.
            return true;

        case EVENT_VBUS_RELEASE_POWER:
            //This means that the device was removed
            APP_ReleasePower();
            return true;
            break;

            /* Here are various other events that a user might want to handle
             * or be aware of.  In this demo we are not handling them so we
             * will just return true to allow the stack to move on from the error.
             */
        case EVENT_HUB_ATTACH:
        case EVENT_UNSUPPORTED_DEVICE:
        case EVENT_CANNOT_ENUMERATE:
        case EVENT_CLIENT_INIT_ERROR:
        case EVENT_OUT_OF_MEMORY:
        case EVENT_UNSPECIFIED_ERROR: // This should never be generated.
            return true;
            break;

        case EVENT_MSD_ATTACH:
            APP_MountDrive(address);
            break;

        default:
            break;
    }

    return false;
}

bool USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event,
        void *pdata,
        uint16_t size) {
    switch ((int) event) {
        case EVENT_TRANSFER:
            break;

        case EVENT_SOF:
            /* We are using the SOF as a timer to time the LED indicator.  Call
             * the LED update function here. */
            // APP_LEDUpdateUSBStatus();
            break;

        case EVENT_SUSPEND:
            /* Update the LED status for the suspend event. */
            //   APP_LEDUpdateUSBStatus();
            break;

        case EVENT_RESUME:
            /* Update the LED status for the resume event. */
            // APP_LEDUpdateUSBStatus();
            break;

        case EVENT_CONFIGURED:
            /* When the device is configured, we can (re)initialize the 
             * demo code. */
            APP_DeviceCDCBasicDemoInitialize();
            break;

        case EVENT_SET_DESCRIPTOR:
            break;

        case EVENT_EP0_REQUEST:
            /* We have received a non-standard USB request.  The HID driver
             * needs to check to see if the request was for it. */
            USBCheckCDCRequest();
            break;

        case EVENT_BUS_ERROR:
            break;

        case EVENT_TRANSFER_TERMINATED:
            break;

        default:
            break;
    }
    return true;
}







void testUSB_HOST(void) {
#ifdef USB_HOST
    RCONbits.SWDTEN = 0; // disable watch dog timer
    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_ON;
    FILEIO_Initialize();
    USBHostInit(0);      //Initialize the stack
    APP_HostMSDDataLoggerInitialize();
    while (1) {
        USBHostTasks();
        USBHostMSDTasks();

        //Application specific tasks
        APP_HostMSDDataLoggerTasks();
    }//end while
#endif
}

void testUSB_DEVICE(void) {
#ifndef USB_HOST
    RCONbits.SWDTEN = 0; // disable watch dog timer
    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF; // make sure it is off
    USBDeviceInit();
    USBDeviceAttach();

    while (1) {

        /* If the USB device isn't configured yet, we can't really do anything
         * else since we don't have a host to talk to.  So jump back to the
         * top of the while loop. */
        if (USBGetDeviceState() < CONFIGURED_STATE) {
            /* Jump back to the top of the while loop. */
            continue;
        }

        /* If we are currently suspended, then we need to see if we need to
         * issue a remote wakeup.  In either case, we shouldn't process any
         * keyboard commands since we aren't currently communicating to the host
         * thus just continue back to the start of the while loop. */
        if (USBIsDeviceSuspended() == true) {
            /* Jump back to the top of the while loop. */
            continue;
        }

        //Application specific tasks
        APP_DeviceCDCBasicDemoTasks();

    }//end while

#endif
}//end testUSB_DEVICE



/****************************************************************************
  Function:
    void testBatteryStatus( void )

  Summary:
    This routine verifies the 12c communications with the battery gas gauge.

  Description:
     The status is requested from the gas gauge every 2 seconds.  The expected 
 * return values are 0b10000000) or 0b10000001.  If the return value is valid the 
 * led will flash at a 1 second period.  Otherwise the led will flash at a longer
 * period to indicate failure.

  Precondition:
     I2C channel 3 must be set up

  Parameters:
     none

  Return Values:
     none

  Remarks:
 this is a forever test loop, wdt must be disabled
 ***************************************************************************/
void testBatteryStatus(void) {
    BYTE status;
    int rate = 200;
    DWORD displayTick_10ms = 0;
    RCONbits.SWDTEN = 0; // disable watch dog timer
    while (1) {
        if ((tick - displayTick_10ms) > rate) {
            displayTick_10ms = tick;
            status = Battery_status();
            if (status == 0b00000000) {
                status_1_pin ^= 1;
                rate = 200;
            } else if (status == 0b11111111) {
                status_1_pin ^= 1;
                rate = 300;
            } else if ((status == 0b10000000) || (status == 0b10000001)) {
                status_1_pin ^= 1;
                rate = 50;  // period is 1 second
            } else {
                status_1_pin ^= 1;
                rate = 400;
            }
        }
    }
}

/****************************************************************************
  Function:
    void testSram(void)

  Description:
    This function writes and reads incrementing data  at  address location 10 every 2
 * seconds. The data is read back and verified, if there is a  match  the led 
 * is turned on.


  Precondition:
    spi 2 must be configured, disable wdt

  Parameters:
     none

  Returns:
 no return this is a forever while loop

  Remarks:
     
 ***************************************************************************/
void testSram() {
    uint8_t write_data, read_data;
    DWORD displayTick_10ms = 0;

    RCONbits.SWDTEN = 0; // disable watch dog timer
    displayTick_10ms = tick;
    write_data = 255;
    while (1) {
        if ((tick - displayTick_10ms) > 200) {
            displayTick_10ms = tick;
            write_data++;
            DRV_RAM_23K256_ByteWrite((uint16_t) 10, write_data);
            read_data = DRV_RAM_23K256_ByteRead((uint16_t) 10);

            if (read_data == write_data) {
                status_1_pin = 1;
            } else {
                status_1_pin = 0;
            }
        }
    }
}

/****************************************************************************
  Function:
    void BYTE testEEuid(void)

  Description:
    This function writes and reads 0xA5 at the specified address location 55 every 2
 * seconds. kThe data is read back and verified, if there is a  match  the led 
 * is turned on.


  Precondition:
    spi 2 must be configured

  Parameters:
    location:  Address from 00 to BF

  Returns:
 no return this is a forever while loop

  Remarks:
     
 ***************************************************************************/
void testEEuid(void) {

    DWORD displayTick_10ms = 0;
    RCONbits.SWDTEN = 0; // disable watch dog timer

    while (1) {
        if ((tick - displayTick_10ms) > 200) {
            displayTick_10ms = tick;

            if (write_read_EE_test(55))
                status_1_pin = 1;
            else
                status_1_pin = 0;
        }
    }
}

/****************************************************************************
  Function:
    BYTE write_read_EE_test(BYTE location)

  Description:
    This function writes 0xA5 at the specified address location.  Data is
 * read back from the same location and compared to 0xA5.


  Precondition:
    spi 2 must be configured

  Parameters:
    location:  Address from 00 to BF

  Returns:
   TRUE if test data written matches test data read; False otherwise

  Remarks:
     cannot write to the protected upper 1/4 of memory
 
     The device is selected by pulling CS low. The 8-bit
     READ instruction is transmitted to the 25AA02UID
     followed by an 8-bit address.
     After the correct READ instruction and address are sent,
     the data stored in the memory at the selected address
     is shifted out on the SO pin. Data stored in the memory
     at the next address can be read sequentially by
     continuing to provide clock pulses to the slave. The
     internal Address Pointer automatically increments to
     the next higher address after each byte of data is
     shifted out. When the highest address is reached
     (FFh), the address counter rolls over to address 00h,
     allowing the read cycle to be continued indefinitely. The
     read operation is terminated by raising the CS pin
 ***************************************************************************/
BOOL write_read_EE_test(BYTE location) {
    BYTE data_in, data_out;
    data_in = 0xA5;
    DRV_NVM_MCHP25AA02E48_Write(location, &data_in, 1);
    DRV_NVM_MCHP25AA02E48_Read(location, &data_out, 1);

    if (data_in == data_out) return TRUE;
    return FALSE;
}

/****************************************************************************
  Function:
    testPixel()

  Description:
    This function writes pixels in the shape of a rectangle toggling
 * between white and black 


  Precondition:
     hardware and graphic library must be initialized.  disable the wdt

  Parameters
     none

  Returns:
     none

  Remarks:
     
 ***************************************************************************/
void testPixel() {
    uint16_t x, y;
    DWORD displayTick_10ms = 0;

    RCONbits.SWDTEN = 0; // disable watch dog timer
    GFX_ColorSet(WHITE);
    GFX_ScreenClear();
    //DRV_GFX_UpdateBiStableScreen(true);
    displayTick_10ms = tick;

    while (1) {
        if ((tick - displayTick_10ms) == 100) {
            GFX_ColorSet(BLACK);
            status_1_pin = 1;
            for (y = 10; y < 30; y++)
                for (x = 5; x < 80; x++)
                    GFX_PixelPut(x, y);
            //DRV_GFX_UpdateBiStableScreen(true);
        } else if ((tick - displayTick_10ms) == 3000) {
            GFX_ColorSet(WHITE);
            status_1_pin = 0;
            for (y = 10; y < 30; y++)
                for (x = 5; x < 80; x++)
                    GFX_PixelPut(x, y);
          //  DRV_GFX_UpdateBiStableScreen(true);
        } else if ((tick - displayTick_10ms) > 5000) {
            displayTick_10ms = tick;
        }
    }
}




/****************************************************************************
  Function:
    testClock_switching()

  Description:
    This function loops through different clock sources and blinks an led for 
 * a period of time allowing for current measurement.  It goes into IDLE keeping
 * the led on to indicate the state.  The watchdog timer expires and proceeds to
 * turn off the led and enters sleep.  Again the watchdog timer expires and
 * the cycle repeats.  Pressing the center button will cause an CN interrupt and
 * will exit the IDLE or SLEEP
 * 
 * switches the clock source from:
 *            32 Mhz doze disabled consumes ~ 40 ma   50 flashes
 *             8 Mhz doze disabled consumes ~ 12 ma   25 flashes
 *            32 Mhz doze enabled consumes  ~ 22 ma   15 flashes  
 *             8 Mhz doze enabled consumes  ~ 5 ma    10 flashes
 *            32 Khz doze disabled consumes ~ 5 ma    20 flashes
 *               IDLE              consumes ~ 2.5 ma  on continuous
 *               SLEEP             consumes ~ 1.5 ma  off


  Precondition:
      enable wdt, this wake the processor from IDLE and SLEEP

  Parameters
     none

  Returns:
     none

  Remarks:
 *  clock switching clears the watch dog timer
     
 ******************************************************************************/
void testClock_switching(void) {

    unsigned int delay = 0;
    unsigned int numberOfFlashes;


    CLKDIVbits.DOZE = 0b111;      // 1:128;
    CLKDIVbits.ROI = 0; // do not return to full-speed CPU operation on interrupts
    
    
    while (1) {

        // consumes about 40 ma
        __builtin_disi(0x3FFF);                  // disable interrupts
        __builtin_write_OSCCONH(OSCCON | 0x03);  // Primary Oscillator w/ PLL module 
        __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 32Mhz
        __builtin_disi(0x0000);                  // enable interrupts

        delay = 0;
        while (delay < 30000) {
            if (OSCCONbits.OSWEN == 0)
                if (OSCCONbits.LOCK == 1) {
                    break;    
                }
            delay++;
        }
        CLKDIVbits.DOZEN = DISABLE_DOZE;
        numberOfFlashes = 50;
        if(delay > 20000)
        delay  = 0;   // this is for break point evaluation
        
        while (numberOfFlashes != 0) {
            if (delay++ > 60000) {
                ClrWdt();                       // make sure wdt does not time out
                delay = 0;
                status_1_pin ^= 1;
                numberOfFlashes--;
            }
        }
        

        // consumes about 10 ma
        __builtin_disi(0x3FFF);                 // disable interrupts
        __builtin_write_OSCCONH(OSCCON | 0x02); // 010 = Primary Oscillator XT
        __builtin_write_OSCCONL(OSCCON | 0x01); // fosc = 8 Mhz
        __builtin_disi(0x0000);                 // enable interrupts
                                                // watchdog timer is cleared
                                                // on clock switch
        delay  = 0;
        while (delay++ < 30000) {
            if (OSCCONbits.OSWEN == 0)
                break;      
        }
        numberOfFlashes = 25;
        if(delay > 20000)
        delay  = 0;  // this is for break point evaluation
        
        while (numberOfFlashes != 0) {
            if (delay++ > 60000) {
                delay = 0;
                status_1_pin ^= 1;
                numberOfFlashes--;
            }
        }
        
        ClrWdt();// this may be needed depending on timeout value
         // consumes about 22 ma
        CLKDIVbits.DOZEN = ENABLE_DOZE;
        __builtin_disi(0x3FFF);                  // disable interrupts
        __builtin_write_OSCCONH(OSCCON | 0x03);  // Primary Oscillator w/ PLL module 
        __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 32Mhz
        __builtin_disi(0x0000);                  // enable interrupts

        delay = 0;
        while (delay < 30000) {
            if (OSCCONbits.OSWEN == 0)
                if (OSCCONbits.LOCK == 1) {
                    break;    //breaks
                }
            delay++;
        }
        numberOfFlashes = 15;
        if(delay > 20000)
        delay  = 0;                        // this is for break point evaluation
        
        while (numberOfFlashes != 0) {
            if (delay++ > 6000) {
                ClrWdt();// this may be needed depending on timeout value
                delay = 0;
                status_1_pin ^= 1;
                numberOfFlashes--;
            }
        }
        
        
       

        // consumes about 5 ma
        __builtin_disi(0x3FFF);                 // disable interrupts
        __builtin_write_OSCCONH(OSCCON | 0x02); // 010 = Primary Oscillator XT
        __builtin_write_OSCCONL(OSCCON | 0x01); // fosc = 8 Mhz
        __builtin_disi(0x0000);                 // enable interrupts

        delay  = 0;
        while (delay++ < 30000) {
            if (OSCCONbits.OSWEN == 0)
                    break;     
        }
        numberOfFlashes = 10;
        if(delay > 20000)
        delay  = 0;   // breaks here
        
        while (numberOfFlashes != 0) {
            if (delay++ > 6000) {
                ClrWdt();// this may be needed depending on timeout value
                delay = 0;
                status_1_pin ^= 1;
                numberOfFlashes--;
            }
        }
        
        CLKDIVbits.DOZEN = DISABLE_DOZE;
        delay  = 0;
        // consumes about ? ma
        __builtin_disi(0x3FFF);                 // disable interrupts
        __builtin_write_OSCCONH(OSCCON | 0x05); // 101 = Low-Power RC Oscillator (LPRC)
        __builtin_write_OSCCONL(OSCCON | 0x01); // LPRC = 32 Khz
        __builtin_disi(0x0000);                 // enable interrupts

        
        while (delay++ < 3000) {
            if (OSCCONbits.OSWEN == 0)
                    break;     
        }
        numberOfFlashes = 20;
//        if(delay > 2000)
//        delay  = 0;   // breaks here
        
        while (numberOfFlashes != 0) {
            
            if (delay++ > 600) {
                ClrWdt();
                delay = 0;
                status_1_pin ^= 1;
                numberOfFlashes--;
            }
        }
        
        
        status_1_pin = 1;                     // solid on to indicate idle 
        Idle();                               // wdt or center button will wake
        
        status_1_pin = 0;
        // consumes 1.3 ma, battery gas gauge is on.
        Sleep();                             // wdt or center button will wake
    }
}












///*********************************
// * Bitmap Structure
// * Label: iconMicrochip
// * Description:  16x16 pixels, 1-bits per pixel
// ***********************************/
//#if defined (__C30__) && (__C30_VERSION__ <= 325)
//asm(".section .const, psv");
//#else
//asm(".section .const, psv, page");
//#endif
//
//
//extern const GFX_RESOURCE_HDR __iconMicrochip[] __attribute__((space(prog), aligned(2)));
//
//
//
//        
//        const  GFX_RESOURCE_HDR iconMicrochip =
//{
//    (GFX_RESOURCE_MEMORY_FLASH | GFX_RESOURCE_TYPE_MCHP_MBITMAP | GFX_RESOURCE_COMP_NONE),
//    ( GFX_RESOURCE_HDR )__iconMicrochip
//};
//        
//        
////const IMAGE_FLASH iconMicrochip =
////{
////    (FLASH | IMAGE_MBITMAP | COMP_NONE),
////    (FLASH_BYTE *)__iconMicrochip
////};
//asm(".section .text, code");
//asm(".global ___iconMicrochip");
//asm(".align 2");
//asm("___iconMicrochip:");
///****************************************
// * Bitmap header
// ****************************************/
//asm(".byte 0x00");           // Compresssion
//asm(".byte 0x01");           // Color Depth
//asm(".byte 0x10, 0x00");     // Height
//asm(".byte 0x10, 0x00");     // Width
///***********************************
// * Color Palette for the image
// **********************************/
//asm(".byte 0x00, 0x00");
//asm(".byte 0xFF, 0xFF");
///********************************
// * Bitmap Image Body
// *********************************/
//asm(".byte 0xC0, 0x03, 0xF8, 0x0F, 0xFC, 0x1F, 0xFE, 0x1F, 0xDE, 0x3B, 0x8F, 0x31, 0x8F, 0x63, 0x1B, 0xE3");
//asm(".byte 0x1B, 0xC6, 0x31, 0x86, 0x70, 0x0C, 0x78, 0x1E, 0xFC, 0x1F, 0xFC, 0x3F, 0xF8, 0x1F, 0xC0, 0x03");
//#if defined (__C30__) && (__C30_VERSION__ <= 325)
//asm(".section .const, psv");
//#else
//asm(".section .const, psv, page");
//#endif






// ****************************************************************************
////////////////////   to be deleted  /////////////////////////////////////////
// ****************************************************************************





/* end of file*/