
/******************************************************************************

 * File Name:       stripChart.c
 * Dependencies:    None
 * Processor:       PIC24FJ256GB106
 * Compiler:        XC16 1.20
 * Company:         Electronic Design Solutions LLC.



Author          Date       Comments
--------------------------------------------------------------------------------
KO          14-Feb-2008    First release
drw
 *******************************************************************************/


#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "stripChart.h"
#include "Utilities.h"
#include "Pervasive_Displays_small_EPD/EPD_interface.h"
//******************************************************************************
//******************************************************************************
// Constants
//******************************************************************************
//******************************************************************************


//******************************************************************************
//******************************************************************************
// Local Function Prototypes
//******************************************************************************
//******************************************************************************

void DrawGridHorizontals(void);
void DrawGridVerticals(void);


//******************************************************************************
//******************************************************************************
// Global Variables
//******************************************************************************
//******************************************************************************
extern APP_SCREEN_UPDATE_STATES update_display;
extern GFX_GOL_OBJ_SCHEME pScheme_button;

volatile int myRSSI;
BYTE graphData[GRAPH_DATA_BUFFER_SIZE];

WORD graphDataHead;
WORD graphDataTail;
BYTE graphOffset;
WORD graphUpdateRate;

/****************************************************************************
  Function:
    void DrawGraphLine( void )

  Description:
    This data draws the line graph of the current graph data.  The routine
    uses the current line type and color, so it can be used to erase a
    line by first calling GFX_ColorSet( BLACK ).

  Precondition:
    The line color must be set prior to calling this routine.  The graph
    variables graphData, graphDataHead, and graphDataTail, must also be
    valid.

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void DrawGraphLine(void) {
    static SHORT previousY;
    WORD tempHead;
    SHORT x;
    SHORT y;

    tempHead = graphDataHead;

    for (x = 0; (x < GRAPH_GRID_WIDTH) && (tempHead != graphDataTail); x++) {
        if (x == 0) {
            GFX_PixelPut(GRAPH_GRID_LEFT + x, graphData[tempHead]);
        } else {
            if (graphData[tempHead] > previousY) {
                for (y = previousY + GRAPH_GRID_TOP; y <= graphData[tempHead] + GRAPH_GRID_TOP; y++)
                    GFX_PixelPut(GRAPH_GRID_LEFT + x, y);
            } else {
                for (y = previousY + GRAPH_GRID_TOP; y >= graphData[tempHead] + GRAPH_GRID_TOP; y--)
                    GFX_PixelPut(GRAPH_GRID_LEFT + x, y);
            }
        }

        previousY = graphData[tempHead];
        tempHead++;
        if (tempHead == GRAPH_GRID_WIDTH + 1) {
            tempHead = 0;
        }
    }
}

/****************************************************************************
  Function:
    void DrawGridHorizontals( void )

  Description:
    This routine draws the horizontal lines on the scrolling graph.  The
    routine uses the current line color, so it can be used to erase the
    current horizontal lines by first calling GFX_ColorSet( BLACK ).

  Precondition:
    The line color must be set prior to calling this routine.

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void DrawGridHorizontals(void) {
    SHORT y;

    GFX_LineStyleSet(GFX_LINE_STYLE_THIN_DOTTED);
    for (y = GRAPH_GRID_TOP + GRID_SPACING_VERTICAL; y < GRAPH_GRID_BOTTOM; y += GRID_SPACING_VERTICAL) {
        GFX_LineDraw(GRAPH_GRID_LEFT, y, GRAPH_GRID_RIGHT, y);
    }
}

/****************************************************************************
  Function:
    void DrawGridVerticals( void )

  Description:
    This routine draws the vertical lines on the scrolling graph.  Since the
    graph is scrolling, the verticals scroll to match.  The current scroll
    offset is given by the variable graphOffset.  The routine uses the
    current line color, so it can be used to erase the current vertical lines
    by first calling GFX_ColorSet( BLACK ).

  Precondition:
    The line color and scroll offset (graphOffset) must be set prior to
    calling this routine.

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void DrawGridVerticals(void) {
    SHORT x;

    GFX_LineStyleSet(GFX_LINE_STYLE_THIN_DOTTED);
    for (x = GRAPH_GRID_LEFT + GRID_SPACING_HORIZONTAL - graphOffset; x < GRAPH_GRID_RIGHT; x += GRID_SPACING_HORIZONTAL) {
        GFX_LineDraw(x, GRAPH_GRID_TOP, x, GRAPH_GRID_BOTTOM);
    }
}

/****************************************************************************
  Function:
    WORD ProcessMessageGraph( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                    GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for the potentiometer graphing.
    The right touchpad increases the graph update rate, the left touchpad
    decreases the graph update rate, and the up touchpad exits the demo.

  Precondition:
    Call ShowScreenGraph() prior to using this function to display the
    correct screen.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    None.
 ***************************************************************************/
bool ProcessMessageGraph(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD controlID;

    if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
        controlID = GFX_GOL_ObjectIDGet(pObj);

        switch (controlID) {
                // case ID_GRAPH_SLOWER:
            case ID_BUTTON_GENERIC_DEC:
                if (graphUpdateRate < (GRAPH_UPDATE_INTERVAL_MAX - GRAPH_UPDATE_INTERVAL_STEP)) {
                    graphUpdateRate += GRAPH_UPDATE_INTERVAL_STEP;
                } else
                    graphUpdateRate = GRAPH_UPDATE_INTERVAL_MAX;
                break;

                // case ID_GRAPH_FASTER:
            case ID_BUTTON_GENERIC_INC:
                if (graphUpdateRate > (GRAPH_UPDATE_INTERVAL_MIN + GRAPH_UPDATE_INTERVAL_STEP)) {
                    graphUpdateRate -= GRAPH_UPDATE_INTERVAL_STEP;
                } else
                    graphUpdateRate = GRAPH_UPDATE_INTERVAL_MIN;
                break;

            case ID_BUTTON_GENERIC_HOME:
                screenState = SCREEN_DISPLAY_MAIN;
                break;

            default:
                break;
        }
        //   return 1;  // display button press
    }

    return false;
}

/****************************************************************************
  Function:
    void ShowScreenGraph( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    the graphing demonstration.  It draws the initial grid, and initializes
    the variables needed for the demo.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced. Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenGraph(void) {
    strcpy(pageInformation.title, " RSSI ");
    InitializeScreen();

    strcpy(pageInformation.buttonTextBottomLeft, " OK"); // center key
 
    
     GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 78, 74, 92, 2,          // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE , // will be displayed after creation
            pageInformation.buttonBitmapTopLeft, // bitmap
            NULL,                       // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 

    GFX_GOL_PanelParameterSet(  GRAPH_PANEL_LEFT,
            GRAPH_PANEL_TOP,
            GRAPH_PANEL_RIGHT,
            GRAPH_PANEL_BOTTOM,
            0,
            WHITE,
            WHITE,
            BLACK,
            NULL,
            GFX_FILL_STYLE_NONE,
            GOL_EMBOSS_SIZE);


    GFX_GOL_PanelDraw();

    // Draw the initial grid.
    GFX_ColorSet(BLACK);
    GFX_LineStyleSet(GFX_LINE_STYLE_THIN_DOTTED);

    previousTick = tick;
    graphOffset = 0;
    graphDataHead = 0;
    graphDataTail = 0;
    graphUpdateRate = GRAPH_UPDATE_INTERVAL_INIT;

}

/****************************************************************************
  Function:
    void TranslateMessageGraph( GFX_GOL_MESSAGE *pMsg )

  Description:
    If the message is valid, this function translates the raw touchpad message
    pMsg->param1 to the proper message for the appropriate control.

  Precondition:
    ShowScreenGraph() must be the last screen display routine called before
    this function executes.

  Parameters:
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void TranslateMessageGraph(GFX_GOL_MESSAGE *pMsg) {
    if ((pMsg->uiEvent == EVENT_KEYSCAN) &&
            (pMsg->type == TYPE_KEYBOARD) &&
            (pMsg->param1 == ID_TOUCH_PAD)) {
        if (pMsg->param2 == SCAN_RIGHT_PRESSED) {
            pMsg->param1 = ID_BUTTON_GENERIC_INC;
            pMsg->param2 = SCAN_CR_PRESSED;
            power_mode(HIGH_POWER);
        } else if (pMsg->param2 == SCAN_LEFT_PRESSED) {
            pMsg->param1 = ID_BUTTON_GENERIC_DEC;
            pMsg->param2 = SCAN_CR_PRESSED;
            power_mode(HIGH_POWER);
        } else if (pMsg->param2 == SCAN_CR_PRESSED) {
            pMsg->param1 = ID_BUTTON_GENERIC_HOME;
            pMsg->param2 = SCAN_CR_PRESSED;
            power_mode(HIGH_POWER);
        }
    }
}


BYTE globalUpdateCounter;
#define ROLLLING_BUFFER_SIZE  8  // choose  values from 1-10
/****************************************************************************
  Function:
    void UpdateGraph( void )

  Description:
    This routine updates the display of the scrolling graph.  The update rate
    is controlled by the variable graphUpdateRate, which the user can
    increase or decrease.  First, the old graph lines and vertical are
    erased.  The graph data is then updated with the new potentiometer
    reading.  Then the graph line and the graph horizontal and vertical lines
    are redrawn.  The horizontal lines are not affected by the scrolling,
    but if we do not redraw them, there will gaps where the previous graph
    line crossed the horizontal lines.

  Precondition:
    The graph information (graphData, graphDataHead, and graphDataTail) must
    be valid.

  Parameters:
    None

  Returns:
    None

  Remarks:
    Since normally a graph's 0 point is on the bottom, but our (0,0) is the
    upper left corner, we need to flip the graph.  This way, it matches the
    graph produced by Excel.
 ***************************************************************************/
void UpdateGraph(void) {
    GFX_GOL_OBJ_HEADER *pObject;

    static BYTE buffer_data_counter;
    static BYTE rollingBufferIndex;
    BYTE i;
    static WORD rollingBuffer[10];
    WORD rollingBufferAVERAGE;

    // Remove the old graph and vertical grid lines
    GFX_ColorSet(WHITE);
    DrawGraphLine();
    //  DrawGridVerticals();


    // Get the latest RSSI reading.
    if (rollingBufferIndex++ >= ROLLLING_BUFFER_SIZE) {
        rollingBufferIndex = 0;
    }
    if (myRSSI < 0) {  // handle timeout condition
        for (i = 0; i < ROLLLING_BUFFER_SIZE; i++) {
            rollingBuffer[i] = 0;
        }
    } else {

        rollingBuffer[rollingBufferIndex] = myRSSI;
    }


    rollingBufferAVERAGE = 0;
    for (i = 0; i < ROLLLING_BUFFER_SIZE; i++) {
        rollingBufferAVERAGE +=  rollingBuffer[i];
    }

    rollingBufferAVERAGE /= ROLLLING_BUFFER_SIZE;
    //rollingBufferAVERAGE >>= 3;  // divide by 8

   // graphData[graphDataTail++] = GRAPH_GRID_HEIGHT - (rollingBufferAVERAGE * (GRAPH_GRID_HEIGHT) / 69);
    graphData[graphDataTail++] = GRAPH_GRID_HEIGHT - (rollingBufferAVERAGE * (GRAPH_GRID_HEIGHT) / NORMALIZE_RSSI);
    
    
    //graphData[graphDataTail++] = GRAPH_GRID_HEIGHT - (myRSSI * (GRAPH_GRID_HEIGHT) / 69);


    if (graphDataTail == GRAPH_DATA_BUFFER_SIZE) {
        graphDataTail = 0;
    }

    // When the graph is full, update the head pointer to wrap and the display offset
    if (graphDataHead == graphDataTail) {
        graphDataHead++;
        if (graphDataHead == GRAPH_DATA_BUFFER_SIZE) {
            graphDataHead = 0;
        }

        graphOffset++;
        if (graphOffset >= GRID_SPACING_HORIZONTAL) {
            graphOffset = 0;
        }
    }
    if (buffer_data_counter++ > 5) {
        buffer_data_counter = 0;
        // Draw the new graph
        GFX_ColorSet(BLACK);
        DrawGraphLine();

        if (globalUpdateCounter++ > 200 ) {
            globalUpdateCounter = 0;
            update_display = GLOBAL_UPDATE;
            //            pObject = GFX_GOL_ObjectFind(ID_BUTTON_GENERIC_HOME);
            //            GFX_GOL_ObjectHideDraw (pObject);
            

        } else {
            update_display = PARTIAL_UPDATE;
        }
    }

}

////////////////////////////// delete  ////////////////////////////////////////




