/*******************************************************************************
  System Specific Definitions

  Company:
    Microchip Technology Inc.

  File Name:
    system.h

  Summary:
    System level definitions for the specific Microchip Development Board used.

*******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
// DOM-IGNORE-END

#ifndef __SYSTEM_H
#define __SYSTEM_H

/*********************************************************************
* PIC Device Specific includes
*********************************************************************/

#include <stdlib.h>
#include <xc.h>
#include "system_config.h"    
#include "resources/font_resources_epd_demo.h"
#include "symbol.h"
#include "console.h"
#include "timer.h"
#include "delay.h"
#include "driver/spi/drv_spi.h"
#include "../MSPI.h"




typedef union {

    struct {
         char RIGHT_BTN : 1;
         char LEFT_BTN : 1;
         char CENTER_BTN : 1;
    };
    WORD val;
} TOUCHPADS;


typedef union
{

    uint8_t  v[4];
    uint16_t w[2];
    uint32_t Val;

}API_UINT32_UNION;

typedef union
{
    uint8_t  v[2];
    uint16_t Val;
}API_UINT16_UNION;

typedef enum{
    LOWEST_POWER = 0,
    LOW_POWER,
    MID_POWER,
    HIGH_POWER
}POWER_MODE;


/*********************************************************************
* Macro: #define	SYS_CLK_FrequencySystemGet()
*
* Overview: This macro returns the system clock frequency in Hertz.
*			* value is 8 MHz x 4 PLL for PIC24
*
********************************************************************/
#define SYS_CLK_FrequencySystemGet()    (32000000ul)

/*********************************************************************
* Macro: #define	SYS_CLK_FrequencyPeripheralGet()
*
* Overview: This macro returns the peripheral clock frequency
*			used in Hertz.
*			* value for PIC24 is <PRE>(SYS_CLK_FrequencySystemGet()/2) </PRE>
*
********************************************************************/
#define SYS_CLK_FrequencyPeripheralGet()    (SYS_CLK_FrequencySystemGet() / 2)

/*********************************************************************
* Macro: #define	SYS_CLK_FrequencyInstructionGet()
*
* Overview: This macro returns instruction clock frequency
*			used in Hertz.
*			* value for PIC24 is <PRE>(SYS_CLK_FrequencySystemGet()/2) </PRE>
*
********************************************************************/
#define SYS_CLK_FrequencyInstructionGet()   (SYS_CLK_FrequencySystemGet() / 2)

/**********************************************************************
 * __delay_ms define - FCY must be defined before including
 * libpic30.h
 **********************************************************************/
#define FCY             (SYS_CLK_FrequencyInstructionGet())

#define NORMALIZE_RSSI 255

#include <libpic30.h>

/*********************************************************************
* Board initialization
*********************************************************************/
void SYSTEM_InitializeBoard(void);

// *****************************************************************************
// Tick Init
// *****************************************************************************
//void SYSTEM_TickInit(void);
void TickInit(void);
//*********************************************************************


void ShowScreenGeneric(void);
void InitializeScreen( void );
bool power_mode(POWER_MODE speed);


unsigned int raw_display_temp_adc(void);
void initialize_temperature(void);
unsigned int end_a_d_conversion(void);
void start_a_d_conversion(void);

extern unsigned int GAUGE_I2C1BRG;
extern unsigned char reset_reason;  // this is determined from the rcon register
extern unsigned char TRAP_FAULT;    // this is set in the TRAP interrupt

extern volatile uint32_t tick, prevTick;     // tick counter,
extern DWORD tick_10ms;
extern unsigned int TIMER_PERIOD_10MS;

extern unsigned int PWM_OC1RS;        // 4,000,000/24 = 166khz
extern unsigned int PWM_OC1R;         // 24/12 = 50 percent duty cycle


extern unsigned int display_Temperature;

extern GFX_GOL_OBJ_SCHEME  VB12_Scheme;  
extern GFX_GOL_OBJ_SCHEME  VB10_Scheme;
extern GFX_GOL_OBJ_HEADER *pObject;

typedef enum
{
    ANSEL_DIGITAL = 0,
    ANSEL_ANALOG = 1
}ANSEL_BIT_STATE;



typedef enum
{
    HW_BUTTON_PRESS = 0,
    HW_BUTTON_RELEASE = 1
}HW_BUTTON_STATE;



#define ENABLE_DOZE 1
#define DISABLE_DOZE 0


#define UNINITIALIZED_W_REGISTER_RESET 8
#define POWER_ON_RESET                 7
#define BROWN_OUT_RESET                6
#define WATCH_DOG_TIME_OUT_RESET       5
#define RESET_INSTRUCTION              4
#define MCLR_RESET                     3
#define CONFIGURATION_MISMATCH         2
#define ILLEGAL_OPCODE_RESET           1
#define TRAP_CONFLICT_RESET            0






#endif




