/*******************************************************************************
  System Specific Initializations

  Company:
    Microchip Technology Inc.

  File Name:
    system.c

  Summary:
    System level initializations for the specific Microchip Development Board used.

 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END

// *****************************************************************************
// Section: Includes
// *****************************************************************************
#include <stdlib.h>
#include <system.h>
#include "driver/uart/drv_uart1.h"
#include "driver/gfx/drv_gfx_display.h"
#include "../Pervasive_Displays_small_EPD/EPD_hardware_driver.h"
#include <i2c.h>
#include "../Battery_utility.h"
#include "adc.h"
#include "../../Pervasive_Displays_small_EPD/drv_serial_ram_mchp23k256.h"
#include "EPD_interface.h"
#include "EPD_hardware_driver.h"
#include "driver/eeprom_spi/drv_nvm_eeprom_spi_25aa02e48.h"
#include "usb.h"



void determine_Reset(void);

// *****************************************************************************
// Configuration bits
// *****************************************************************************
// PIC24FJ256GB106 Configuration Bit Settings


#define PLL_96MHZ_OFF   0xFFFF
#define PLL_96MHZ_ON    0xF7FF

#define USE_AND_OR
////#define USE_FRC_32MHZ
////PS16384
//_CONFIG1(WDTPS_PS16384 & FWPSA_PR128 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & COE_OFF & JTAGEN_OFF & GCP_OFF);
//_CONFIG3(WPFP_WPFP511 & WPDIS_WPDIS & WPCFG_WPCFGEN & WPEND_WPENDMEM);
//#ifdef USE_FRC_32MHZ
////_CONFIG2(POSCMOD_NONE & DISUVREG_ON & IOL1WAY_ON & OSCIOFNC_ON & FCKSM_CSECME & FNOSC_FRC & PLL_96MHZ_ON & PLLDIV_DIV2 & IESO_ON);
//_CONFIG2(POSCMOD_NONE & DISUVREG_ON & IOL1WAY_ON & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_FRCPLL & PLL_96MHZ_ON & PLLDIV_DIV2 & IESO_OFF);
//#else
//#ifdef USE_FRC_8MHZ
//_CONFIG2(POSCMOD_NONE & DISUVREG_ON & IOL1WAY_ON & OSCIOFNC_ON & FCKSM_CSECME & FNOSC_FRC & PLL_96MHZ_ON & PLLDIV_DIV2 & IESO_ON);
//#else
//_CONFIG2(POSCMOD_XT & DISUVREG_ON & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSECME & FNOSC_PRI & PLL_96MHZ_ON & PLLDIV_DIV2 & IESO_ON);
////CONFIG2(POSCMOD_XT & DISUVREG_ON & IOL1WAY_ON & OSCIOFNC_ON & FCKSM_CSECME & FNOSC_PRI & PLL_96MHZ_ON & PLLDIV_DIV2 & IESO_ON);
//#endif
//#endif



// CONFIG3
#pragma config WPFP = WPFP511           // Write Protection Flash Page Segment Boundary (Highest Page (same as page 170))
#pragma config WPDIS = WPDIS            // Segment Write Protection Disable bit (Segmented code protection disabled)
#pragma config WPCFG = WPCFGEN          // Configuration Word Code Page Protection Select bit (Last page and Flash configuration words are code-protected)
#pragma config WPEND = WPENDMEM         // Segment Write Protection End Page Select bit (Write Protect from WPFP to the last page of memory)

// CONFIG2
#pragma config POSCMOD = XT             // Primary Oscillator Select (XT oscillator mode selected)
#pragma config DISUVREG = ON            // Internal USB 3.3V Regulator Disable bit (Regulator is enabled)
#pragma config IOL1WAY = OFF            // IOLOCK One-Way Set Enable bit (Unlimited Writes To RP Registers)
#pragma config OSCIOFNC = ON            // Primary Oscillator Output Function (OSCO functions as port I/O (RC15))
#pragma config FCKSM = CSECME           // Clock Switching and Monitor (Both Clock switching and Fail-safe Clock Monitor are enabled)
#pragma config FNOSC = PRI              // Oscillator Select (Primary oscillator (XT, HS, EC))
#pragma config PLL_96MHZ = ON           // 96MHz PLL Disable (Enabled)
#pragma config PLLDIV = DIV2            // USB 96 MHz PLL Prescaler Select bits (Oscillator input divided by 2 (8MHz input))
#pragma config IESO = ON                // Internal External Switch Over Mode (IESO mode (Two-speed start-up) enabled)

// CONFIG1
#pragma config WDTPS = PS16384          // Watchdog Timer Postscaler (1:16,384)
#pragma config FWPSA = PR128            // WDT Prescaler (Prescaler ratio of 1:128)
#pragma config WINDIS = OFF             // Watchdog Timer Window (Standard Watchdog Timer enabled,(Windowed-mode is disabled))
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog Timer is disabled)
#pragma config ICS = PGx1               // Comm Channel Select (Emulator functions are shared with PGEC1/PGED1)
#pragma config GWRP = OFF               // General Code Segment Write Protect (Writes to program memory are allowed)
#pragma config GCP = OFF                // General Code Segment Code Protect (Code protection is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG port is disabled)



// *****************************************************************************
// System Settings and Variables
// *****************************************************************************
volatile uint32_t tick = 0, prevTick; // tick counter,
extern TOUCHPADS buttons;

unsigned char reset_reason;
BYTE trap_reason;
BYTE TRAP_FAULT __attribute__((persistent));

extern GFX_GOL_OBJ_SCHEME pScheme_button;

unsigned int display_Temperature;

unsigned int TIMER_PERIOD_10MS;   // assigned a value dependent on FCY
extern bool USB_or_BLE_DEMO_or_RSSI_DEMO;


#define POR    0x0003             //BOR is also set after a Power-on Reset.
#define BOR    0x0002
#define WDTO   0x0010
#define SWR    0x0040
#define EXTR   0x0080
#define CM     0x0200
#define IOPUWR 0x0400
#define TRAPR  0x8000

#define MATHERR 0x0010
#define ADDRERR 0x0008
#define STKERR  0x0004
#define OSCFAIL 0x0002




/****************************************************************************
  Function:
    bool power_mode( POWER_MODE speed )

  Description:
    This function changes the oscillator and system clock to save power.  The
    Graphic functions require a faster cpu and peripheral clock to give a more 
    responsive user interface. Doze is enabled the system clock continues
    to operate from the same source and at the same speed. Peripheral modules 
    continue to be clocked at the same speed while the CPU clock speed is reduced.
  Precondition:
    external crystal needs to be populated

  Parameters:
    POWER_MODE
 *       -  LOW_POWER
 *       -  MID_POWER
 *       -  HIGH_POWER
 * 

  Returns:
   - TRUE if the clock switch is successful.

  Remarks:
 * momentarily run the clock at full speed when functions interact with the 
 * graphic library.  When all the graphic objects are drawn and and transfered 
 * from the video ram buffer the clock speed can be reduced to save power.  When
 * operating the usb functionality the clock should not be switched. 
 * switching the clock clears the watchdog timer

 *****************************************************************************/
bool power_mode(POWER_MODE speed){
    bool result = false;
    int delay = 0;
    switch(speed){
        case LOWEST_POWER:  // needs work
//            PR4 = 20;       // TIMER_PERIOD_10MS/1000;
//            CLKDIVbits.DOZEN = DISABLE_DOZE;
//            __builtin_disi(0x3FFF);                  // disable interrupts
//            __builtin_write_OSCCONH(OSCCON | 0x05);  // 101 = Low-Power RC Oscillator (LPRC)
//            __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 32k
//            __builtin_disi(0x0000);                  // enable interrupts
//            
//            while (delay++ < 30000) {
//                if (OSCCONbits.OSWEN == 0)
//                       result = true;
//                       break;
//                   }
//            break;
        
        case LOW_POWER:
            PR4 = TIMER_PERIOD_10MS/4;
            CLKDIVbits.DOZEN = ENABLE_DOZE;
             __builtin_disi(0x3FFF);                 // disable interrupts
            __builtin_write_OSCCONH(OSCCON | 0x02);  // 010 = Primary Oscillator XT
            __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 8 Mhz
            __builtin_disi(0x0000);                  // enable interrupts
            
            while (delay++ < 30000) {
                if (OSCCONbits.OSWEN == 0)
                       result = true;
                       break;
                   }
            break;
        case MID_POWER:
            PR4 = TIMER_PERIOD_10MS;
            CLKDIVbits.DOZEN = ENABLE_DOZE;
            __builtin_disi(0x3FFF);                  // disable interrupts
            __builtin_write_OSCCONH(OSCCON | 0x03);  // Primary Oscillator w/ PLL module 
            __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 32Mhz
            __builtin_disi(0x0000);                  // enable interrupts

            while (delay++ < 30000) {
                if (OSCCONbits.OSWEN == 0)
                    if (OSCCONbits.LOCK == 1) {
                      result = true;
                      break;
                    }
            } 
            break;
        case HIGH_POWER:
            PR4 = TIMER_PERIOD_10MS;
            CLKDIVbits.DOZEN = DISABLE_DOZE;
            __builtin_disi(0x3FFF);                  // disable interrupts
            __builtin_write_OSCCONH(OSCCON | 0x03);  // Primary Oscillator w/ PLL module 
            __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 32Mhz
            __builtin_disi(0x0000);                  // enable interrupts

            while (delay++ < 30000) {
                if (OSCCONbits.OSWEN == 0)
                    if (OSCCONbits.LOCK == 1) {
                      result = true;
                      break;
                    }
            } 
            break;            
          default:
            CLKDIVbits.DOZEN = DISABLE_DOZE;
            __builtin_disi(0x3FFF);                  // disable interrupts
            __builtin_write_OSCCONH(OSCCON | 0x03);  // Primary Oscillator w/ PLL module 
            __builtin_write_OSCCONL(OSCCON | 0x01);  // fosc = 32Mhz
            __builtin_disi(0x0000);                  // enable interrupts

            while (delay++ < 30000) {
                if (OSCCONbits.OSWEN == 0)
                    if (OSCCONbits.LOCK == 1) {
                      result = true;
                      break;
                    }
            } 
            break;           
    }
    return result;
}




/****************************************************************************
  Function:
    void determine_Reset( void )

  Description:
    This function reads the rcon register and determines the reset and determines
    the trap condition.  This information is displayed on power up and helps 
    identify system failures
  Precondition:
    error condition should not occur prior to calling this function

  Parameters:
  - none

  Returns:
   - none

  Remarks:
 * 
 *****************************************************************************/
void determine_Reset(void) {

    unsigned int rcon_register;
    unsigned int intcon1_register;

    rcon_register = RCON & 0b1100001111010011;
    RCON = 0x00;

    switch (rcon_register) {
        case POR:
            reset_reason = POWER_ON_RESET;
            break;
        case BOR:
            reset_reason = BROWN_OUT_RESET;
            break;
        case WDTO:
            reset_reason = WATCH_DOG_TIME_OUT_RESET;
            break;
        case SWR:
            reset_reason = RESET_INSTRUCTION;
            break;
        case EXTR:
            reset_reason = MCLR_RESET;
            break;
        case CM:
            reset_reason = CONFIGURATION_MISMATCH;
            break;
        case IOPUWR:
            reset_reason = ILLEGAL_OPCODE_RESET;
            break;
        case TRAPR:
            reset_reason = TRAP_CONFLICT_RESET;
            break;
        default:
            reset_reason = UNINITIALIZED_W_REGISTER_RESET;
            break;
    }


    intcon1_register = INTCON1 & 0b0000000000011110;
    INTCON1 = 0x00;
    switch (intcon1_register) {
        case MATHERR:
            trap_reason = 0;
            break;
        case ADDRERR:
            trap_reason = 1;
            break;
        case STKERR:
            trap_reason = 2;
            break;
        case OSCFAIL:
            trap_reason = 3;
            break;
        default:
            trap_reason = 4;
            break;
    }
}



// *****************************************************************************
/*  Function:
    void SYSTEM_InitializeBoard(void)

    Summary:
        This function initializes the system board modules and
        peripheral ports.

    Description:
        This function initializes the system board modules and
        peripheral ports that is specific to the PIC device
        used in this configuration.

 */
// *****************************************************************************

#define PPRE_1_1        0b11         // Primary prescale 1:1  
#define PPRE_1_4        0b10         // Primary prescale 4:1

#define SPRE_1_1        0b111        // Secondary prescale 1:1
#define SPRE_1_2        0b110        // Secondary prescale 2:1

#define CHANNEL_2       2            // spi channel 2
#define CHANNEL_3       3            // spi channel 3
#define CKE_1           1

void SYSTEM_InitializeBoard(void) {

    AD1PCFGL = portConfiguration;   // set analog channels
    sw_3_TRIS = INPUT_PIN;
    sw_3_weakpullup = 1;
    EXTERNAL_POWER_SENSE_TRIS = INPUT_PIN;

    IFS1bits.CNIF = 0;              // clear change notice interrupt flag
    changeNoticeInt_enable = 1;     // never turn this off

    DRV_EPD_ControlSignalsInitialize();
    DRV_EPD_PowerDisable();

    EXTERNAL_POWER_SENSE_PIN_cn_enable = 1;
    determine_Reset();

    status_1_pin = 0;               // turn off status led
    status_1_TRIS = OUTPUT_PIN;     // set status led as output

    CLKDIVbits.CPDIV = 0b00;        // pass 32mhz clock through to the mux
    USB_or_BLE_DEMO_or_RSSI_DEMO = false;


    //-----------------
    //  WIRELESS
    //-----------------
    IRQ1_INT_TRIS = INPUT_PIN;
    IRQ0_INT_TRIS = INPUT_PIN;    // IRQ0

    Data_nCS_TRIS = OUTPUT_PIN;
    Config_nCS_TRIS = OUTPUT_PIN;
    PHY_RESETn_TRIS = OUTPUT_PIN;
    Data_nCS = 1;
    Config_nCS = 1;

    //-----------------
    //  MEMORY
    //-----------------
    MCHP_23K256_CS_TRIS = OUTPUT_PIN;
    MCHP_23K256_CS_LAT  = 1;                  // cs is active low
    mac_ee = 1;                               // cs is active low
    mac_ee_TRIS = OUTPUT_PIN;

    DRV_EPD_DISPLAY_RESET_LAT = 0;
    DRV_EPD_DischargeReset();
    DRV_EPD_SPI_CS_LAT     = 0;               // active low but should set to 0
    DRV_EPD_BorderDisable();                  // since this part is off

    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF; // turn off 5 v regulator
    USB_POWER_CTL_EXT_TRIS = OUTPUT_PIN;
    
    
    const DRV_SPI_INIT_DATA SRAM_SPI_Init_Data = {
        CHANNEL_2,  PPRE_1_1,
        SPRE_1_1, CKE_1,
        SPI_BUS_MODE_1, SPI_TRANSFER_MODE_8BIT
    };


    //Max clock speed of 25AA02 is 5Mhz  Fcy 16Mhz
    const DRV_SPI_INIT_DATA ID_PROM_SPI_Init_Data = {
        CHANNEL_2, PPRE_1_4,
        SPRE_1_1, CKE_1,
        SPI_BUS_MODE_1, SPI_TRANSFER_MODE_8BIT
    };

    // SPI initialization for EPD
    // max speed of display is 10Mhz
    // choose spi clock speed to 8
    const DRV_SPI_INIT_DATA EPD_SPI_Init_Data = {
        CHANNEL_3, PPRE_1_1,
        SPRE_1_2, 1,
        SPI_BUS_MODE_1, SPI_TRANSFER_MODE_8BIT
    };



    //-----------------
    // Configure PPS Related Pins
    //-----------------
    // Unlock to config PPS
    __builtin_write_OSCCONL(OSCCON & 0XBF); // clear the bit 6 of OSCCONL

    // radio using interrupt
    RPINR0bits.INT1R = 2;        // Mapping IRQ1 to RD8(RP2) *  MRF89XA

    // radio using spi1
    RPINR20bits.SDI1R = 29;       // Mapping SDI1 to(RP29)    
    RPOR1bits.RP3R = 8;           // Mapping (RP3) to SCK1(8)    
    RPOR6bits.RP12R = 7;          // Mapping (RP12) to SDO1(7)   

    // memory using spi2
    RPINR22bits.SDI2R = 14;       // Mapping SDI2 to RB14(RP14)
    RPOR4bits.RP9R = 11;          // Mapping (RP9) to SCK2(11)
    RPOR4bits.RP8R = 10;          // Mapping (RP8) to SDO2(10)

    // display using spi3
    RPINR28bits.SDI3R = 25;       // Mapping SDI3 to (RP25)
    RPOR10bits.RP20R = 33;        // Mapping (RP20) to SCK3(33)
    RPOR12bits.RP24R = 32;        // Mapping (RP24) to SDO3(32)

    // bluetooth async
    RPINR18bits.U1RXR = 27;       // mapping uart1 RX to RP27
    RPOR9bits.RP19R = 3;         // mapping (RP19) to uart1 TX()
    RPOR9bits.RP18R = 4;         // mapping (RP18) to uart1 RTS()
    RPINR18bits.U1CTSR = 17;       // mapping uart1 CTS to RP17
    
    __builtin_write_OSCCONL(OSCCON | 0x40); //set the bit 6 of OSCCONL to
    //lock Pin Re-map

    // initialize SRAM spi driver
    DRV_RAM_23K256_Initialize((DRV_SPI_INIT_DATA*) & SRAM_SPI_Init_Data);

    // initialize UID spi driver
    DRV_NVM_MCHP25AA02E48_Initialize((DRV_SPI_INIT_DATA*) & ID_PROM_SPI_Init_Data);

    //-----------------
    //  DISPLAY
    //-----------------
    // initialize EPD driver
    DRV_EPD_SPI_Initialize((DRV_SPI_INIT_DATA*) & EPD_SPI_Init_Data);

    // Set the callback for temperature reading (required by the EPD driver)
    DRV_EPD_TemperatureGetCallbackSet((DRV_EPD_TEMPERATURE_GET_FUNC) & raw_display_temp_adc);
    EPD_display_init();           // Initialize e-paper (EPD) gpio
    DRV_GFX_Initialize();         // fill memory with FF and set image address
    GFX_Initialize();

    SPI1CON1 = SPI1CON1_500KHZ;   // SLOWED DOWN TO 500KHZ
    SPI1STAT = 0x8000;            // radio can run to 1mhz

    INTCON2bits.INT1EP = 0;       // Interrupt on positive edge
    INTCON2bits.INT0EP = 0;       // Interrupt on positive edge

#if defined(ENABLE_TIME_SYNC)
    __builtin_write_OSCCONL(0x46);
    __builtin_write_OSCCONL(0x57);
    OSCCONbits.SOSCEN = 1;
    T1CON = 0x8012;
    PR1 = 0xFFFF;
    IFS0bits.T1IF = 0;
    IEC0bits.T1IE = 1;
#endif

    PHY_IRQ1 = 0;                 // clear interrupt flags
    PHY_IRQ0 = 0;
    PHY_IRQ1_En = 1;              // enable interrupts
    PHY_IRQ0_En = 1;

    //-----------------
    //  A/D CONVERTER
    //-----------------
    initialize_temperature();


    //-----------------
    //  BLUETOOTH
    //-----------------

    //    RN4020       PIC
    //    rts -------> cts rp17  RN4020 is ready for more data
    //    cts <------- rts rp18  PIC is ready for more data when port is low
    //    tx  -------> rx  rp27
    //    rx  <------- tx  rp19



    //UxRTS pin is asserted (driven low) whenever the receiver is ready to receive data
    BLUETOOTH_RTS_TRIS = OUTPUT_PIN;
    BLUETOOTH_RTS_LAT = 0;

    BLUETOOTH_WAKESW_TRIS = OUTPUT_PIN;
    BLUETOOTH_WAKESW_LAT = 0;

    BLUETOOTH_CTS_TRIS = INPUT_PIN; // RTS from module is driven low to request data
    BLUETOOTH_CTS_LAT = 0;

    BLUETOOTH_WAKEHW_TRIS = OUTPUT_PIN;
    BLUETOOTH_WAKEHW_LAT = 0;

    BLUETOOTH_CMD_MLDP_TRIS = OUTPUT_PIN;
    BLUETOOTH_CMD_MLDP_LAT = 0;

    BLUETOOTH_TX_TRIS = OUTPUT_PIN;
    BLUETOOTH_TX_LAT = 0;

    BLUETOOTH_RX_TRIS = INPUT_PIN;
    BLUETOOTH_EVENT_TRIS = INPUT_PIN;
    
    BLUETOOTH_CONNECT_TRIS = INPUT_PIN;
    
    DRV_UART1_InitializerDefault();

    //-----------------
    //  USB
    //-----------------
    EXTERNAL_POWER_SENSE_TRIS = INPUT_PIN; // external power detection 
    EXTERNAL_POWER_SENSE_PIN_cn_enable = 0;
    USB_D_PLUS_LAT = 0;
    USB_D_PLUS_TRIS = OUTPUT_PIN;

    USB_D_MINUS_LAT = 0;
    USB_D_MINUS_TRIS = OUTPUT_PIN;

    USB_ID_LAT = 0;
    USB_ID_TRIS = OUTPUT_PIN;

    //-----------------
    //  POWER
    //-----------------
    chrg_TRIS = INPUT_PIN;           // battery charger status
    chrg_input_weakPullUp = 1;       // enable weak pullup

    GasGauge_alert_TRIS = INPUT_PIN; // battery charge out of alert threshold
    GasGauge_weakPullUp = 1;         // open collector device needs pullup
    GasGauge_cn_enable = 0;          // firmware to poll the bit

    CloseI2C3();			   		 //Disable I2C3 module if enabled previously
    //OpenI2C3(I2C_ON | I2C_7BIT_ADD, GAUGE_I2C1BRG);
    OpenI2C3(0x8000, GAUGE_I2C1BRG);
    setControlRegister();            // set up coulomb counting IC
    configure_gasGauge_threshold();
    clearGasGaugeAlert();            // start off with no alerts.
    EXTERNAL_POWER_SENSE_TRIS = INPUT_PIN; // sense for external 5v power

    //-----------------
    //  USER INTERFACE
    //-----------------
    sw_1_TRIS = INPUT_PIN;
    sw_2_TRIS = INPUT_PIN;
    sw_3_TRIS = INPUT_PIN;

    sw_1_cn_enable = 0;           // change notice not enabled
    sw_2_cn_enable = 0;
    sw_3_cn_enable = 1;           // wake from sleep

    sw_1_weakpullup = 1;
    sw_2_weakpullup = 1;
    sw_3_weakpullup = 1;

    status_1_TRIS = OUTPUT_PIN;   // set status led as output
    status_1_pin = 0;             // turn off status led

    //-----------------
    //  EXTERNAL TEST POINTS
    //-----------------


    //-----------------
    //  SYSTEM CLOCK
    //-----------------
    TickInit();                   // Initialize the reference timer.
    // power conservation
    CLKDIVbits.DOZE = 0b111;      // 1:128;
    CLKDIVbits.DOZEN = DISABLE_DOZE;
    CLKDIVbits.CPDIV = 0;
    //1 = Interrupts clear the DOZEN bit and reset the CPU peripheral clock ratio to 1:1
    //0 = Interrupts have no effect on the DOZEN bit
    CLKDIVbits.ROI = 1;           

    //-----------------
    //  INTERRUPTS
    //-----------------
    RCON = 0;                     // clear power on reset bit
    RCONbits.SWDTEN = 1;          // software watchdog enabled
    RCONbits.PMSLP = 1;           // PMSLP: Program Memory Power During Sleep bit
                                  // 1 = Program memory bias voltage remains 
                                  //  powered during Sleep.
                                  // 0 = Program memory bias voltage is powered 
                                  // down during Sleep and voltage regulator 
                                  // enters Standby mode.
    INTCON1bits.NSTDIS = 0b1;     // 0 enable nesting interrupts ; 1 disable
    IPC6bits.T4IP = 0b101;        // raise timer 4 interrupt level by one
}

/****************************************************************************
  Function:
    void InitializeScreen( void )

  Description:
    This function erases the current list of graphics objects, clears the
    screen, puts the Microchip icon in the upper left corner of the screen,
    and displays a title at the top of the screen.  The title is obtained
    from the global pageInformation structure.

  Precondition:
    GOLInit() must be called prior to calling this function.

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void InitializeScreen(void) {
    GFX_GOL_ObjectListFree(); // free up linked list of objects
    GFX_ColorSet(WHITE);
    GFX_ScreenClear();

    //   // GFX_GOL_PictureControlCreate
    //    GFX_GOL_PictureControlCreate(ID_ICON, // ID
    //            0, 0, 15, 15, // dimension
    //            GFX_GOL_PICTURECONTROL_DRAW_STATE, // will be displayed, has frame
    //            1, // scale factor is x1
    //          (GFX_RESOURCE_HDR *)  iconMicrochip, // bitmap
    //            NULL); // default GOL scheme);


    //    // TODO remove microchip icon
    //    PictCreate(ID_ICON, // ID
    //            0, 0, 15, 15, // dimension
    //            PICT_DRAW, // will be dislayed, has frame
    //            1, // scale factor is x1
    //            &iconMicrochip, // bitmap
    //            NULL); // default GOL scheme
    //

    GFX_GOL_StaticTextCreate(ID_TITLE,
            16, 0, GFX_MaxXGet() - 11, GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_12) + 1,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.title,
            GFX_ALIGN_CENTER,
            &VB12_Scheme);

    previousTick = tick;
}



//******************************************************************
//* Temperature sensor  Configuration
//******************************************************************
static float IntDegC;
unsigned int raw_display_adc = 256;  // 25 deg C
#define Voltag_base     (float)((3.0*100)/1024)
#define Deg_0_voltag    (float)(500/10)    //500mv/10



void start_a_d_conversion(void) {
    EnableADC1;
    AD1CON1bits.SAMP = 1;
}

unsigned int end_a_d_conversion(void) {
    unsigned int adcval;
    ConvertADC10();
    while (BusyADC10());
    adcval = ReadADC10(0);
    raw_display_adc = adcval;
    //display_Temperature = adcval;
    IntDegC = (adcval * Voltag_base) - Deg_0_voltag;
    display_Temperature = IntDegC;
    DisableADC1;
    return (unsigned int) IntDegC;
}


// .  The temp sensor
// is powered at the same time as the display.  start the adc at the beginnig
// of the display update.  Just before powering down the display, the ad conversion
// should be finished.  this will eliminate the blocking code inside of the get_temperature
// routine.  

unsigned int raw_display_temp_adc(void) {
    // 
    return ((unsigned int) raw_display_adc);
}

/**
 * \brief Initialize the temperature sensor
 */
void initialize_temperature(void) {

    // todo some reason the #define use_and_or is not recognized
    unsigned int uiConfigADC_1, uiConfigADC_2, uiConfigADC_3, ad1pcfgh_low, ad1pcfgh_high, AD1CSSLow, AD1CSShigh;
    //    uiConfigADC_1 = ADC_MODULE_OFF | ADC_FORMAT_INTG | ADC_CLK_MANUAL |  
    //                     ADC_AUTO_SAMPLING_OFF | ADC_SAMP_OFF;
    uiConfigADC_1 = 0x8000;
    //    uiConfigADC_2 = ADC_VREF_AVDD_AVSS | ADC_SCAN_OFF | ADC_ALT_INPUT_OFF;
    uiConfigADC_2 = 0x8000;
    //    uiConfigADC_3 = ADC_CONV_CLK_INTERNAL_RC | ADC_CONV_CLK_16Tcy; //ADC_CONV_CLK_INTERNAL_RC
    uiConfigADC_3 = 0x800F;

    ad1pcfgh_low = portConfiguration;    // an7 set at analog
    ad1pcfgh_high = 0xFFFF;

    AD1CSSLow = 0x0080; //scan select register
    AD1CSShigh = 0x0000;

    OpenADC10(uiConfigADC_1, uiConfigADC_2, uiConfigADC_3, ad1pcfgh_low, ad1pcfgh_high,  \
              AD1CSSLow, AD1CSShigh);

    SetChanADC10(ADC_CH0_POS_SAMPLEA_AN7);

}


/****************************************************************************
  Function:
    void ShowScreenGeneric( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen of a generic format.  The screen consists of:
 * the Microchip icon
 * a one line title
 * four buttons
 * one line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenGeneric(void) {
    InitializeScreen(); // set up logo and title text, clears link list of
    // screen objects

    GFX_GOL_ButtonCreate(ID_TOP_LEFT, // ID
            1, 16, 62, 30, 2, // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE | GFX_GOL_BUTTON_PRESSED_STATE, // will be displayed after creation
            pageInformation.buttonBitmapTopLeft, // bitmap
            NULL, // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextTopLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //

    GFX_GOL_ButtonCreate(ID_BOTTOM_LEFT, // ID
            1, 32, 62, 46, 2, // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE, // will be displayed after creation
            pageInformation.buttonBitmapBottomLeft, // bitmap
            NULL, // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 

    GFX_GOL_ButtonCreate(ID_TOP_RIGHT, // ID
            64, 16, 125, 30, 2, // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE, // will be displayed after creation
            pageInformation.buttonBitmapTopRight, // bitmap
            NULL, // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextTopRight, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 

    GFX_GOL_ButtonCreate(ID_BOTTOM_RIGHT, // ID
            64, 32, 125, 46, 2, // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE, // will be displayed after creation
            pageInformation.buttonBitmapBottomRight, // bitmap
            NULL, // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomRight, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); // 

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_12) + 1,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_LEFT,
            &VB12_Scheme);

}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    interrupt service routines
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/////////////////////////////////// TIMER 4 SETUP //////////////////////////////

/****************************************************************************
  Function:
    void TickInit( void )

  Description:
    This function sets up Timer 4 to generate an interrupt every 10 ms.
   Fosc = 32,000,000
 * Tcy  = 16,000,000
 * prescale is 8 therefore counter gets incremented 2,000,000 times per second
 * timer period is 20,000; timer interrupt occurs every 20,000/2,000,000
 * = 1/100 seconds or 10 ms.
  Precondition:
     call switchXTAL_32 to properly set the timer period

  Parameters:
    None

  Returns:
    None

  Remarks:
    The timer period and prescaler values must be defined,
    since they are dependent on the oscillator speed.
 ***************************************************************************/

#define STOP_TIMER_IN_IDLE_MODE     0x2000
#define TIMER_SOURCE_INTERNAL       0x0000
#define TIMER_ON                    0x8000
#define GATED_TIME_DISABLED         0x0000
#define TIMER_16BIT_MODE            0x0000
#define TIMER_PRESCALER_1           0x0000
#define TIMER_PRESCALER_8           0x0010
#define TIMER_PRESCALER_64          0x0020
#define TIMER_PRESCALER_256         0x0030
#define TIMER_INTERRUPT_PRIORITY    0x1000

void TickInit(void) {
    TMR4 = 0;
    PR4 = TIMER_PERIOD_10MS;
    T4CON = TIMER_ON | STOP_TIMER_IN_IDLE_MODE | TIMER_SOURCE_INTERNAL |
            GATED_TIME_DISABLED | TIMER_16BIT_MODE | TIMER_PRESCALER;

    IFS1bits.T4IF = 0; //Clear flag
    IEC1bits.T4IE = 1; //Enable interrupt
    T4CONbits.TON = 1; //Run timer
}

/****************************************************************************
  Function:
    void __attribute__((interrupt, shadow, auto_psv)) _T4Interrupt(void)

  Description:
    This function updates the tick count and  monitor the
    touchpads.

  Precondition:
    Timer 4 and the Timer 4 interrupt must be enabled in order for
                    this function to execute.  

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void __attribute__((interrupt, shadow, auto_psv)) _T4Interrupt(void) {

    static char sw1, sw2, sw3;
    static signed char debounce1, debounce2, debounce3;
    static DWORD offCount;

    IFS1bits.T4IF = 0;    // Clear flag

    tick++;


    if (sw_1_input != sw1) //right button
    {
        debounce1 = 3;
        sw1 = sw_1_input;
    }
    if (debounce1 != 0)
        debounce1--;
    else {
        buttons.RIGHT_BTN = !sw_1_input;
    }


    if (sw_2_input != sw2) // left button
    {
        debounce2 = 3;
        sw2 = sw_2_input;
    }
    if (debounce2 != 0)
        debounce2--;
    else {
        buttons.LEFT_BTN = !sw_2_input;
    }

    if (sw_3_input != sw3) // center button
    {
        debounce3 = 3;
        sw3 = sw_3_input;
    }
    if (debounce3 != 0)
        debounce3--;
    else {
        buttons.CENTER_BTN = !sw_3_input;
    }

    if (sw_3_input == 0) // check if button pressed
    {
        offCount++;
        if (offCount == 300) {
            offCount = 0; // reset off counter
            switch_duration = LONG_PRESS;
        }
    } else {
        offCount = 0;
    }
}

/*********************************************************************
 * Function:        void _CN_Interrupt_ISR(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None.
 *
 * Overview:        This is the interrupt handler for push button 1
 *
 *                  Push button 3 and External power is used to wake
 *                  up the controller from Sleep State.
 ********************************************************************/
void _ISRFAST __attribute__((interrupt, auto_psv)) _CNInterrupt(void) {
    IFS1bits.CNIF = 0;

//    if (sw_1_cn_enable == 1)
//        if (sw_1_input == 0) {
//            sw_1_cn_enable = 0;
//        }
}

void _ISRFAST __attribute__((interrupt, auto_psv)) _T1Interrupt(void) {
    IFS0bits.T1IF = 0;
}


/*  Function:
    GFX_STATUS GFX_ExternalResourceCallback(
                                GFX_RESOURCE_HDR *pResource,
                                uint32_t offset,
                                uint16_t nCount,
                                void     *pBuffer)

    Summary:
        This function performs data fetch from external memory.

    Description:
        This function must be implemented in the application.
        The library will call this function each time when
        the external memory data will be required. The application
        must copy requested byte quantity into the buffer provided.
        Data start address in external memory is a sum of the address
        in GFX_RESOURCE_HDR structure and offset.

    Precondition:
        None.

    Parameters:
        pResource - Pointer to the external memory resource information.
        offset - offset of the data from the location of the resource
                 in external memory.
        nCount - Number of bytes to be transferred into the buffer.
        buffer - Pointer to the buffer that will hold the retrieved data.

    Returns:
        GFX_STATUS_SUCCESS when all the data was succesfully retrieved.
        GFX_STATUS_FAILURE when partial or no data was retrieved.

    Example:
        None.

 */
// *****************************************************************************
// If there are several memories in the system they can be selected by IDs.
// In this demo ID for memory chip installed on Graphics PICTail board is assumed to be 0.
#define SST39_MEMORY    0

GFX_STATUS GFX_ExternalResourceCallback(
        GFX_RESOURCE_HDR *pResource,
        uint32_t offset,
        uint16_t nCount,
        void *pBuffer) {
    uint32_t addr;

    // get the proper address
    switch (pResource->type) {
        case GFX_RESOURCE_FONT_EXTERNAL_NONE:
            addr = pResource->resource.font.location.extAddress;
            break;
        case GFX_RESOURCE_MCHP_MBITMAP_EXTERNAL_RLE:
        case GFX_RESOURCE_MCHP_MBITMAP_EXTERNAL_NONE:
            addr = pResource->resource.image.location.extAddress;
            break;
        default:
            // type is incorrect
            return (GFX_STATUS_FAILURE);
    }
    addr += offset;

    if (pResource->ID == SST39_MEMORY) {
        // Read data requested into buffer provided
        //DRV_NVM_SST25VF016_Read(addr, (uint8_t*) pBuffer, nCount);
    }

    return (GFX_STATUS_SUCCESS);
}

/* Interrupt handler for USB host. */
void __attribute__((interrupt, auto_psv)) _USB1Interrupt() {
#ifdef USB_HOST
    USB_HostInterruptHandler();
#else
    USBDeviceTasks();
#endif
}

/*******************************************************************************
Function:       void __attribute__((__interrupt__, auto_psv)) _XXX(void)

These are just here to catch any spurious interrupts that we see during
debugging.

 *******************************************************************************/
void __attribute__((interrupt, auto_psv)) _DefaultInterrupt(void) {
    RCONbits.SWDTEN = 1;   // make sure wdt in on
    while (1) {
        Nop();
        Nop();
        Nop();
        status_1_pin = 1;
        TRAP_FAULT = 4;
    }
}

void __attribute__((interrupt, auto_psv)) _OscillatorFail(void) {
    int delay = 0;
    RCONbits.SWDTEN = 1;  // make sure wdt in on
    OSCCONbits.OSWEN = 0;
    OSCCONbits.NOSC = 0b000;
    __builtin_write_OSCCONH(OSCCON | 0x00);
    __builtin_write_OSCCONL(OSCCON | 0x00);
    OSCCONbits.OSWEN = 1;
    delay = 0;
    while (OSCCONbits.OSWEN == 1)
        if (delay++ > 50000) {
            delay = 0;
            status_1_pin ^= 1;
        }

    status_1_pin = 1;
    TRAP_FAULT = 3;
    //    INTCON1bits.OSCFAIL = 0;
}

void __attribute__((interrupt, auto_psv)) _AddressError(void) {
    unsigned int delay  = 0;
    RCONbits.SWDTEN = 1;   //// make sure wdt in on
    while (1) {
        if (delay++ > 50000) {
            delay = 0;
            status_1_pin ^= 1;
        }
        Nop();
        Nop();
        Nop();
        TRAP_FAULT = 1;
    }
}

void __attribute__((interrupt, auto_psv)) _StackError(void) {
    unsigned int delay = 0;
    RCONbits.SWDTEN = 1;   // make sure wdt in on
    while (1) {
        if (delay++ > 40000) {
            delay = 0;
            status_1_pin ^= 1;
        }
        Nop();
        Nop();
        Nop();
        TRAP_FAULT = 2;
    }
}

void __attribute__((interrupt, auto_psv)) _MathError(void) {
    unsigned int delay = 0;
    RCONbits.SWDTEN = 1;    // make sure wdt in on
    while (1) {
        if (delay++ > 50000) {
            delay = 0;
            status_1_pin ^= 1;
        }
        Nop();
        Nop();
        Nop();
        TRAP_FAULT = 0;
    }
}

//////////// delete ///////////////////////////////////////////////////////////



