/*******************************************************************************
  System Specific Definitions

  Company:
    Electronic Design Solutions LLC.

  File Name:
    System_config.h

  Summary:
    System level definitions for the wireless epaper development board used.

 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************


You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END

#ifndef __SYSTEM_CONFIG_H
#define __SYSTEM_CONFIG_H


/*********************************************************************
 * PIC Device Specific includes
 *********************************************************************/
#include "miwi_config.h"          //MiWi Application layer configuration file
#include "miwi_config_mesh.h"     //MiWi Protocol layer configuration file
#include "config_89xa.h"
#include "gfx_config.h"
#include"../usb_config.h"
#include "../fileio_config.h"
#include "GenericTypeDefs.h"
#include "drv_uart_config.h"


#define MILLISECONDS_PER_TICK       10
#ifdef USE_FRC_32MHZ
#define GetSystemClock()            32000000UL
#define TIMER_PRESCALER             TIMER_PRESCALER_8 //TIMER_PRESCALER_1
#define TIMER_PERIOD_1MS            2000
#define SPI1CON1_500KHZ             0b0000000100111001
#else
#ifdef USE_FRC_8MHZ
#define GetSystemClock()            8000000UL
#define TIMER_PRESCALER             TIMER_PRESCALER_8
#define TIMER_PERIOD_1MS            500
#define SPI1CON1_500KHZ             0b0000000100111101
//#define SPI2CON1 running at 1 to 1 max speed is 8 Mhz  part will handle 16
#else
//#define GetSystemClock()            32000000UL
 #define TIMER_PRESCALER             TIMER_PRESCALER_8   // 8MHz: TIMER_PRESCALER_1
#define TIMER_PERIOD_1MS            2000
#define SPI1CON1_500KHZ             0b0000000100111001  //0x0139
//#define SPI2CON1 running at 1 to 1 max speed is 16 Mhz
#endif
#endif


#define BAUDRATE2       57600UL
#define BRG_DIV2        4
#define BRGH2           1

// Define the RTCC default initialization.
#define RTCC_DEFAULT_DAY        20        // Date
#define RTCC_DEFAULT_MONTH       3        // Month
#define RTCC_DEFAULT_YEAR       15        // Year
#define RTCC_DEFAULT_WEEKDAY     1        // Day
#define RTCC_DEFAULT_HOUR       10        // hour
#define RTCC_DEFAULT_MINUTE     00        // Minute
#define RTCC_DEFAULT_SECOND     00        // Second



/** TRIS ***********************************************************/
#define INPUT_PIN       1
#define OUTPUT_PIN      0

#define EPD_144         0
#define EPD_200         1
#define EPD_270         2
#define EPD_190         3
#define EPD_260         4

#define GFX_USE_DISPLAY_CONTROLLER_EPD


// -----------------------------------
// For E paper display
// -----------------------------------
#define DISP_ORIENTATION                0
#define DISP_HOR_RESOLUTION             128
#define DISP_VER_RESOLUTION             96



// -----------------------------------
// Default font to be used in the demo
// -----------------------------------
#define APP_DEMO_FONT Gentium27



// define the SPI channel used for EPD 
#define DRV_SPI_CONFIG_CHANNEL_3_ENABLE
// define the SPI channel used for Serial RAM and Unique ID 
#define DRV_SPI_CONFIG_CHANNEL_2_ENABLE
// define the SPI channel for RF communications
#define DRV_SPI_CONFIG_CHANNEL_1_ENABLE


#define TMRL TMR2

/*********************************************************************/

/*********************************************************************
 * HARDWARE PROFILE for UART to program external memory
 *********************************************************************/
//#define USE_COMM_PKT_MEDIA_SERIAL_PORT
//#define COMM_PKT_RX_MAX_SIZE    (1024)
//
//#define DRV_UART2_CONFIG_BAUD_RATE    115200ul 
//
//#define DRV_UART2_TX_BYTEQ_LENGTH 1
//#define DRV_UART2_RX_BYTEQ_LENGTH 1



#define portConfiguration              0xFF7F                  // analog 7

/////////////////////////////////////// MRF89XA pins ///////////////////////////
#define Data_nCS                       LATDbits.LATD6           // mrf_cs
#define Data_nCS_TRIS                  TRISDbits.TRISD6         //

#define Config_nCS                     LATFbits.LATF1           // mrf_wake
#define Config_nCS_TRIS                TRISFbits.TRISF1

#define PHY_RESETn                     LATDbits.LATD3           // mrf_reset
#define PHY_RESETn_TRIS                TRISDbits.TRISD3

#define IRQ0_INT_PIN                   PORTDbits.RD0            // mrf_int
#define IRQ0_INT_TRIS                  TRISDbits.TRISD0

#define PHY_IRQ0                       IFS0bits.INT0IF          // mrf_int
#define PHY_IRQ0_En                    IEC0bits.INT0IE

#define IRQ1_INT_PIN                   PORTDbits.RD8            // RP2 mrf_irq_1
#define IRQ1_INT_TRIS                  TRISDbits.TRISD8

#define PHY_IRQ1                       IFS1bits.INT1IF          // mrf_irq_1
#define PHY_IRQ1_En                    IEC1bits.INT1IE

#define SPI_SDI                        PORTBbits.RB15           // RP29  spi 1
#define SDI_TRIS                       TRISBbits.TRISB15
  
#define SPI_SDO                        LATDbits.LATD11          // RP12  spi 1
#define SDO_TRIS                       TRISDbits.TRISD11
  
#define SPI_SCK                        LATDbits.LATD10          // RP3   spi 1
#define SCK_TRIS                       TRISDbits.TRISD10
////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////// MikroElektronika pins //////////////////
#define MIKROBUS_INT_TRIS             TRISBbits.TRISB5         // RP18 RB5
#define MIKROBUS_INT_LAT              LATBbits.LATB5
#define MIKROBUS_INT                  PORTBbits.RB5

#define MIKROBUS_INT_weakpullDown     CNPD1bits.CN7PDE
#define MIKROBUS_INT_weakpullup       CNPU1bits.CN7PUE
#define MIKROBUS_INT_cn_enable        CNEN1bits.CN7IE

#define MIKROBUS_EXT_P5_TRIS          TRISEbits.TRISE3         // RE3
#define MIKROBUS_EXT_P5_LAT           LATEbits.LATE3
#define MIKROBUS_EXT_P5               PORTEbits.RE3

#define MIKROBUS_SCL2_TRIS            TRISFbits.TRISF5         // RP17 RF5
#define MIKROBUS_SCL2_LAT             LATFbits.LATF5

#define MIKROBUS_SDA2_TRIS            TRISFbits.TRISF4         // RF4
#define MIKROBUS_SDA2_LAT             LATFbits.LATF4


#define MIKROBUS_RST_TRIS             TRISEbits.TRISE4         // 
#define MIKROBUS_RST_LAT              LATEbits.LATE4

#define MIKROBUS_RX_TRIS              TRISGbits.TRISG8         // RP19 RG8
#define MIKROBUS_RX_LAT               LATGbits.LATG8

#define MIKROBUS_TX_TRIS              TRISGbits.TRISG9         // RP27 RG9
#define MIKROBUS_TX_LAT               LATGbits.LATG9

#define MIKROBUS_PWM_TRIS             TRISGbits.TRISG6        // RP21
#define MIKROBUS_PWM_LAT              LATGbits.LATG6

#define MIKROBUS_CS_TRIS             TRISGbits.TRISG7         // RG7
#define MIKROBUS_CS                  PORTGbits.RG7
#define MIKROBUS_CS_LAT              LATGbits.LATG7

#define MIKROBUS_AN_TRIS             TRISBbits.TRISB2
#define MIKROBUS_AN_LAT              LATBbits.LATB2

#define MKROE_power_off()  USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF
#define MKROE_power_on()  USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_ON




////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////// bluetooth pins /////////////////////////

#define BLUETOOTH_EVENT_TRIS           TRISGbits.TRISG6         // RG6
#define BLUETOOTH_EVENT                PORTGbits.RG6
#define BLUETOOTH_EVENT_LAT            LATGbits.LATG6
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////// memory pins ////////////////////////////
// Unique ID EE prom
#define mac_ee                          LATBbits.LATB12         // mac_ee
#define mac_ee_TRIS                     TRISBbits.TRISB12       // chip select
#define MCHP25AA02E48_CS_LAT            LATBbits.LATB12         // mac_ee

// EPD Serial RAM pins
#define MCHP_23K256_CS_TRIS             TRISBbits.TRISB13       // ee_memory
#define MCHP_23K256_CS_LAT              LATBbits.LATB13

#define MCHP_23K256_SCK_TRIS            TRISBbits.TRISB9        // RP9 memory_sck
#define MCHP_23K256_SDO_TRIS            TRISBbits.TRISB8        // RP8 memory_sdo
#define MCHP_23K256_SDI_TRIS            TRISBbits.TRISB14       // RP14 memory_sdi

#define MCHP_23K256_SCK_LAT             LATBbits.LATB9          //
#define MCHP_23K256_SDO_LAT             LATBbits.LATB8          //
#define MCHP_23K256_SDI_LAT             LATBbits.LATB14         //
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////// display pins ///////////////////////////
#define DRV_EPD_SPI_CS_TRIS             TRISFbits.TRISF0        // display_cs
#define DRV_EPD_SPI_CS_LAT              LATFbits.LATF0

#define DRV_EPD_SPI_SCK_TRIS            TRISDbits.TRISD5        // display_sclk
#define DRV_EPD_SPI_SDO_TRIS            TRISDbits.TRISD1        // display_si(mosi) 
#define DRV_EPD_SPI_SDI_TRIS            TRISDbits.TRISD4        // display_so(miso)

#define DRV_EPD_SPI_SCK_LAT             LATDbits.LATD5          //
#define DRV_EPD_SPI_SDO_LAT             LATDbits.LATD1          //
#define DRV_EPD_SPI_SDI_LAT             LATDbits.LATD4          //

// EPD busy pin
#define DRV_EPD_DISPLAY_BUSY_TRIS       TRISDbits.TRISD7        // busy
#define DRV_EPD_DISPLAY_BUSY_PORT       PORTDbits.RD7
// EPD reset pin
#define DRV_EPD_DISPLAY_RESET_TRIS      TRISDbits.TRISD2        // display_reset
#define DRV_EPD_DISPLAY_RESET_LAT       LATDbits.LATD2
// EPD PWM pin   not available in hardware rev 04
//#define DRV_EPD_DISPLAY_PWM_TRIS        TRISDbits.TRISD8        // display_pwm
//#define DRV_EPD_DISPLAY_PWM_LAT         LATDbits.LATD8  
// EPD panel on pin
#define DRV_EPD_DISPLAY_PANEL_ON_TRIS   TRISEbits.TRISE1        // panel_on
#define DRV_EPD_DISPLAY_PANEL_ON_LAT    LATEbits.LATE1
// EPD discharge pin
#define DRV_EPD_DISPLAY_DISCHARGE_TRIS  TRISDbits.TRISD9        // discharge
#define DRV_EPD_DISPLAY_DISCHARGE_LAT   LATDbits.LATD9 
// EPD border pin
#define DRV_EPD_DISPLAY_BORDER_TRIS                             // no border 
#define DRV_EPD_DISPLAY_BORDER_LAT                              // pins
// EPD temperature pin
#define DRV_EPD_DISPLAY_TEMP_TRIS       TRISBbits.TRISB7        // temperature
#define DRV_EPD_DISPLAY_TEMP_PORT       PORTBbits.RB7
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////// power pins /////////////////////////////
#define USB_POWER_CTL_EXT_TRIS          TRISEbits.TRISE0        // 5v usb and
#define USB_POWER_CTL_EXT_pin           LATEbits.LATE0          // bluetooth
#define USB_POWER_CTL_EXT_OFF           0                       // off
#define USB_POWER_CTL_EXT_ON            1                       // on

#define EXTERNAL_POWER_SENSE_TRIS       TRISBbits.TRISB4        //
#define EXTERNAL_POWER_SENSE_PIN        PORTBbits.RB4           //
#define EXTERNAL_POWER_SENSE_PIN_cn_enable CNEN1bits.CN6IE      // wake on power

#define chrg_TRIS                       TRISEbits.TRISE2        // battery charge
#define chrg_input                      PORTEbits.RE2
#define chrg_input_weakPullUp           CNPU4bits.CN60PUE

#define GasGauge_alert_TRIS             TRISEbits.TRISE5        // alert for
#define GasGauge_alert_input            PORTEbits.RE5           // over and 
#define GasGauge_weakPullUp             CNPU4bits.CN63PUE       // uncharge
#define GasGauge_cn_enable              CNEN4bits.CN63IE

#define GasGauge_SDA3                                           // I2C com
#define GasGauge_SCL3                                           // I2C com

#define Fsck	                        100000
// todo this has an extra divide by two  retest 
//#define I2C_BRG	                        (SYS_CLK_FrequencyPeripheralGet()/2/Fsck)-1    //((Fcy/2/Fsck)-1)
#define I2C_BRG	                        (SYS_CLK_FrequencyPeripheralGet()/Fsck)-1    //((Fcy/2/Fsck)-1)
#define GAS_GAUGE_ADDRESS               0b01100100
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////// user interface pins ////////////////////
#define status_1_TRIS                TRISBbits.TRISB10          // status
#define status_1_pin                 LATBbits.LATB10

#define sw_1_TRIS                    TRISBbits.TRISB11          // push button
#define sw_1_input                   PORTBbits.RB11             // top right
#define sw_1_weakpullup              CNPU2bits.CN29PUE
#define sw_1_cn_enable               CNEN2bits.CN29IE

#define sw_2_TRIS                    TRISBbits.TRISB6           // push button
#define sw_2_input                   PORTBbits.RB6              // top left
#define sw_2_weakpullup              CNPU2bits.CN24PUE
#define sw_2_cn_enable               CNEN2bits.CN24IE

#define sw_3_TRIS                    TRISBbits.TRISB3           // push button
#define sw_3_input                   PORTBbits.RB3              // center
#define sw_3_weakpullup              CNPU1bits.CN5PUE
#define sw_3_cn_enable               CNEN1bits.CN5IE
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////// USB pins ///////////////////////////////

#define USB_D_PLUS_LAT               LATGbits.LATG2              // usb D+
#define USB_D_PLUS_TRIS              TRISGbits.TRISG2            

#define USB_D_MINUS_LAT              LATGbits.LATG3              // usb D+
#define USB_D_MINUS_TRIS             TRISGbits.TRISG3       

#define USB_ID_LAT                   LATFbits.LATF3              // USBID
#define USB_ID_TRIS                  TRISFbits.TRISF3



////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////// External pins //////////////////////////


////////////////////////////////////////////////////////////////////////////////




#define changeNoticeInt_enable       IEC1bits.CNIE

//#define USE_EXTERNAL_EEPROM



#define LINE_SIZE_INSTRUCTION       40    // number of characters in display line
#define LINE_SIZE_TITLE             30
#define LINE_SIZE_BUTTON_TEXT       15
#define LOCATION_MIN_Y              16

#define XCHAR   char   

#define SHORT_PRESS                 0
#define LONG_PRESS                  1


// ID's of controls on the generic menu screen
#define ID_ICON                     200
#define ID_TITLE                    201
#define ID_TOP_LEFT                 202
#define ID_BOTTOM_LEFT              203
#define ID_TOP_RIGHT                204
#define ID_BOTTOM_RIGHT             205
#define ID_THIRDROW		            206
#define ID_INSTRUCTIONS             207
#define ID_INSTRUCTIONS_1           208
#define ID_INSTRUCTIONS_2           209
#define ID_INSTRUCTIONS_3           210
#define ID_INSTRUCTIONS_4           211
#define ID_HR                       216
#define ID_HR_VALUE                 217
#define ID_SEC_VALUE                218
#define ID_SEC                      219

#define ID_TOUCH_PAD                100
#define ID_TOUCH_BUTTON_01          101
#define ID_TOUCH_BUTTON_02          102
#define ID_TOUCH_BUTTON_03          103
#define ID_TOUCH_BUTTON_04          104
#define ID_TOUCH_BUTTON_05          105

//*****************************************************************************
//*****************************************************************************
// Version Information
//*****************************************************************************
//*****************************************************************************


#define HARDWARE_VERSION_STRING ".4"


// *****************************************************************************
// Demo States
// *****************************************************************************
typedef enum {
    NO_UPDATE = 0,
    GLOBAL_UPDATE,
    PARTIAL_UPDATE
} APP_SCREEN_UPDATE_STATES;

typedef enum {
    SCREEN_BOARD_TEST = 0,
    SCREEN_DISPLAY_RADIO_SETUP,
    SCREEN_DISPLAY_MAIN,
    SCREEN_MAIN,
    SCREEN_DISPLAY_UTILITIES,
    SCREEN_UTILITIES,
    SCREEN_DISPLAY_TIME,
    SCREEN_TIME,
    SCREEN_DISPLAY_ADJUST_MONTH,
    SCREEN_ADJUST_MONTH,
    SCREEN_DISPLAY_ADJUST_MINUTES,
    SCREEN_ADJUST_MINUTES,
    SCREEN_DISPLAY_ADJUST_DAY,
    SCREEN_ADJUST_DAY,
    SCREEN_DISPLAY_ADJUST_HOUR,
    SCREEN_ADJUST_HOUR,
    SCREEN_DISPLAY_MKROE,
    SCREEN_DISPLAY_BLE_LOOPBACK,
    SCREEN_MKROE,
    SCREEN_BLE_LOOPBACK,
    SCREEN_DISPLAY_USB_MEMORY,
    SCREEN_USB_MEMORY,
    SCREEN_DISPLAY_GRAPH,
    SCREEN_GRAPH,
    SCREEN_DISPLAY_BATTERY,
    SCREEN_BATTERY                      ,
    SCREEN_POWER_DOWN_BATTERY_LOW,
    SCREEN_DISPLAY_USER_POWER_DOWN,
    SCREEN_POWER_DOWN,
    SCREEN_EXTERNAL_POWER_APPLIED
} APP_SCREEN_STATES;








typedef struct {
    XCHAR title[LINE_SIZE_TITLE];

    XCHAR buttonTextTopLeft[LINE_SIZE_BUTTON_TEXT];
    XCHAR buttonTextBottomLeft[LINE_SIZE_BUTTON_TEXT];
    XCHAR buttonTextTopRight[LINE_SIZE_BUTTON_TEXT];
    XCHAR buttonTextBottomRight[LINE_SIZE_BUTTON_TEXT];
    XCHAR buttonTextThirdrow[LINE_SIZE_BUTTON_TEXT];

    void *buttonBitmapTopLeft;
    void *buttonBitmapBottomLeft;
    void *buttonBitmapTopRight;
    void *buttonBitmapBottomRight;
    void *buttonBitmapThirdrow;

    XCHAR instructions[LINE_SIZE_INSTRUCTION];
    XCHAR instructions1[LINE_SIZE_INSTRUCTION];
    XCHAR instructions2[LINE_SIZE_INSTRUCTION];
    XCHAR instructions3[LINE_SIZE_INSTRUCTION];
    XCHAR instructions4[LINE_SIZE_INSTRUCTION];

} PAGE_INFORMATION;


typedef enum {
    INITIALIZE = 0,
    SPLASH_SCREEN,
    DISPLAY_TEMPERATURE,
    EXIT_DEMO
} APP_SCREEN_IR_DEMO_STATES;


extern PAGE_INFORMATION pageInformation;
extern char switch_duration;       // determine long and short key press
extern DWORD previousTick;
extern APP_SCREEN_STATES   screenState;
extern APP_SCREEN_IR_DEMO_STATES CLICK_DEMO_STATE;


#endif

